/*----------------------------------------------------------------------------*/
// File: coast.c USGS Navigation, handle coastline files
// Blame: John Gann
// Start Date: 12 May 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "mnutils.h"
#include "nav_utils.h"
#include "sensor.h"
#include "coast.h"

// private
static t_coast coast;
static char coastLineLoaded = FALSE;

// externals
extern char RefreshRequired;
extern t_nav * curNav;      // pointer to the current navigation 


/*---------------------------------------------------------------------------*/
// Select the Active Sensor
/*---------------------------------------------------------------------------*/
int doCoastLineFm(ob_desc * ob)
{
char filename[_MAX_PATH];
t_setup * config = QuerySetup();
int retv;
char path[_MAX_PATH];

strcpy(path, CurrentDir());
strcat(path, "\\*.cst");

retv = pickFilename("Select Coastline File", path, filename);
if (retv == SUCCESS)
	{
	// load the new coastline
	strcpy(config->CoastName, filename);
	curNav->lat = coast.pts[coast.numPts/2].lat;
	curNav->lon = coast.pts[coast.numPts/2].lon;
	read_coastline();

	// cause the screen to refresh
	RefreshRequired = TRUE;
	}
return retv;
}


/*------------------------------------------------------------------*/
// Pick coastline file
/*------------------------------------------------------------------*/
int pickFilename(char * title, char * fileSpec, char * fileName)
{
fm_desc *fm;
static int pick=0;
static int top=0;
int retv;
file_list fList;
int flist_retv;

Notify( -1, -1, "Scanning directory..." );
flist_retv = CreateFileList( &fList, fileSpec, 0 );
NotifyOff();

/* check for possible error returns */
switch( flist_retv )
	{
	case  1 : /* no files found */
		NotifyUser(-1, -1, fileSpec, "No Match" );
		return NO_MATCHING_FILES;
		break;

	case -1 : /* out of memory */
		DestroyFileList( &fList );
		NotifyUser(-1, -1, fileSpec, "Out of Memory" );
		return NO_MEMORY;
		break;
	}

/* create and open list form with list of file names */
fm = CreateListForm( title, 7, 2, 20, 7, 0 );

fm->ob_lst[9].label = "Okay";
fm->ob_lst[9].state = 8;
fm->ob_lst[9].flags |= IS_EXITABLE;
fm->ob_lst[9].key = 'O';

fm->ob_lst[10].label = "Cancel";
fm->ob_lst[10].flags |= IS_EXITABLE;
fm->ob_lst[10].key = 'C';
fm->ob_lst[10].state = 8;

OpenForm( fm, -1, -1);
do
	{
	retv  = ProcessListForm( fm, fList.fnames, fList.nfiles, &top, &pick );
	switch ( retv ) 
		{
		case 9:            // Okay
			strcpy(fileName, fList.fnames[pick]);
			break;

		case 10:           // cancel
			break;
		}
	} while (retv != 9 && retv != 10);

// get rid of list form 
CloseForm( fm );
DestroyForm( fm );

// get rid of file list 
DestroyFileList( &fList );
return (retv == 9) ? SUCCESS : -1;
}





/*----------------------------------------------------------------------------*/
// read the Coastline
/*----------------------------------------------------------------------------*/
int read_coastline(void)
{
FILE *fp;
int k;
t_setup * config = QuerySetup();

if ((fp = fopen(config->CoastName, "rb")) == NULL)
  {
  printf("\nCan't open file %s", config->CoastName);
  return FALSE;
  }

k = fread( (char *) &coast, sizeof(t_coast), 1, fp);
if (k == -1)
	{
	fclose(fp);
	printf("\nfread failed");
	return FALSE;
	}

coastLineLoaded = TRUE;

fclose(fp);
return TRUE;
}



/*----------------------------------------------------------------------------*/
// return a pointer to the coastline structure
/*----------------------------------------------------------------------------*/
t_coast * queryCoastline(void)
{
return (coastLineLoaded) ? (t_coast *) &coast : (t_coast *) NULL;
}



