/*----------------------------------------------------------------------------*/
// File: config.c   USGS Navigation, utility forms for general configuration
// Blame: John Gann
// Start Date: 3 May 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"
#include "mnutils.h"
#include "serial.h"

#include "config.h"

// private data
static char * NO_BRIDGE_MSG =  "No Port is configured for Bridge Terminal";

// externals
extern char RefreshRequired;
extern fm_desc * mainMenu;

/*----------------------------------------------------------------------------*/
// Set the coordinate system
/*----------------------------------------------------------------------------*/
int doCoordSysFm(ob_desc * ob)
{
int retv;
fm_desc * fm;
char * frf = "coordsys.frf";
t_setup * config = QuerySetup();

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// set up Units form
fm->ob_lst[2].flags &= ~IS_CHECKED;
fm->ob_lst[3].flags &= ~IS_CHECKED;

switch (config->CoordSys)
	{
	case LATITUDE_LONGITUDE:
		fm->ob_lst[2].flags |=  IS_CHECKED;
		break;

	case UNIVERSAL_TRANSVERSE_MERCATOR:
		fm->ob_lst[3].flags |=  IS_CHECKED;
		break;
	}

OpenForm ( fm, ob->area.Xmin, ob->area.Ymax ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the flags
			switch (get_flag_fm_group(fm, 1))
				{
				case 0:
					config->CoordSys = LATITUDE_LONGITUDE;
					break;
			
				case 1:
					config->CoordSys = UNIVERSAL_TRANSVERSE_MERCATOR;
					break;
				}
			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL && retv != 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


/*----------------------------------------------------------------------------*/
// Set the display Units
/*----------------------------------------------------------------------------*/
int doUnitsFm(ob_desc * ob)
{
int retv;
fm_desc * fm;
char * frf = "units.frf";
ftag * tag;
t_setup * config = QuerySetup();
int fm_width;

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

fm_width = (fm->area.Xmax - fm->area.Xmin) / 2;

// set up Units form
fm->ob_lst[2].flags &= ~IS_CHECKED;
fm->ob_lst[3].flags &= ~IS_CHECKED;
fm->ob_lst[4].flags &= ~IS_CHECKED;
tag = (ftag *) fm->ob_lst[5].tag;
sprintf(tag->fld, "%d", config->DecimalAccuracy);

switch (config->AngleType)
	{
	case DEC_DEG:
		fm->ob_lst[2].flags |=  IS_CHECKED;
		break;

	case DEG_MIN:
		fm->ob_lst[3].flags |=  IS_CHECKED;
		break;

	case DEG_MIN_SEC:
		fm->ob_lst[4].flags |=  IS_CHECKED;
		break;
	}

OpenForm ( fm, ob->area.Xmin - fm_width, ob->area.Ymax ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the flags
			switch (get_flag_fm_group(fm, 1))
				{
				case 0:
					config->AngleType = DEC_DEG;
					break;
			
				case 1:
					config->AngleType = DEG_MIN;
					break;
			
				case 2:
					config->AngleType = DEG_MIN_SEC;
					break;
				}

			config->DecimalAccuracy = atoi(tag->fld);
			if (config->DecimalAccuracy > 10)
				{
				config->DecimalAccuracy = 10;
				}

			if (config->DecimalAccuracy < 0)
				{
				config->DecimalAccuracy = 0;
				}
			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL && retv != 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}




/*----------------------------------------------------------------------------*/
// Set the foreground color
/*----------------------------------------------------------------------------*/
void doForegroundColor ( ob_desc * ob )
{
int retv;
attr_blk *attr;
fm_desc * fm;
t_setup * config = QuerySetup();

if ((fm = loadFRF("color.frf")) == (fm_desc *) NULL ) 
	return;

set_flag_fm_group(fm, 0, config->fg);

attr = GetAttrBlk ( attrCBut );

fm->ob_lst[4].attr = attrUsr1;
attr->fg_clr = 0;
PutAttrBlk ( attrUsr1, attr );

fm->ob_lst[5].attr = attrUsr2;
attr->fg_clr = 1;
PutAttrBlk ( attrUsr2, attr );

fm->ob_lst[6].attr = attrUsr3;
attr->fg_clr = 2;
PutAttrBlk ( attrUsr3, attr );

fm->ob_lst[7].attr = attrUsr4;
attr->fg_clr = 3;
PutAttrBlk ( attrUsr4, attr );

fm->ob_lst[8].attr = attrUsr5;
attr->fg_clr = 4;
PutAttrBlk ( attrUsr5, attr );

fm->ob_lst[9].attr = attrUsr6;
attr->fg_clr = 5;
PutAttrBlk ( attrUsr6, attr );

fm->ob_lst[10].attr = attrUsr7;
attr->fg_clr = 6;
PutAttrBlk ( attrUsr7, attr );

fm->ob_lst[11].attr = attrUsr8;
attr->fg_clr = 7;
PutAttrBlk ( attrUsr8, attr );

fm->ob_lst[12].attr = attrUsr9;
attr->fg_clr = 8;
PutAttrBlk ( attrUsr9, attr );

fm->ob_lst[13].attr = attrUsr10;
attr->fg_clr = 9;
PutAttrBlk ( attrUsr10, attr );

fm->ob_lst[14].attr = attrUsr11;
attr->fg_clr = 10;
PutAttrBlk ( attrUsr11, attr );

fm->ob_lst[15].attr = attrUsr12;
attr->fg_clr = 11;
PutAttrBlk ( attrUsr12, attr );

fm->ob_lst[16].attr = attrUsr13;
attr->fg_clr = 12;
PutAttrBlk ( attrUsr13, attr );

fm->ob_lst[17].attr = attrUsr14;
attr->fg_clr = 13;
PutAttrBlk ( attrUsr14, attr );

fm->ob_lst[18].attr = attrUsr15;
attr->fg_clr = 14;
PutAttrBlk ( attrUsr15, attr );

fm->ob_lst[19].attr = attrUsr16;
attr->fg_clr = 15;
PutAttrBlk ( attrUsr16, attr );

OpenForm ( fm, -1, -1 );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );

	switch ( retv ) 
		{
		case OKAY : 
			config->fg = get_flag_fm_group(fm, 0);
			break;

		case DEFAULT : 
			break;

		case CANCEL : 
			break;

		default : 
			break;
		}
	} while ( ( retv != OKAY ) && ( retv != CANCEL ) );

CloseForm      ( fm );
DestroyFRFForm ( fm );

if (retv == CANCEL || retv == NOT_SET) 
	return;

SetAttrTblColors ( config->fg, config->bg );
OpenApp( NAV_VERSION , mainMenu, NULL);

fm = (fm_desc *) &mainMenu->ob_lst[4].tag;          // the config menu
DrawForm(fm);

fm = (fm_desc *) &fm->ob_lst[2].tag;               // the foreground menu
ToggleRect ( &fm->area, rProt );
}


/*----------------------------------------------------------------------------*/
// Set the bridge display config & optionally send a message
/*----------------------------------------------------------------------------*/
int doBridgeCfgFm(ob_desc * ob)
{
int retv;
fm_desc * fm;
char * frf = "brdg_cfg.frf";
ftag * tag;
t_setup * config = QuerySetup();
int fm_width;
t_bridgeMsg bridgeMsg;
t_bridgeSetup bridgeSetup;
int flag;
int port;

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

fm_width = (fm->area.Xmax - fm->area.Xmin) / 2;

// set up Units form
if (config->bridge.mode == LINE_INFO)
	{
	fm->ob_lst[3].flags |=  IS_CHECKED;
	}
else
	{
	fm->ob_lst[4].flags |=  IS_CHECKED;
	}

// get a pointer to the message fld
tag = (ftag *) fm->ob_lst[5].tag;

OpenForm ( fm, 10, ob->area.Ymax ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the flags
			flag = (fm->ob_lst[3].flags & IS_CHECKED) ? LINE_INFO : SITE_INFO;
			if (flag != config->bridge.mode)
				{
				// See if the bridge terminal is enabled
				port = queryBridgePort();
				if (port < 0)
					{
					NotifyUser(-1, -1, "Yo", NO_BRIDGE_MSG);
					}
				else
					{
					bridgeSetup.mode = flag;
					sendPacket(port,                                  // portnum
                  BRIDGE_SETUP,                             // packet ID
                  (unsigned char *) &bridgeSetup,           // data 
                  sizeof(t_bridgeSetup));                   // size of data
					}
				}

			config->bridge.mode = flag;
			break;

		case 1:                                  // CANCEL:
			break;

		case SEND_MSG:
			// See if the bridge terminal is enabled
			port = queryBridgePort();
			if (port < 0)
				{
				NotifyUser(-1, -1, "Yo", NO_BRIDGE_MSG);
				}
			else
				{
				strcpy(bridgeMsg.msg, tag->fld);
				bridgeMsg.urgency = INFORMATION_ONLY;                // Don't need ack
				sendPacket(port,                                     // portnum
                   BRIDGE_ALERT,                             // packet ID
                   (unsigned char *) &bridgeMsg,             // data 
                   sizeof(t_bridgeMsg));                     // size of data
				}
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL && retv != 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


/*----------------------------------------------------------------------------*/
// See if the bridge terminal is enabled
/*----------------------------------------------------------------------------*/
int queryBridgePort(void)
{
return queryComIdxByNav(BRIDGE_TERM);
}

