;---------------------------------------------------------------------
;         I T H A C A   S T R E E T   S O F T W A R E,   I N C.
;               Copyright (c) 1989 - All rights reserved
;---------------------------------------------------------------------
;   File:  CRITERR.ASM
;   Desc:  LIM EMS functions
;   Date:  June, 1989
;   Model: LARGE
;   Calls: C
;---------------------------------------------------------------------

.MODEL LARGE

;---------------------------------------------------------------------
;  Initialized data...
;---------------------------------------------------------------------

.DATA

PUBLIC    _is_critical_err, _ce_disk_err
PUBLIC    _ce_drive_num, _ce_err_code

_is_critical_err   dw   0
_ce_disk_err       dw   0FFFFh
_ce_drive_num      dw   0FFFFh
_ce_err_code       dw   0FFFFh
vect_36_seg        dw   0
vect_36_ofs        dw   0

;---------------------------------------------------------------------
; Code...
;---------------------------------------------------------------------

.CODE

PUBLIC  _ResetCritErr, _InstallCEH, _RemoveCEH

;---------------------------------------------------------------------
; Usage:   Not to be called by an application
; Desc:    Handle critical errors
;---------------------------------------------------------------------

_CritErrHandler PROC far

        push ds                            ; save registers
        push si
        push bx
        push cx

        sti                                ; enable interrupts

        mov  si, DGROUP                    ; turn on critical err flag
        mov  ds, si
        mov  _is_critical_err, 0001h

        mov  bx, ax                        ; set disk err flag
        and  bx, 8000h
        xor  bx, 8000h
        mov  cl, 1
        rol  bx, cl
        mov  _ce_disk_err, bx

        mov  bx, ax                        ; set drive number
        and  bx, 00FFh
        mov  _ce_drive_num, bx

        mov  bx, di                        ; set error code
        and  bx, 00FFh
        mov  _ce_err_code, bx

        and  ax, 0FF00h                    ; set ax for DOS

        pop  cx                            ; restore registers
        pop  bx
        pop  si
        pop  ds

        iret                               ; return from interrupt

_CritErrHandler ENDP

;---------------------------------------------------------------------
; Usage:   void far ResetCritErr( void );
; Desc:    Clears critical error flags
; Returns: Nothing
;---------------------------------------------------------------------

_ResetCritErr PROC far

        push ds                            ; save registers
        push si

        mov  si,   DGROUP                  ; turn off critical err flag
        mov  ds,   si
        mov  _is_critical_err, 0

        pop  si                            ; restore registers, return
        pop  ds
        ret

_ResetCritErr ENDP

;---------------------------------------------------------------------
; Usage:   void far InstallCEH( void );
; Desc:    Save current CEH vector, install ours
; Returns: Nothing
;---------------------------------------------------------------------

_InstallCEH PROC far

        push ds                            ; save registers
        push es
        push di
        push si

        mov  si, DGROUP                    ; access our data
        mov  ds, si

        mov  ah, 35h                       ; get, save current vector
        mov  al, 24h
        int  21h
        mov  vect_36_seg, es
        mov  vect_36_ofs, bx

        mov  ah, 25h                       ; install our handler
        mov  al, 24h
        push cs
        mov  dx, offset _CritErrHandler
        pop  ds
        int  21h

        pop  si                            ; restore registers, return
        pop  di
        pop  es
        pop  ds
        ret

_InstallCEH ENDP

;---------------------------------------------------------------------
; Usage:   void far RemoveCEH( void );
; Desc:    Restore previous CEH vector
; Returns: Nothing
;---------------------------------------------------------------------

_RemoveCEH PROC far

        push ds                            ; save registers
        push si

        mov  si, DGROUP                    ; access our data
        mov  ds, si
        push ds

        mov  ah, 25h                       ; install old handler
        mov  al, 24h
        mov  dx, vect_36_ofs
        mov  si, vect_36_seg
        mov  ds, si
        int 21h

        pop  ds                            ; clear error flag
        xor  ax, ax
        mov  _is_critical_err, ax

        pop  si                            ; restore registers, return
        pop  ds
        ret

_RemoveCEH ENDP

              end
