/*----------------------------------------------------------------------------*/
// File: dispasci.c
// Author: John Gann
// Start Date: 5 Sep 1991
// Puprose: Replacement for the memory hog display.c.  Displays and prints 
//          an ascii text file using scroll bars.
/*----------------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <time.h>
#include <bios.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "criterr.h"

#include "dispasci.h"

#define FORM_FEED  12

// Global data
static int LeftCol = 0;                       // current lefthand display col
static int Rows = 0;                          // rows on the display
static int Cols = 0;                          // columns on the display
static long DisplayFileSize = 0L;             // Filesize in bytes
static FILE * DisplayFP = (FILE *) NULL;      // the input file pointer
static char DisplayFilename[_MAX_PATH];       // the input filename
static fm_desc * DisplayFm;                   // pointer to display form
static t_Lines theScreen[MAX_ROWS];           // lines that make a screen

static void eject_printer_page(void);

extern int (*usrKeyTask)(ob_desc * ob);       // user supplied keyTask

/*----------------------------------------------------------------------------*/
/* Non-memory hog replacement for displaying an ascii file                    */
/*----------------------------------------------------------------------------*/
int DisplayAsciiFile(char * path, int x, int y, int rows, int cols)
{
int retv;

// Make sure the file exists and conatins ascii data
if (retv = VerifyFileName(path))
	{
	return retv;
	}

// Open File
if ((DisplayFP = fopen(path, "r")) == (FILE *) NULL)
	{
	return BAD_FILE;
	}
else
	{
	strcpy(DisplayFilename, path);
	}

// Create the display form 
if ((DisplayFm = CreateDisplayForm(FileNameP(path), rows, cols)) == (fm_desc *) NULL)
	{
	return UNABLE_TO_CREATE_FORM;
	}

// Open and Initialize the form
InitializeDisplayForm(x, y);

// Process the display form 
ProcessDisplayForm();

// Clean Up
CleanUpDisplay();

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// See if the file exists and contains ascii data                              
/*----------------------------------------------------------------------------*/
static int VerifyFileName(char * filename)
{
char     fnam[16];
struct find_t fbuf;

strcpy( fnam, " " );
strcat( fnam, FileNameP(filename) );
strcat( fnam, " " );

/* see if file exists */
if (_dos_findfirst(filename, _A_NORMAL, &fbuf))
  {
	NotifyUser(-1, -1, filename, "File Not Available");
	return FILE_DOES_NOT_EXIST;
	}

if (fbuf.size <= 0L)
	{
	NotifyUser(-1, -1, filename, "File is Empty, continuing.....");
	}
else
	{
	DisplayFileSize = fbuf.size;
	}

return SUCCESS;
}





/*----------------------------------------------------------------------------*/
// Create the display form
/*
 *
 *  Form Ob Layout:   Ob(s)         Desc
 *                    -----------   ----------------------
 *                    0             Edit display area
 *                    1 to n        n edit lines
 *                    n+1           V sb
 *                    n+2           H sb
 *                    n+3           Okay Button
 *                    n+4           Print Button
 *
 */
/*----------------------------------------------------------------------------*/
static fm_desc * CreateDisplayForm(char * title, int rows, int cols)
{
static fm_desc *fm;
ob_desc *ob;
ob_desc *slot_ob;
ob_desc *h_sb;
ob_desc *v_sb;
int      i, w, h, d, x, y;

// Normalize the rows and columns
if (rows > MAX_ROWS)
	rows = MAX_ROWS;

if (rows < MIN_ROWS)
	rows= MIN_ROWS;

if (cols > MAX_COLS)
	cols = MAX_COLS;

if (cols < MIN_COLS)
	cols = MIN_COLS;

// Save these values as globals
Rows = rows;
Cols = cols;

// create form and objects
fm = CreateForm(rows + 5);
if ( fm == NULL ) 
	return( NULL );

// Assign the title
fm->title = title;
ob = fm->ob_lst;

// figure out sizes and dimensions 
d = QryDefSbWid();
w = (cols*charW + 2) + (charH*3) + d;
h = ((rows+4)*charH + 2) + (charH*3) + d;

// Build the form
SetRect( &fm->area, 0, 0, w, h );
fm->flags = IS_INIT;
fm->attr  = attrEdit;

// the display area 
x = charH;
y = charH*3;
w = cols*charW + 2;
h = rows*charH + 2;
SetRect( &ob->area, x, y, x+w-1, y+h-1 );
ob->attr  = attrArea;
ob->otyp  = obSlot;
ob->flags = IS_PASSIVE;

// the display lines 
x++;
y++;
w -= 2;
h = charH;
for( i=0, slot_ob=ob+1; i<rows; i++, slot_ob++ )
  {
  SetRect( &slot_ob->area, x, y, x+w-1, y+h-1 );
  slot_ob->attr  = attrSlot;
  slot_ob->otyp  = obSlot;
  slot_ob->flags = IS_PASSIVE;
  slot_ob->hook =  -1L ;
  slot_ob->links = LINK_ANY;
  y += h;
  }

// vertical scroll bar 
v_sb = ob+rows+1;
h = rows*charH;
x = ob->area.Xmax + charH;
y = ob->area.Ymin + 1;
v_sb->otyp = obSBar;
v_sb->ttyp = SBR_TAG;
v_sb->attr = attrVArea;
v_sb->links = LINK_ANY;
v_sb->task = VerticalScrollTask;
v_sb->flags = IS_TAGGED | IS_SELECTABLE | IS_INVERTIBLE;
InitSBar( v_sb, ALIGN_VERT|READ_DEC, d, h );
OffsetObList( v_sb, 1, x, y );
v_sb->flags |= IS_EXECUTABLE | IS_PRESSABLE;
v_sb->task = VerticalScrollTask;
v_sb->links = LINK_ANY;

// horizontal scroll bar 
h_sb = ob+rows+2;
h = cols*charW;
x = ob->area.Xmin + charW;
y = ob->area.Ymax + charH;
h_sb->otyp = obSBar;
h_sb->ttyp = SBR_TAG;
h_sb->attr = attrVArea;
h_sb->links = LINK_ANY;
h_sb->task = HorizontalScrollTask;
h_sb->flags = IS_TAGGED | IS_SELECTABLE | IS_INVERTIBLE;
InitSBar( h_sb, ALIGN_HORZ|READ_DEC, d, h );
OffsetObList( h_sb, 1, x, y );
h_sb->flags |= IS_EXECUTABLE | IS_PRESSABLE;
h_sb->task = HorizontalScrollTask;
h_sb->links = LINK_ANY;

// ok button 
x = h_sb->area.Xmin;
y = h_sb->area.Ymax + charH;
memcpy( &ob[rows+3], defaultBut(), sizeof(ob_desc) );
SetRect( &ob[rows+3].area, x, y, x+charW*7, y+d+2 );
ob[rows+3].key = 'O';
ob[rows+3].state = 8;
ob[rows+3].label = "Okay";
BxArea2Work( &ob[rows+4].area, &ob[rows+4].area, ob[rows+4].attr );

// print button
x = h_sb->area.Xmax - charH * 3;
y = h_sb->area.Ymax + charH;
memcpy( &ob[rows+4], defaultBut(), sizeof(ob_desc) );
SetRect( &ob[rows+4].area, x, y, x+charW*7, y+d+2 );
ob[rows+4].key = 'P';
ob[rows+4].state = 8;
ob[rows+4].label = "Print";
ob[rows+4].task = PrintTask;
ob[rows+4].flags |= IS_EXECUTABLE | IS_SELECTABLE | IS_INVERTIBLE;
BxArea2Work( &ob[rows+4].area, &ob[rows+4].area, ob[rows+4].attr );

return( fm );
}








/*----------------------------------------------------------------------------*/
// Task to handle Vertical Scroll Bar
/*----------------------------------------------------------------------------*/
static int VerticalScrollTask(ob_desc * ob)
{
int pc;
long v;
app_msg * Msg;
stag * tag;
long l_retv;

switch (WhichTagPick( ob ))
	{
	case SBR_BGN:
		rewind(DisplayFP);
		DrawDisplayPage();
		UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
		break;

	case SBR_MINUS: /* scrl up */
		tag = (stag *) ob;
		l_retv = ScrollOneLine(DIR_UP);
		UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);

		/* Pause before repeating */
		WaitFor( 3 );

		/* Repeat until mouse is released */
		while ((!MsgInQueue()) && (l_retv > 0L))
			{
			if ((!PtInRect(MouseXY(1), &tag->plus)) &&
			    (!PtInRect(MouseXY(1), &tag->minus)) )
				break;

			// Continue while mouse button is depressed
			Msg = ReReadMsgQueue();
			if (isMsPress(Msg))
				{
				l_retv = ScrollOneLine(DIR_UP);
				UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
				}
			else
				{
				break;
				}
			}
		break;

	case SBR_KNOB: 
		pc = AdjustSbKnob( ob );
		v = LongScaled( DisplayFileSize, (long) pc, 100L );

		if (v < 1L)
			v = 1L;

		fseek(DisplayFP, v, SEEK_SET);
		GetOffset(DisplayFP, Rows);
		DrawDisplayPage();
		UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
		break;

	case SBR_PLUS: 
		tag = (stag *) ob;
		l_retv = ScrollOneLine(DIR_DOWN);
		UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);

		/* Pause before repeating */
		WaitFor( 3 );

		/* Repeat until mouse is released */
		while ((!MsgInQueue()) && (l_retv > 0L))
			{
			if ((!PtInRect(MouseXY(1), &tag->plus)) &&
			    (!PtInRect(MouseXY(1), &tag->minus)) )
				break;

			// Continue while mouse button is depressed
			Msg = ReReadMsgQueue();
			if (isMsPress(Msg))
				{
				l_retv = ScrollOneLine(DIR_DOWN);
				UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
				}
			else
				{
				break;
				}
			}
		break;


	case SBR_END: 
		fseek(DisplayFP, 0L, SEEK_END);
		GetOffset(DisplayFP, Rows);
		DrawDisplayPage();
		UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
		break;
	}    /* end of Switch */

UnpressObject( ob, 0 );
return TRUE;
}



/*----------------------------------------------------------------------------*/
// Task to handle Horizontal Scroll Bar
/*----------------------------------------------------------------------------*/
static int HorizontalScrollTask(ob_desc * ob)
{
return USER_CONTINUE;
}

/*----------------------------------------------------------------------------*/
// Task to Open display and draw first page
/*----------------------------------------------------------------------------*/
static int InitializeDisplayForm(int x, int y)
{
// Check passed params
if (DisplayFm == (fm_desc *) NULL)
	{
	return BAD_FORM;
	}

if (DisplayFP == (FILE *) NULL)
	{
	return BAD_FILE;
	}

// Set a key task to handle our form
SetUserKeyTask (DisplayKeyTask);

// Open the form
OpenForm(DisplayFm, x, y);

// Draw First Page
DrawDisplayPage();
}


/*----------------------------------------------------------------------------*/
// Task to Clean up after display
/*----------------------------------------------------------------------------*/
static int ProcessDisplayForm(void)
{
int retv;
do
	{
	retv = ProcessObList(DisplayFm->ob_lst, DisplayFm->ob_cnt, 0);
	} while ((retv != DisplayFm->ob_cnt - 2) && ( retv != NOT_SET) );

return retv;
}





/*----------------------------------------------------------------------------*/
// Task to Clean up after display
/*----------------------------------------------------------------------------*/
static void CleanUpDisplay(void)
{
if (DisplayFP != (FILE *) NULL)
	{
	fclose(DisplayFP);
	}
SetUserKeyTask(usrKeyTask);
CloseForm( DisplayFm );
DestroyForm( DisplayFm );
}




/*----------------------------------------------------------------------------*/
// Task to Draw a page of text
/*----------------------------------------------------------------------------*/
static int DrawDisplayPage(void)
{
int i = 1;
int rows = Rows;
int cols = Cols;
char text[MAX_TEXT];
long this_pos = 0L;

// Check passed params
if (DisplayFm == (fm_desc *) NULL)
	{
	return BAD_FORM;
	}

if (DisplayFP == (FILE *) NULL)
	{
	return BAD_FILE;
	}

// Clear the display area
ClearDisplayArea( );

while (rows-- && ! feof(DisplayFP))
	{
	this_pos = ftell(DisplayFP);

	// get a line of text
	if ((fgets(text, MAX_TEXT - 1, DisplayFP)) == (char *) NULL)
		{
		break;
		}

	// write it
	DrawLineOfText(i++, text, 0, cols, this_pos);
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Clear the Display Area 
/*----------------------------------------------------------------------------*/
static void ClearDisplayArea( void )
{
rect area;
int i;
ob_desc * ob;

SetRect( &area, DisplayFm->ob_lst[0].area.Xmin, DisplayFm->ob_lst[0].area.Ymin,
         DisplayFm->ob_lst[0].area.Xmax, DisplayFm->ob_lst[0].area.Ymax );
ProtectRect( &area );
PenColor( QryFG(DisplayFm->ob_lst[0].attr) );
BackColor( QryBG(DisplayFm->ob_lst[0].attr) );
FillRect( &area, 0 );
ProtectOff();

/* Reset all hooks */
for (i=0, ob = &DisplayFm->ob_lst[1]; i<Rows; ob++, i++)
	{
	ob->hook = -1;
	}

}


	


/*----------------------------------------------------------------------------*/
// Draw a line of text on the display area
/*----------------------------------------------------------------------------*/
static int DrawLineOfText(int ob_idx, char * text, int col1, int col2, long FO)
{
ob_desc * ob = &DisplayFm->ob_lst[ob_idx];

ProtectRect(&ob->area);
MoveTo( ob->area.Xmin, ob->area.Ymax );
DrawText( text, col1, col2 );
ProtectOff();

// store current file offset and text at proper line
ob->hook = FO; 

// store text in global Array
strcpy(theScreen[ob_idx-1].text, text);

return TRUE;
}



/*----------------------------------------------------------------------------*/
// Key task to handle PageUp, PageDn, End and Home Keys for display
/*----------------------------------------------------------------------------*/
static int DisplayKeyTask ( ob_desc * ob )
{
app_msg *key;
point * pt = MouseXY(0);
long this_pos;

key = ReReadMsgQueue ();

if ( isKeyPadKey ( key ) )
	{
	switch (key->msgValue)
		{
		case kPGUP:
			// Are we already at the top of form??
			if (DisplayFm->ob_lst[1].hook == 0L)
				{
				return USER_IGNORE;
				}

			// Set the file pointer to the top of the form
			fseek(DisplayFP, DisplayFm->ob_lst[1].hook, SEEK_SET);

			// Get the previous page
			this_pos = GetOffset(DisplayFP, Rows);
			DrawDisplayPage();

			/*fseek(DisplayFP, this_pos, SEEK_SET);*/
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			break;

		case kPGDN:
			// Set the file pointer to the bottom of the form
			fseek(DisplayFP, DisplayFm->ob_lst[LastRow()].hook, SEEK_SET);

			DrawDisplayPage();
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			break;

		case kHOME:
			rewind(DisplayFP);
			DrawDisplayPage();
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			break;

		case kEND:
			fseek(DisplayFP, 0L, SEEK_END);
			GetOffset(DisplayFP, Rows);
			DrawDisplayPage();
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			break;
		}          /* End of switch() */
	}   // End of if KeyPadKey

/* Handle Cursor Keys */
if ( isCursorKey ( key ) )
	{
	switch(key->msgValue)
		{
		case uARROW:
			HideCursor();
			ScrollOneLine(DIR_UP);
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			ShowCursor();
			return USER_IGNORE;
			break;

		case dARROW:
			HideCursor();
			ScrollOneLine(DIR_DOWN);
			UpdateVertScrollBar(&DisplayFm->ob_lst[Rows+1]);
			ShowCursor();
			return USER_IGNORE;
			break;
		}
	}    /* end of if cursor Key */

return ( USER_CONTINUE );
}





/*----------------------------------------------------------------------------*/
// Get the file offset to nlines of text
/*----------------------------------------------------------------------------*/
static long GetOffset(FILE * fp, int nlines)
{
long this_pos =0L;
int count=0;
int ch=0;
int i=1;

this_pos = ftell(fp);

// read backwards nlines
do
	{
	this_pos = GetPreviousLine(fp);
	if (this_pos > 0L)
		count ++;
	} while (this_pos > 0L && count < nlines);

fseek(fp, this_pos, SEEK_SET);
return this_pos;
}




/*----------------------------------------------------------------------------*/
// Get the file offset to the previous line and put the file pointer there
// I assume that the file pointer is currently sitting at the start of an ascii
// record, i.e. the first record or else the position just after the CR LF
/*----------------------------------------------------------------------------*/
static long GetPreviousLine(FILE * fp)
{
long this_pos =0L;
int count=0;
int ch=0;
int i=1;

this_pos = ftell(fp);
if (this_pos > 2L)
	{
	this_pos -= 2L;            // skip back past CR LF
	fseek(fp, this_pos, SEEK_SET);
	}
else
	{
	return 0L;
	}

// read backwards 1 line
do
	{
	// set the file pointer back one char
	this_pos--;
	fseek(fp, this_pos, SEEK_SET);

	// read the char at that location
	ch = fgetc(fp);

	// if it's a newline then return that position
	if (ch == '\n')
		{
		fseek(fp, this_pos++, SEEK_SET);
		return this_pos;
		}
	} while (this_pos > 0L);

return this_pos;
}




/*----------------------------------------------------------------------------*/
/* Scroll the screen one line at a time if there is one more line of input    */
/* to be had. Return the slot number that was written or a negative error     */
/*----------------------------------------------------------------------------*/
static long ScrollOneLine(int direction)
{
ob_desc * ob;
int ob_idx;
long this_pos = ftell(DisplayFP);
char text[MAX_TEXT];

// Get the slot at the top (or bottom) of the screen
switch (direction)
	{
	case DIR_UP:
		// Are we already at the top of form??
		ob = (ob_desc *) &DisplayFm->ob_lst[1];     
		ob_idx = 1;
		if (ob->hook == 0L)
			{
			return -1L;
			}

		// Set the file pointer to the top of the form
		fseek(DisplayFP, ob->hook, SEEK_SET);

		// Get previous line of text
		this_pos = GetPreviousLine(DisplayFP);
		if (this_pos >= 0L)
			{
			fseek(DisplayFP, this_pos, SEEK_SET);
			InsDRow( 0, 0, Cols-1 );

			// get a line of text
			if ((fgets(text, MAX_TEXT - 1, DisplayFP)) == (char *) NULL)
				{
				Beep();
				return -1L;
				}
			}
		else
			{
			Beep();
			return -1L;
			}

		break;

	case DIR_DOWN:
		ob = (ob_desc *) &DisplayFm->ob_lst[Rows];     
		ob_idx = Rows;

		// Set the file pointer to the bottom of the form
		fseek(DisplayFP, DisplayFm->ob_lst[LastRow()].hook, SEEK_SET);

		// get bottom line of text again
		if ((fgets(text, MAX_TEXT - 1, DisplayFP)) == (char *) NULL)
			{
			Beep();
			return -1L;
			}

		// store the file offset for this next record
		this_pos = ftell(DisplayFP);

		// then get next line of text
		if ((fgets(text, MAX_TEXT - 1, DisplayFP)) == (char *) NULL)
			{
			Beep();
			return -1L;
			}

		// Clear out the old text
		DelDRow( 0, 0, Cols-1 );
		break;
	}
		
AdjustHooks((char) direction, this_pos);
DrawLineOfText(ob_idx, text, 0, Cols, this_pos);

return ftell(DisplayFP);
}



/*--------------------------------------------------------------------*/
/*  Insert a row from col1 to col2 in edit display...                 */
/*--------------------------------------------------------------------*/
static void InsDRow( int row, int col1, int col2 )
{
rect r;
rect area;

DupRect( &DisplayFm->ob_lst->area, &area );
r.Ymin = area.Ymin + row*charH + 1;
r.Ymax = area.Ymax - charH - 1;
r.Xmin = area.Xmin + col1*charW + 1;
r.Xmax = area.Xmin + (col2+1)*charW;
ProtectRect( &area );
ScrollRect( &r, 0, charH );
ProtectOff();
}

/*--------------------------------------------------------------------*/
/*  Delete a row from col1 to col2 in edit display...                 */
/*--------------------------------------------------------------------*/
static void DelDRow( int row, int col1, int col2 )
{
rect r;
rect area;

DupRect( &DisplayFm->ob_lst->area, &area );
r.Ymin = area.Ymin + (row+1)*charH + 1;
r.Ymax = area.Ymax - 1;
r.Xmin = area.Xmin + col1*charW + 1;
r.Xmax = area.Xmin + (col2+1)*charW;
ProtectRect( &area );
ScrollRect( &r, 0, -charH );
ProtectOff();
}





/*----------------------------------------------------------------------------*/
/* Function to adjust Scroll Bar Knob on Edit Form                            */
/*----------------------------------------------------------------------------*/
static void UpdateVertScrollBar( ob_desc * knob )
{
PositionSbKnob( knob, LongPercent(ftell(DisplayFP), DisplayFileSize));
}



/*----------------------------------------------------------------------------*/
/* return the int of a percentage computation                                 */
/*----------------------------------------------------------------------------*/
static int LongPercent(long numerator, long denominator)
{
if ((denominator >= 0L) && (denominator <= numerator))
	return 100L;

return (denominator <= 0L) ? (int) 0 : 
                             (int) (((100L * numerator) / denominator) % 100L);
}




/*----------------------------------------------------------------------------*/
// return the last row with a valid hook
/*----------------------------------------------------------------------------*/
static int LastRow(void)
{
int i = 1;

for (i=1; i< Rows; i++)
	{
	if (DisplayFm->ob_lst[i].hook == -1L)
		{
		return (i == 1) ? i : i-1;
		}
	}

return Rows;
}


/*----------------------------------------------------------------------------*/
/* When the user scrolls the display by one line adjust the hook value for
   each slot object.  Each slot's hook contains the file offset for this text.
/*----------------------------------------------------------------------------*/
static void AdjustHooks(char direction, long new_val)
{
fm_desc * fm = DisplayFm;
int i=0, j=0;
ob_desc * ob;

switch (direction)
	{
	case DIR_DOWN:
		for (i=0, ob = &fm->ob_lst[1]; i<Rows-1; ob++, i++)
			{
			ob->hook = (ob+1)->hook;
			}
		ob->hook = new_val;
		break;

	case DIR_UP:
		/* start at the last slot and work backwards */
		for (i=0, ob = &fm->ob_lst[Rows]; i<Rows-1; ob--, i++)
			{
			ob->hook = (ob-1)->hook;
			}
		ob->hook = new_val;
		break;
	}
}



/*----------------------------------------------------------------------------*/
/* return the (i * m) / d long version of IScaled                             */
/*----------------------------------------------------------------------------*/
static long LongScaled(long i, long m, long d)
{
return (d == 0L) ? 0L : (i * m) / d;
}



/*--------------------------------------------------------------------*/
/*  Insert a column from row1 to row2 in edit display...              */
/*--------------------------------------------------------------------*/
static void InsDCol( int col, int row1, int row2 )
{
rect r;
rect area;

DupRect( &DisplayFm->ob_lst->area, &area );
r.Xmin = area.Xmin + col*charW + 1;
r.Xmax = area.Xmax - charW - 1;
r.Ymin = area.Ymin + row1*charH + 1;
r.Ymax = area.Ymin + (row2+1)*charH;
ProtectRect( &area );
ScrollRect( &r, charW, 0 );
ProtectOff();
}

/*--------------------------------------------------------------------*/
/*  Delete a column from row1 to row2 in edit display...              */
/*--------------------------------------------------------------------*/

static void DelDCol( int col, int row1, int row2 )
{
rect r;
rect area;

DupRect( &DisplayFm->ob_lst->area, &area );
r.Xmin = area.Xmin + (col+1)*charW + 1;
r.Xmax = area.Xmax - 1;
r.Ymin = area.Ymin + row1*charH + 1;
r.Ymax = area.Ymin + (row2+1)*charH;
ProtectRect( &area );
ScrollRect( &r, -charW, 0 );
ProtectOff();
}




/*--------------------------------------------------------------------*/
// Scroll the display left or right
/*--------------------------------------------------------------------*/
static int ScrollOneColumn(int direction)
{
ob_desc * hKnob = (ob_desc *) &DisplayFm->ob_lst[Rows+2];

// Get the slot at the top (or bottom) of the screen
switch (direction)
	{
	case DIR_LEFT:
		// Are we already at the Left of form??
		if (LeftCol == 0)
			return -1;

		LeftCol--;
		InsDCol( 0, 0, Rows-1 );
		break;

	case DIR_RIGHT:
		// Are we already at the Right of form??
		if (LeftCol > MAX_COLS - Cols)
			return -1;
		break;
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Task to print the file 
/*----------------------------------------------------------------------------*/
static int PrintTask(ob_desc * ob)
{
return print_file("PRN");
}



/*----------------------------------------------------------------------------*/
// print the file 
/*----------------------------------------------------------------------------*/
static int print_file(char * dst_file)
{
long count = 0L;								/* records actually read */
double pct = 0.0;								/* per cent copmlete during read */
fm_desc *ind;										/* indicator guage */
static char title[80];					/* indicator form title */
app_msg * aM;										/* a Menuet message */
int handle;                     /* file handle */
int ch;                         /* the character to be copied */
FILE * dst_fp;                  /* Destination file stream */

if (PRSTATUS(0) != PR_OK)
	{
	NotifyUser(-1, -1, "Print File", "Printer Not Ready");
	return FALSE;
	}

if ((stricmp(DisplayFilename, dst_file)) == 0)
	{
	return FALSE;
	}

/* Open Destination File */
if ((dst_fp = fopen(dst_file, "w")) == NULL)
	{
	NotifyUser(-1, -1, dst_file, "Can't Open");
	return FALSE;
	}

/* set up the indicator guage */
sprintf(title, "Processing %s", DisplayFilename);
ind = OpenIndicator(-1, -1, title, "50:0:100");

// rewind the source file
rewind (DisplayFP);

/* get source file handle */
handle = fileno(DisplayFP);

while (count++ <= DisplayFileSize && ch != EOF)
	{
	/* get a char and write it */
	if ((ch = fgetc(DisplayFP)) != EOF) {
		fputc(ch, dst_fp);
		fflush( dst_fp );	/* fix extra page prob */
	}

	/* Has user done anything liked clicked or hit a key */
	if (MsgInQueue())
		{
		aM = ReadMsgQueue();
		if ((isEscapeKey(aM)) || (isMsPress(aM)))
			{
			if (!QueryUser(-1, -1, "Yo !", "Do You Want To Stop?"))
				{
				CloseIndicator(ind);
				fclose(dst_fp);
				return FALSE;
				}
			}
		}			/* end of if (MsgIn... */

		/* Update the indicator guage */
		if (count % 256 == 0)
			{
			pct = 100.0 * ( (float) count / (float) DisplayFileSize);
			UpdateIndicator(ind, (int) pct);
			}

	}  /* end of while */

eject_printer_page();
fclose(dst_fp);
CloseIndicator(ind);
}


/*-----------------------------------------------------------------------*/
// write a form feed to the printer
/*-----------------------------------------------------------------------*/
static void eject_printer_page(void)
{
fputc(FORM_FEED, stdprn); 
fflush(stdprn);
}
