/*----------------------------------------------------------------------------*/
// File: ec_comm.c   USGS Navigation, Essential Communications functions
// Blame: John Gann
// Start Date: 6 July 1992
/*----------------------------------------------------------------------------*/
#include <io.h>
#include <time.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

/* MetaGraphics Headers */
#include <GRCONST.H>
#include <GRPORTS.H>
#include <GREXTRN.H>

/* Menuet Header */
#include <MENUET.H>

/* ECL Serial Driver File */
#include <XC.H>

/* Essential Communications Stuff */
#include <ECLPROTO.H>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"

#define TICKS_PER_SECOND 18
#define MAX_PC_PORTS      2
#define EC_BAD_INSTALL    -101

static int ecInstalled = FALSE;
static int comPorts[MAX_PC_PORTS];
extern char _SmartPortInstalled;

long xc_test();
/*----------------------------------------------------------------------------*/
// write a character to the port
// returns success or -1 if timeout
/*----------------------------------------------------------------------------*/
int ecPutChar ( int port, unsigned char c )
{
int retv;

/* Send byte to port */
retv = xc_putc ( port, c );
return (retv == 0) ? SUCCESS : -1;
}
/*----------------------------------------------------------------------------*/
// read a character to the port
// returns success or -1 if timeout
/*----------------------------------------------------------------------------*/
int ecGetChar (int port)
{
int to;
unsigned char c;

to = ( int )( TICKS_PER_SECOND*5);

/* Set the timeout */
xctp_set ( port, to );

// While not timed out check for byte, if one is there then grab it and go .

while ( xctp_test ( port ) )
	{
	if ( xc_test   ( port ) ) 
		{
		c = ( unsigned char ) xc_getc ( port );
		return (int) c;
		}
	}

/* Must have timed out */
return  -1;
}




/*----------------------------------------------------------------------------*/
// Initialize Essential comms lib
/*----------------------------------------------------------------------------*/
int ecInit(void)
{
int retv;

/* Initialize ECL with 'size' kilobytes of buffer space */
retv = xc_entr ( 4 );

switch(retv)
	{
	case 0: 
		ecInstalled = TRUE;
		break;

	case -2:
		printf("Warning: Comms Kernal Already installed, removing...\n");
		xc_exit();
		if (xc_entr(4) != 0)
			{
			printf("Unable to Install Communications Kernal\n");
			return EC_BAD_INSTALL;
			}
		break;

	default:
		ecInstalled = TRUE;
		if (retv > 0)
			printf(
        "Warning: Requested %d but could only allocate %d receive buffers\n",
         4, retv);
		break;
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// disable interrupts at the given port
/*----------------------------------------------------------------------------*/
void ecDisablePort(int portnum)
{
if (portnum < MAX_PC_PORTS)
	{
	if (comPorts[portnum])
		{
   	xc_unlk (portnum);
		comPorts[portnum] = FALSE;
		}
	}
}


/*----------------------------------------------------------------------------*/
// return the parity in EC format
/*----------------------------------------------------------------------------*/
int ecParity(unsigned char parity)
{

switch(parity)
	{
	case 'N':
		return NOPAR;
		break;

	case 'E':
		return EVENPAR;
		break;

	case 'O':
		return ODDPAR;
		break;
	}

return NOPAR;
}


/*----------------------------------------------------------------------------*/
// return the stop bits in EC format
/*----------------------------------------------------------------------------*/
int ecStopBits(unsigned char bits)
{
return (bits == '1') ? STOP1 : STOP2;
}

/*----------------------------------------------------------------------------*/
// return the data bits in EC format
/*----------------------------------------------------------------------------*/
int ecDataBits(unsigned char bits)
{
return (bits == '7') ? DATA7 : DATA8;
}

/*----------------------------------------------------------------------------*/
// return the baud in EC format
/*----------------------------------------------------------------------------*/
int ecBaud(unsigned int baud)
{
switch(baud)
	{
	case 110:
		return BAUD110;
		break;

	case 300:
		return BAUD300;
		break;

	case 600:
		return BAUD600;
		break;
	
	case 1200:
		return BAUD1200;
		break;

	case 2400:
		return BAUD2400;
		break;

	case 4800:
		return BAUD4800;
		break;

	case 9600:
		return BAUD9600;
		break;
	}

return -1;
}




/*----------------------------------------------------------------------------*/
// initialize Essential Comm port with passed baud and parameters
/*----------------------------------------------------------------------------*/
int ecSetBaud(int port, 
              unsigned int uBaud, 
              unsigned char uParity, 
              unsigned char uData, 
              unsigned char uStop)
{
int baud   = ecBaud(uBaud);
int parity = ecParity(uParity);
int stop   = ecStopBits(uStop);
int data   = ecDataBits(uData);
char buf[128];
int retv;

// don't try to init ports greater than two
if (port > 1)
	return -1;

/* Enable the port interrupts, no line/modem controls */
retv = xc_link ( port, 0 );

if ( retv == -1 ) 
	{
	sprintf(buf, "%s:%d Interrupts already enabled\n", __FILE__, __LINE__);
	logMsg(buf);
	}
     
/* Setup the port parameters */
xc_init ( port, baud, parity, data, stop );
comPorts[port] = TRUE;

/* Enable timer handler */
retv = xctp_link ( port );
if (retv != 1) 
	{
	sprintf(buf, "%s:%d timers are already active\n", __FILE__, __LINE__);
	logMsg(buf);
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// return the number of bytes waiting in input Queue
/*----------------------------------------------------------------------------*/
unsigned int ecInputQueue(int port)
{
return (unsigned int) xc_test(port);
}

/*----------------------------------------------------------------------------*/
// Close the essentil comm functions
/*----------------------------------------------------------------------------*/
void ecTermComm(void)
{
int i;
int ecActive = FALSE;

for (i=0; i<MAX_PC_PORTS; i++)
	{
	if (comPorts[i])
		{
		// Disable the timer handler 
		xctp_unlk ( i );

 		// Disable interrupts 
		xc_unlk ( i );

		// Set flag
		ecActive = TRUE;

		// reset port flag
		comPorts[i] = FALSE;
		}
	}

// Terminate ECL operation 
if (ecInstalled)
	{
	xc_exit ();
	ecInstalled = FALSE;
	}

return;
}


/*----------------------------------------------------------------------------*/
// Set control lines high
/*----------------------------------------------------------------------------*/
void setControlLines(int port, int rts, int dtr)
{
if (_SmartPortInstalled)
	{
	return;
	}
else
	{
	xc_rts(port, rts);
	xc_dtr(port, dtr);
	}
}


