/*----------------------------------------------------------------------------*/
// File: lines.c   USGS Navigation, allow user to manipulate survey lines
// Author: John Gann
// Start Date: 6 Dec 1991
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "dispasci.h"
#include "nav.h"                 // Prototypes
#include "geo_utils.h"           // Prototypes
#include "lines.h"               // Prototypes
#include "nav_util.h"            // Prototypes
#include "sensor.h"              // Prototypes
#include "mnutils.h"             // Prototypes
#include "printer.h"             // Prototypes

// Head of line doubly linked list
static t_line * lineHead = (t_line *) NULL;
static t_line * lastLine = (t_line *) NULL;
static fm_desc * lineFm = (fm_desc *) NULL;
static char * CANT_OPEN_MSG = "Can't Open";

// externals
extern char RefreshRequired;
//extern int sp_to_ll(double y, double x, double * y_rads, double * x_rads);

/*------------------------------------------------------------------*/
// the Add Lines function
/*------------------------------------------------------------------*/
int doAddLine(ob_desc * ob)
{
int retv;

// load the line file into memory if it exists
load_linefile();

retv = doLineForm(ob, lastLine, NULL, ADD);

// Save the lines
write_line_file();

return retv;
}


/*------------------------------------------------------------------*/
// the Display Lines function
/*------------------------------------------------------------------*/
int doDisplayLines(ob_desc * ob)
{
// load the line file into memory if it exists and has not yet been loaded
load_linefile();

return pickLines(ob, "Select Lines for Display", DISPLAY);
}


/*------------------------------------------------------------------*/
// the Delete Lines function
/*------------------------------------------------------------------*/
int doDeleteLine(ob_desc * ob)
{
// load the line file into memory if it exists and has not yet been loaded
load_linefile();

return pickLines(ob, "Select Lines for Deletion", DELETE);
}


/*------------------------------------------------------------------*/
// the Print Lines function
/*------------------------------------------------------------------*/
int doListLines(ob_desc * ob)
{
char tmpname[_MAX_PATH] = {"nvXXXXXX"};
t_line * aLine = lineHead;
FILE * fp;

// Create a temp file name to store the output to.
if (mktemp(tmpname) == NULL)
	{
	return UNABLE_TO_CREATE_TMPFILE;
	}
else
	{
	if ((fp = fopen(tmpname, "w")) == (FILE *) NULL)
		{
		return BAD_FILE_OPEN;
		}
	}

// load the line file into memory if it exists and has not yet been loaded
load_linefile();

while (aLine != (t_line *) NULL)
	{
	printLine(fp, aLine);
	aLine = aLine->next;
	}
	
fclose(fp);

// get a temp file name
DisplayAsciiFile( tmpname, 1, 32, 10, 50);

// get rid of temp file
remove(tmpname);

return SUCCESS;
}


/*------------------------------------------------------------------*/
// function to load the binary linefile
/*------------------------------------------------------------------*/
int load_linefile(void)
{
FILE * fp;
t_line aLine;
int retv;

// if the lines have already been read in, return
if (lineHead != (t_line *) NULL)
	return LINES_ALREADY_LOADED;

// attempt to open the linefile
if ((fp = fopen(LINEFILE, "r+b")) == (FILE *) NULL)
	{
	return BAD_FILE_OPEN;
	}
// scrollMsg("Opening Linefile\n");

// read the file in mallocing as you go
while((retv = getNextLine(fp, &aLine, sizeof(t_line))) == SUCCESS)
	{
	// add the line to the linked list
	if ((retv = addLineItem(&aLine)) != SUCCESS)
		{
		fclose(fp);
		printf("addLineItem() failed with %d\n", retv);
		return retv;
		}
	}

fclose(fp);
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Add an item to the end of the doubly linked linefile list
// return the pointer to the next node if successful, else return NULL
/*----------------------------------------------------------------------------*/
int addLineItem(t_line * newLine)
{
t_line * last = getLastLine(); 
t_line * new;

if (newLine == (t_line *) NULL)
	{
	return NULL_LINE;
	}

if (last == (t_line *) NULL)            // is the list empty
	{
	if ((last = (t_line *) malloc(sizeof(t_line))) == (t_line *) NULL)
		{
		printf("malloc failed in %s at line %d\n", __FILE__, __LINE__);
		return BAD_MALLOC;
		}
	newLine->previous = (t_line *) NULL;
	newLine->next = (t_line *) NULL;
	memcpy((void *) last, (void *) newLine, sizeof(t_line));
	lineHead = last;
	lastLine = last;
	}
else
	{
	if ((last->next = (t_line *) malloc(sizeof(t_line))) == (t_line *) NULL)
		{
		printf("malloc failed in %s at line %d\n", __FILE__, __LINE__);
		return BAD_MALLOC;
		}

	memcpy((void *) last->next, (void *) newLine, sizeof(t_line));
	new = last->next;
	new->previous = last;
	new->next = (t_line *) NULL;

	lastLine = last->next;
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Retun the number of lines in the list
/*----------------------------------------------------------------------------*/
int getLineCount(void)
{
t_line * aLine = lineHead;
int i=0;
// scrollMsg("getLineCount()\n");

while (aLine != (t_line *) NULL)
	{
	aLine = aLine->next;
	i++;
	}

return i;
}



/*----------------------------------------------------------------------------*/
// Return the Line in the linked list or that matches the name
/*----------------------------------------------------------------------------*/
t_line * getNamedLine(char * name)
{
t_line * aLine = lineHead;

while (aLine != (t_line *) NULL)
	{
	if ((strcmp(aLine->name, name)) == SUCCESS)
		return aLine;

	aLine = aLine->next;
	}

return (t_line *) NULL;
}


/*----------------------------------------------------------------------------*/
// Retun the Nth Line in the linked list or Null if not found
/*----------------------------------------------------------------------------*/
t_line * getNthLine(int n)
{
t_line * aLine = lineHead;
int i=0;

// scrollMsg("getNthLine()\n");
while (aLine != (t_line *) NULL)
	{
	if (i == n)
		return aLine;

	aLine = aLine->next;
	i++;
	}

return (t_line *) NULL;
}




/*----------------------------------------------------------------------------*/
// print a line
/*----------------------------------------------------------------------------*/
void printLine(FILE * fp, t_line * aLine)
{
char x_buf[40];
char y_buf[40];
char z_buf[40];

// sanity checks
if (aLine == (t_line *) NULL)
	return;

// sanity checks
if (fp == (FILE *) NULL)
	return;

PositionString(x_buf, y_buf, z_buf, aLine->start.lon, aLine->start.lat, 0.0, 0);
fprintf(fp, "%s\n Starts: %-18s %-18s\n", aLine->name, y_buf, x_buf);
PositionString(x_buf, y_buf, z_buf, aLine->end.lon, aLine->end.lat, 0.0, 0);
fprintf(fp, "   Ends: %-18s %-18s\n", y_buf, x_buf);
fprintf(fp, "  Range: %.1lf\nBearing %.1lf\n\n", aLine->range, RAD_TO_DEG * aLine->bearing);
}


/*----------------------------------------------------------------------------*/
// print Header for Line list
/*----------------------------------------------------------------------------*/
void printLineHeader(void)
{
fprintf(stdprn, "%c%-25s %-20s %-20s %-20s %-20s\n%s\n", FF, "Line Name", "Starting Latitude", "Starting Longitude", "Ending Latitude", "Ending Longitude", "________________________________________________________________________________");
}


/*----------------------------------------------------------------------------*/
// Delete the Nth line from the linked list
/*----------------------------------------------------------------------------*/
int deleteLineList(char marks[])
{
t_line * aLine = lineHead;
int i=0;
int max_lines = getLineCount();
t_line * next;

for (i=0; i<max_lines; i++)
	{
	next = aLine->next;
	if (marks[i] == (char) '+')
		{
		deleteLine(aLine);
		}

	aLine = next;
	}

write_line_file();
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Delete the passed t_line *
/*----------------------------------------------------------------------------*/
int deleteLine(t_line * aLine)
{
t_line * previous = aLine->previous;
t_line * next = aLine->next;

if (lineHead == (t_line *) NULL)
	{
	return FALSE;
	}

if (aLine == (t_line *) NULL)
	{
	return FALSE;
	}

if (aLine == lineHead)
	{
	if (lineHead->next == (t_line *) NULL)
		// there are no more lines after this one
		{
		free(aLine);
		lineHead = (t_line *) NULL;
		lastLine = (t_line *) NULL;
		aLine = (t_line *) NULL;
		return TRUE;
		}
	lineHead = next;
	lineHead->previous = (t_line *) NULL;
	free(aLine);
	return TRUE;
	}

if (aLine == lastLine)
	{
	if (lastLine->previous == (t_line *) NULL)
		{
		// there are no more lines after this one
		free(aLine);
		lineHead = (t_line *) NULL;
		lastLine = (t_line *) NULL;
		aLine = (t_line *) NULL;
		return TRUE;
		}
	free(aLine);
	lastLine = previous;
	lastLine->next = (t_line *) NULL;
	return TRUE;
	}

// This line is neither the first or the last
aLine->previous->next = next;
aLine->next->previous = previous;
free(aLine);

return TRUE;
}


/*----------------------------------------------------------------------------*/
// return the last item in the doubly linked line list
// if there are no lines return NULL                -- empty list
// if there is one line return lineHead             -- one item
// if there is more than line return lastLine       -- several items
/*----------------------------------------------------------------------------*/
t_line * getLastLine( void )
{
if (lastLine == (t_line *) NULL)
	{
	return lineHead;            // either one or none lines in linked list
	}

return lastLine;              // must be greater han one line in list
}


/*---------------------------------------------------------------------------*/
// The Line Form
/*---------------------------------------------------------------------------*/
int doLineForm(ob_desc * ob, t_line * aLine, char * title, char mode)
{
static int idx=0;
int retv;
int retval;
char buf[128];
char x_buf_1[30];         /* String for Starting Lon or Easting */
char y_buf_1[30];         /* String for Starting Lat or Northing */
char x_buf_2[30];         /* String for Ending Lon or Easting */
char y_buf_2[30];         /* String for Ending Lat or Northing */
int CoordinateSystem =  QuerySetup()->CoordSys;     
int HorizUnits =        QuerySetup()->HorizUnits;  
int AngleMode =         QuerySetup()->AngleType;  
int DecimalAccuracy =   QuerySetup()->DecimalAccuracy;
t_ellips * elps = query_ellipsoid();
int zone_num;
char zone_ltr;
ftag * tag;
t_line newLine = {NULL, "New", 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, NULL}; 
double back_azm;
t_setup * config = QuerySetup();

if ( ( lineFm = loadFRF ( "add_line.frf" ) ) == (fm_desc *) NULL ) 
	return FALSE;

// Set up a task for the flip button
lineFm->ob_lst[HELP].flags |= IS_EXECUTABLE;
lineFm->ob_lst[HELP].task   = flipLineTask;

if (title == (char *) NULL)
	{
	strcpy(lineFm->title, "Line Entry Form");
	}
else
	{
	strcpy(lineFm->title, title);
	}

// if passed a NULL line, then just use a default line
if (aLine == (t_line *) NULL)
	{
	aLine = (t_line *) &newLine;
	}

zone_num = grid_zone_EW(aLine->start.lon);
zone_ltr = grid_zone_NS(aLine->start.lat);

if (!OpenForm( lineFm, 10, ob->area.Ymax))
	{
	sprintf(buf, "Failed to open form in %s at line %d\n", __FILE__, __LINE__);
	Puke(buf);
	}

// Initialize the form with the passed line
UpdatePositionEntryFields(lineFm, STARTING, aLine->start.lon, aLine->start.lat); 
UpdatePositionEntryFields(lineFm, ENDING, aLine->end.lon, aLine->end.lat); 

tag = (ftag *) lineFm->ob_lst[4].tag;
strcpy(tag->fld, aLine->name);
DrawTaggedOb(&lineFm->ob_lst[4]); 

tag = (ftag *) lineFm->ob_lst[9].tag;
sprintf(tag->fld, "%7.1lf", aLine->range);
DrawTaggedOb(&lineFm->ob_lst[9]); 

tag = (ftag *) lineFm->ob_lst[10].tag;
sprintf(tag->fld, "%5.1lf", aLine->bearing * RAD_TO_DEG);
DrawTaggedOb(&lineFm->ob_lst[10]); 

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( lineFm->ob_lst, lineFm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY : 
			/* See if all of the fields are valid */
			tag = (ftag *) lineFm->ob_lst[5].tag;
			strcpy(y_buf_1, tag->fld);
			tag = (ftag *) lineFm->ob_lst[6].tag;
			strcpy(x_buf_1, tag->fld);
			tag = (ftag *) lineFm->ob_lst[7].tag;
			strcpy(y_buf_2, tag->fld);
			tag = (ftag *) lineFm->ob_lst[8].tag;
			strcpy(x_buf_2, tag->fld);
			if (PositionXYToRads(y_buf_1, x_buf_1, 
                           &newLine.start.lat, &newLine.start.lon, 
                           zone_num, zone_ltr))
				{
				retv = 99;        /* Stay in LOOP */
				break;
				}

			if (PositionXYToRads(y_buf_2, x_buf_2, 
                           &newLine.end.lat, &newLine.end.lon, 
                           zone_num, zone_ltr))
				{
				retv = 99;        /* Stay in LOOP */
				break;
				}

			// Get the line name
			tag = (ftag *) lineFm->ob_lst[4].tag;
			strcpy(newLine.name, tag->fld);

			newLine.display = TRUE;

			// compute the range and bearing 
			arclen(newLine.start.lat, newLine.start.lon, 
             newLine.end.lat, newLine.end.lon, 
					   &newLine.range, &newLine.bearing, &back_azm, 
					   elps->a, elps->b);

			if (newLine.bearing < 0.0)
				newLine.bearing += TWOPI;

			switch (mode)
				{
				case ADD:
					// add the Line to the Linked list
					if ((retval = addLineItem(&newLine)) != SUCCESS)
						{
						printf("addLineItem() failed with %d\n", retval);
						}
					break;

				case EDIT:
					// if they are editing current line update that too
					if (!memcmp((void *) &config->curLine.name, 
                      (void *) &aLine->name, sizeof(aLine->name)))
						{
						memcpy((void *) &config->curLine, 
                 (void *) &newLine, sizeof(t_line));
						NotifyUser(-1, -1, newLine.name, "Edited currently selected line");
						}

					// change the Line in the Linked list
					memcpy((void *) aLine->name,                 // update the name
                 (void *) &newLine.name, 
                 sizeof(aLine->name));
					memcpy((void *) &aLine->start,               // update starting pos
                 (void *) &newLine.start, 
                 sizeof(t_pos));
					memcpy((void *) &aLine->end,                 // update ending pos
                 (void *) &newLine.end, 
                 sizeof(t_pos));

					aLine->num = newLine.num;                    // update the line num
					aLine->range = newLine.range;                // update line length
					aLine->bearing = newLine.bearing;            // update line bearing

					// dumpLine(aLine);
					// dumpLine(&newLine);
					break;
				}

			break;

		case DEFAULT : 
			UpdatePositionEntryFields(lineFm, STARTING, 
                                    aLine->start.lon, aLine->start.lat); 
			UpdatePositionEntryFields(lineFm, ENDING,  
                                    aLine->end.lon, aLine->end.lat); 

			
			// update the line name
			tag = (ftag *) lineFm->ob_lst[4].tag;
			strcpy(tag->fld, aLine->name);
			DrawTaggedOb(&lineFm->ob_lst[4]); 

			tag = (ftag *) lineFm->ob_lst[9].tag;
			sprintf(tag->fld, "%7.1lf", aLine->range);
			DrawTaggedOb(&lineFm->ob_lst[9]); 

			tag = (ftag *) lineFm->ob_lst[10].tag;
			sprintf(tag->fld, "%5.1lf", aLine->bearing * RAD_TO_DEG);
			DrawTaggedOb(&lineFm->ob_lst[10]); 

			break;

		case CANCEL : 
			break;

		case HELP : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL);


CloseForm ( lineFm );
DestroyFRFForm(lineFm);
RefreshRequired = (char) (retv == OKAY);
return  (retv == OKAY);
}





/*----------------------------------------------------------------------------*/
// This func gets called when any position form field is exited
/*----------------------------------------------------------------------------*/
int PositionExitTask( ob_desc *ob )
{
ftag *tag = (ftag *) ob->tag;
double rads;
char work[_MAX_PATH];
int CoordinateSystem =  QuerySetup()->CoordSys;     

switch(CoordinateSystem)
	{
	case LATITUDE_LONGITUDE:
		strcpy(work, tag->fld);
		switch(ob->key)
			{
			case '1':
			case '3':
				if (PositionStrToRads(work, &rads, LAT))
					{
					LatString(work, rads);
					}
				else
					{
					return USER_CONTINUE;
					}
				strcpy(tag->fld, work);
				break;
	
			case '2': /* Longitude */
			case '4': /* Longitude */
				if (PositionStrToRads(work, &rads, LON))
					{
					LonString(work, rads);
					}
				else
					{
					return USER_CONTINUE;
					}
				strcpy(tag->fld, work);
				break;
			}  /* end of inner switch */
		break;

	case UNIVERSAL_TRANSVERSE_MERCATOR:
		break;

	case STATE_PLANE:
		break;
	}  /* end of outter switch  */

return USER_CONTINUE;
}



/*---------------------------------------------------------------------------*/
/* Update the coordinate fields on the Position entry form                    */
// Offset: 0=starting coordinates, 1=ending coordinates
/*---------------------------------------------------------------------------*/
void UpdatePositionEntryFields(fm_desc * fm, int offset,
                               double new_x, double new_y)
{
char x_buf[30];         /* String for Lon or Easting */
char y_buf[30];         /* String for Lat or Northing */
char z_buf[30];
ftag * tag;

PositionString(x_buf, y_buf, z_buf, new_x, new_y, 0.0, 0);
			
tag = (ftag *) fm->ob_lst[5 + offset].tag;          /* Copy Y Value to form */
strcpy(tag->fld, y_buf);
	
tag = (ftag *) fm->ob_lst[6 + offset].tag;          /* Copy X Value to form */
strcpy(tag->fld, x_buf);
	
DrawTaggedOb(&fm->ob_lst[5 + offset]);      /* x */
DrawTaggedOb(&fm->ob_lst[6 + offset]);      /* y */
}





/*----------------------------------------------------------------------------*/
// Save the line file 
/*----------------------------------------------------------------------------*/
int write_line_file(void)
{
FILE *fp;
int  k;
char buf[80];
t_line * aLine = lineHead;

if ((fp = fopen(LINEFILE, "wb")) == NULL)
  {
  printf("\nCan't open file %s", LINEFILE);
  return FALSE;
  }

while (aLine != (t_line *) NULL)
	{
	k = fwrite( (void *) aLine, sizeof(t_line), 1, fp);
	if (k == -1)
		{
		sprintf(buf, "Write failed %d in %s at line %d\n", k, __FILE__, __LINE__);
		Puke(buf);
		}
	aLine = aLine->next;
	}

fclose(fp);
return TRUE;
}





/*------------------------------------------------------------------*/
// the Select Line for nav function
/*------------------------------------------------------------------*/
int doSelectLine(ob_desc * ob)
{
// load the line file into memory if it exists and has not yet been loaded
load_linefile();

return pickLine(ob, "Select Line For Steering", SELECT);
}



/*------------------------------------------------------------------*/
// the Edit Lines function
/*------------------------------------------------------------------*/
int doEditLine(ob_desc * ob)
{
int retv;

// load the line file into memory if it exists and has not yet been loaded
load_linefile();

retv = pickLine(ob, "Pick a Line to Edit", EDIT);

// Save the lines
write_line_file();

return retv;
}


/*------------------------------------------------------------------*/
// Pick a single Line from a list box
/*------------------------------------------------------------------*/
int pickLine(ob_desc * ob, char * title, char mode)
{
fm_desc *lf;
int max_length = 0, i, j;
char *names [ MAX_LINES ];
int cur_slot, top_slot, retv, count = 0;
t_line * aLine;
static int last_slot = 0;                   // last recently used slot
char buf[512];
t_setup * config = QuerySetup();

/* load the line names */
if ((count = loadLineNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, "Yo", "No Lines in the File." );
   return -1;
   }

/* create a list form */
lf = CreateListForm ( title, 7, 4, max_length+2, 8, 0 );

lf->ob_lst [  9 ].label  = "Okay";
lf->ob_lst [  9 ].flags |= IS_EXITABLE;
lf->ob_lst [  9 ].key    = 'O';
lf->ob_lst [  9 ].state  = 8;
   
lf->ob_lst [ 10 ].label  = "Default";
lf->ob_lst [ 10 ].flags |= IS_EXITABLE;
lf->ob_lst [ 10 ].key    = 'D';
lf->ob_lst [ 10 ].state  = 8;
   
lf->ob_lst [ 11 ].label  = "Cancel";
lf->ob_lst [ 11 ].flags |= IS_EXITABLE;
lf->ob_lst [ 11 ].key    = 'C';
lf->ob_lst [ 11 ].state  = 8;
   
lf->ob_lst [ 12 ].label  = "Help";
lf->ob_lst [ 12 ].flags |= IS_EXITABLE;
lf->ob_lst [ 12 ].key    = 'H';
lf->ob_lst [ 12 ].state  = 8;

cur_slot = top_slot = (last_slot < count && last_slot >= 0) ? last_slot : 0 ;

/* open a form to put the list on */
OpenForm( lf, ob->area.Xmin-40, ob->area.Ymax);

do {
retv = ProcessListForm(lf, names, count, &top_slot, &cur_slot );

switch ( retv ) 
	{
	case 9 :        // Okay
		last_slot = cur_slot;
		aLine = getNthLine(cur_slot);
		if (aLine == (t_line *) NULL)
			{
			NotifyUser(-1, -1, "Error", "Unable to match line\n");
			}
		else
			{
			switch (mode)
				{
				case EDIT:
					sprintf(buf, "Edit Line %d of %d\n", cur_slot+1, getLineCount());
					if (doLineForm(ob, aLine, buf, EDIT))
						{
						loadLineNames(names, &max_length);
						if ((count = loadLineNames(names, &max_length)) <= 0)
   						{
   						NotifyUser(-1, -1, "Yo", "No Lines in the File." );
   						}
						else
							{
							DrawForm(lf);
							}
						}
					break;

				case SELECT:
					// Store the current line in the configuration file
					memcpy((void *) &config->curLine, 
                 (void *) aLine, sizeof(t_line));
					queryTrackInfo()->initialized = FALSE;
					j = sprintf(buf, "Selected New Line\n");
					dumpLine(&config->curLine, buf+j);
					printMsg(buf);
					logMsg(buf);
					break;
				}   // end of switch

			RefreshRequired = TRUE;
			resetTrack(aLine);
			}
		break;
   
	case 10:        // Default
		break;

	case 11 :       // Cancel
		break;
   
	case 12 :       // Help
		break; 
	}
} while ( (retv != NOT_SET) && ( retv != 11 ) && (retv != 9) );

// clean up alloced memory
i = count;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );


if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}




/*------------------------------------------------------------------*/
// Pick several lines from a checklist
/*------------------------------------------------------------------*/
int pickLines(ob_desc * ob, char * title, char mode)
{
fm_desc *lf;
int max_length = 0, i;
char *names [ MAX_LINES ];
int cur_slot, top_slot, retv, count = 0;
char marks[MAX_LINES];

memset((void *) marks, ' ', sizeof(marks));

if ((count = loadLineNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, "Yo", "No Lines in the File." );
   return -1;
   }

if (mode == DISPLAY)
	{
	lineDisplayFlags(marks, GET);
	}

/* create a list form */
lf = CreateChkListForm( title, 7, 4, max_length+2, 7);
lf->ob_lst[9].label = "Okay";
lf->ob_lst[9].flags |= IS_EXITABLE;
lf->ob_lst[9].key = 'O';
lf->ob_lst[9].state = 8;

lf->ob_lst[10].label = "All";
lf->ob_lst[10].flags |= IS_EXITABLE;
lf->ob_lst[10].key = 'A';
lf->ob_lst[10].state = 8;

lf->ob_lst[11].label = "None";
lf->ob_lst[11].flags |= IS_EXITABLE;
lf->ob_lst[11].key = 'N';
lf->ob_lst[11].state = 8;

lf->ob_lst[12].label = "Cancel";
lf->ob_lst[12].flags |= IS_EXITABLE;
lf->ob_lst[12].key = 'C';
lf->ob_lst[12].state = 8;

cur_slot = top_slot = 0;

/* open a form to put the list on */
OpenForm( lf, ob->area.Xmin-40, ob->area.Ymax);

do 
	{
	retv = ProcessChkListForm( lf, names, marks, count, &top_slot);
	switch ( retv ) 
		{
		case 9 :        // Okay
			switch (mode)
				{
				case DISPLAY:
					RefreshRequired = TRUE;
					lineDisplayFlags(marks, SET);
					break;

				case DELETE:
					RefreshRequired = TRUE;
					deleteLineList(marks);
					break;
				}
			break;
   
		case 10:        // All On
			for (i=0; i< count; i++)
				marks[i] = '+';
			top_slot = 0;
			DrawForm(lf);
			break;
	
		case 11 :       // All Off
			for (i=0; i< count; i++)
				marks[i] = ' ';
			top_slot = 0;
			DrawForm(lf);
			break;
  	 
		case 12 :       // Cancel
			break; 
		}
	} while ( (retv != 9) && (retv != NOT_SET) && ( retv != 12 ) );

// clean up alloced memory
i = count;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );

if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}


/*----------------------------------------------------------------------------*/
// Load the line names from the linked list
/*----------------------------------------------------------------------------*/
int loadLineNames(char * names[], int * max_length)
{
int i = 0;
int count = 0;
t_line * aLine;

while(count < MAX_LINES && (aLine = getNthLine(count)))
   {
   names [ count ] = ( char * ) malloc ( strlen ( aLine->name )+1 );

   if ( names [ count ] == (char *) NULL ) 
      {
      i = count;
      while ( i ) 
				free ( names [ --i ] );
      NotifyUser(-1, -1, "ERROR", "Not Enough Memory." );
      return -1;
      }
   else 
      {
      strcpy ( names [ count++ ], aLine->name );
      *max_length = max ( *max_length, (int) strlen ( aLine->name ) );
      }
	}  /* end of while */
return count;
}



/*----------------------------------------------------------------------------*/
// display the passed line
/*----------------------------------------------------------------------------*/
char * dumpLine(t_line * aLine, char * buf)
{
int j = 0;
char latBuf[40];
char lonBuf[40];
char altBuf[40];

if (aLine == (t_line *) NULL)
	{
	j += sprintf(buf+j, "Null Line");
	}
else
	{
	j += sprintf(buf+j, "name   : %s\n", aLine->name);
	j += sprintf(buf+j, "num    : %d\n", aLine->num);
	PositionString(lonBuf, latBuf, altBuf, 
                 aLine->start.lon, 
                 aLine->start.lat, 0.0, 0);
	j += sprintf(buf+j, "Starts : %s %s\n", latBuf, lonBuf);
	PositionString(lonBuf, latBuf, altBuf, 
                 aLine->end.lon, 
                 aLine->end.lat, 0.0, 0);
	j += sprintf(buf+j, "Stops  : %s %s\n", latBuf, lonBuf);
	j += sprintf(buf+j, "Range  : %.2lf\n", aLine->range);
	j += sprintf(buf+j, "Bearing: %.2lf\n", aLine->bearing * RAD_TO_DEG);
	}

return buf;
}




/*----------------------------------------------------------------------------*/
// do line generate form
/*----------------------------------------------------------------------------*/
int doLineGeneratorFm ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "line_gen.frf";
ftag * fromLatTag;
ftag * fromLonTag;
ftag * toLatTag;
ftag * toLonTag;
ftag * spacingTag;
ftag * limitTag;
char buf[80];
static int lastSide = GENERATE_BOTH;
static t_pos lastFrom;
static t_pos lastTo;
static double lastRange = 0.0;
static double lastBearing = 0.0;
static double lastLimit = 0.0;
static double lastSpacing = 0.0;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 
double back_azm;
t_line rootLine;
char tmpname[_MAX_PATH] = {"nvXXXXXX"};
FILE * fp;
int genLineCnt = 0;
t_line genLines[MAX_GEN_LINES];
int i;

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// Clear all buttons
fm->ob_lst [ 7 ].flags &= ~IS_CHECKED;
fm->ob_lst [ 8 ].flags &= ~IS_CHECKED;
fm->ob_lst [ 9 ].flags &= ~IS_CHECKED;

// Set the active one
fm->ob_lst [ 7 + lastSide ].flags |= IS_CHECKED;

OpenForm ( fm, 10, ob->area.Ymin ); 

// set up form
fromLatTag = (ftag *) fm->ob_lst[2].tag;
fromLonTag = (ftag *) fm->ob_lst[3].tag;
toLatTag   = (ftag *) fm->ob_lst[4].tag;
toLonTag   = (ftag *) fm->ob_lst[5].tag;
spacingTag = (ftag *) fm->ob_lst[6].tag;
limitTag   = (ftag *) fm->ob_lst[10].tag;

// update from position
PositionString(fromLonTag->fld, 
               fromLatTag->fld, 
               buf, 
               lastFrom.lon, 
               lastFrom.lat, 
               0.0, 0);

// update to position
PositionString(toLonTag->fld, 
               toLatTag->fld, 
               buf, 
               lastTo.lon, 
               lastTo.lat, 
               0.0, 0);

// clear range and bearing
sprintf(spacingTag->fld, "%.1lf", lastSpacing);
sprintf(limitTag->fld, "%.1lf", lastLimit);

DrawTaggedOb(&fm->ob_lst[2]); 
DrawTaggedOb(&fm->ob_lst[3]); 
DrawTaggedOb(&fm->ob_lst[4]); 
DrawTaggedOb(&fm->ob_lst[5]); 
DrawTaggedOb(&fm->ob_lst[6]); 
DrawTaggedOb(&fm->ob_lst[10]); 

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case CALCULATE:
			genLineCnt = 0;
			lastSpacing = atof(spacingTag->fld);
			lastLimit   = atof(limitTag->fld);
			lastSide    = get_flag_fm_group(fm, 1);
			if ((retv = suckPositions(fm, 2, &lastFrom, &lastTo)) != SUCCESS)
				{
				NotifyUser(-1, -1, "Yo!", "Bad Coordinate Entry");
				retv = 99;
				}
			else
				{
				// compute the range and bearing
				if (arclen(lastFrom.lat, lastFrom.lon, lastTo.lat, lastTo.lon, 
           &lastRange, &lastBearing, &back_azm, 
           ellipse->a, ellipse->b) == SUCCESS)
					{
					memcpy( (void *) &rootLine.start, (void *) &lastFrom, sizeof (t_pos));
					memcpy( (void *) &rootLine.end, (void *) &lastTo, sizeof (t_pos));
					rootLine.bearing = lastBearing;
					rootLine.range   = lastRange;
					strcpy(rootLine.name, "TMP");

					// Generate lines
					genLineCnt = generateLines(&rootLine, genLines, lastSpacing, lastLimit, lastSide);
					sprintf(buf, "Generated %d lines\n", genLineCnt);
					NotifyUser(-1, -1, "Yo", buf);
					}
				else
					{
					NotifyUser(-1, -1, "Yo!", "arclen(): Failed");
					retv = 99;
					}
				}
			break;

		case DONE:
			break;

		case VIEW:
			if (genLineCnt > 0)
				{
				// Create a temp file name to store the output to.
				if (mktemp(tmpname) == NULL)
					{
					break;
					}
				else
					{
					if ((fp = fopen(tmpname, "w")) == (FILE *) NULL)
						{
						NotifyUser(-1, -1, CANT_OPEN_MSG, tmpname);
						break;
						}
					}

				// dump each line out
				for (i=0; i<genLineCnt; i++)
					{
					fprintf(fp, "%s", dumpLine(&genLines[i], buf));
					}
	
				fclose(fp);
			
				// Show It
				DisplayAsciiFile( tmpname, 1, 32, 10, 50);
			
				// get rid of temp file
				remove(tmpname);
				}

			break;

		case ACCEPT:
			// add the line to the linked list
			for (i=0; i<genLineCnt; i++)
				{
				if ((retv = addLineItem(&genLines[i])) != SUCCESS)
					{
					printf("addLineItem() failed with %d\n", retv);
					break;
					}
				}
			// Save the lines
			write_line_file();

			sprintf(buf, "Added %d lines to linefile\n", genLineCnt);
			NotifyUser(-1, -1, "Yo", buf);
			RefreshRequired = TRUE;
			break;

		default : 
			break;
		}
	} while (retv != DONE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


/*----------------------------------------------------------------------------*/
// Generate up to MAX_GEN_LINES according to instructions
/*----------------------------------------------------------------------------*/
int generateLines(t_line * rootLine, t_line genLines[], 
                  double spacing, double limit, int side)
{
int i;
int lineCnt = 0;
int linesToGenerate = 1 + (int) (limit / spacing * ((side == GENERATE_BOTH) ? 2.0 : 1.0));
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 
double bearings[2];
char newName[40];
// char buf[512];
int max_lines = getLineCount();

bearings[GENERATE_LEFT]  = fmod ( rootLine->bearing - HALFPI, TWOPI );
bearings[GENERATE_RIGHT] = fmod ( rootLine->bearing + HALFPI, TWOPI );

// Save with root
memcpy( (void *) &genLines[lineCnt++], (void *) rootLine, sizeof(t_line));

for (i=1; lineCnt < MAX_GEN_LINES && lineCnt < linesToGenerate; i++)
	{
	// Start with root line and work in one direction
	switch(side)
		{
		case GENERATE_LEFT:
			computeRelativeLine(rootLine, &genLines[lineCnt], 
                          spacing * (double) i, 
                          bearings[GENERATE_LEFT]);
			sprintf(newName, "L%02d", max_lines + lineCnt);
			strcpy(genLines[lineCnt].name, newName);
			// logMsg(dumpLine(&genLines[lineCnt], buf));
			lineCnt++;
			break;

		case GENERATE_RIGHT:
			computeRelativeLine(rootLine, &genLines[lineCnt], 
                          spacing * (double) i, 
                          bearings[GENERATE_RIGHT]);
			sprintf(newName, "R%02d", max_lines + lineCnt);
			strcpy(genLines[lineCnt].name, newName);
			// logMsg(dumpLine(&genLines[lineCnt], buf));
			lineCnt++;
			break;

		case GENERATE_BOTH:
			computeRelativeLine(rootLine, &genLines[lineCnt], 
                          spacing * (double) i, 
                          bearings[GENERATE_LEFT]);
			sprintf(newName, "L%02d", max_lines + lineCnt);
			strcpy(genLines[lineCnt].name, newName);
			// logMsg(dumpLine(&genLines[lineCnt], buf));
			lineCnt++;
			computeRelativeLine(rootLine, &genLines[lineCnt], 
                          spacing * (double) i, 
                          bearings[GENERATE_RIGHT]);
			sprintf(newName, "R%02d", max_lines + lineCnt);
			strcpy(genLines[lineCnt].name, newName);
			// logMsg(dumpLine(&genLines[lineCnt], buf));
			lineCnt++;
			break;
		}
	}

return lineCnt;
}



/*----------------------------------------------------------------------------*/
// Build a Parallel line
/*----------------------------------------------------------------------------*/
int computeRelativeLine(t_line * rootLine, t_line * theLine, 
                           double offset, double bearing)
{
t_ellips * ellipse = query_ellipsoid();
int retv;

// make the same to start
memcpy((void *) theLine, (void *) rootLine, sizeof(t_line));

// compute endpoints of new line
if ((retv = proj(rootLine->start.lat, rootLine->start.lon, offset, bearing, 
                &theLine->start.lat, &theLine->start.lon, 
                ellipse->a, ellipse->b)) != SUCCESS)
	{
	return retv;
	}

if ((retv = proj(rootLine->end.lat, rootLine->end.lon, offset, bearing, 
                &theLine->end.lat, &theLine->end.lon, 
                ellipse->a, ellipse->b)) != SUCCESS)
	{
	return retv;
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// This func gets called when the Flip button is pressed
/*----------------------------------------------------------------------------*/
int flipLineTask( ob_desc *ob )
{
ftag *slatTag = (ftag *) lineFm->ob_lst[5].tag;
ftag *slonTag = (ftag *) lineFm->ob_lst[6].tag;
ftag *elatTag = (ftag *) lineFm->ob_lst[7].tag;
ftag *elonTag = (ftag *) lineFm->ob_lst[8].tag;
ftag *rngTag  = (ftag *) lineFm->ob_lst[9].tag;
ftag *brgTag  = (ftag *) lineFm->ob_lst[10].tag;
t_pos from;
t_pos to;
double range, bearing, back_azm;
char buf1[80], buf2[80];
t_ellips * elps = query_ellipsoid();

// save the starting lat/lon
strcpy(buf1, slatTag->fld);
strcpy(buf2, slonTag->fld);

// copy the end lat/lon
strcpy(slatTag->fld, elatTag->fld);
strcpy(slonTag->fld, elonTag->fld);

// copy the start lat/lon
strcpy(elatTag->fld, buf1);
strcpy(elonTag->fld, buf2);

// get the new coords
suckPositions(lineFm, 5, &from, &to);

// compute the reciprocal range and bearing 
arclen(from.lat, from.lon, 
       to.lat, to.lon, 
       &range, &bearing, &back_azm, 
       elps->a, elps->b);

// update the form
sprintf(rngTag->fld, "%7.1lf", range);
sprintf(brgTag->fld, "%5.1lf", bearing * RAD_TO_DEG);

DrawTaggedOb(&lineFm->ob_lst[5]); 
DrawTaggedOb(&lineFm->ob_lst[6]); 
DrawTaggedOb(&lineFm->ob_lst[7]); 
DrawTaggedOb(&lineFm->ob_lst[8]); 
DrawTaggedOb(&lineFm->ob_lst[9]); 
DrawTaggedOb(&lineFm->ob_lst[10]); 

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Load the display flag from the linked list
/*----------------------------------------------------------------------------*/
int lineDisplayFlags(char marks[], int action)
{
int count = 0;
t_line * aLine;

while(count < MAX_LINES && (aLine = getNthLine(count)))
	{
	if (aLine == (t_line *) NULL)
		return count;

	if (action == GET)
		{
		marks [ count++ ] = (char) ((aLine->display) ? '+' : ' ');
		}
	else
		{
		aLine->display = (marks[count++] == '+') ? TRUE: FALSE;
		}

	}

return count;
}


