#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <time.h>
#include <conio.h>

#include "navdefs.h"
#include "navtypes.h"
#include "graph.h"

#define WGS84_DATUM 6

static t_setup setup;

void main(void);
int write_setup(void);

void main(void)
{
write_setup();
}




/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int write_setup(void)
{
FILE *fp;
int  i, k;

if ((fp = fopen(CFG_FILE_NAME, "wb")) == NULL)
  {
  printf("\nCan't open file %s", CFG_FILE_NAME);
  fclose(fp);
  return FALSE;
  }

setup.Initialized =     0;
setup.CoordSys =        LATITUDE_LONGITUDE;
setup.AngleType =       DEG_MIN;
setup.HorizUnits =      METERS;
setup.DecimalAccuracy = 5;
setup.Datum       =     WGS84_DATUM;
setup.Gyro        =     BUSTER_GYRO;
setup.DisplayType =     PLAN_DISPLAY;
setup.SidePanelType =   LINE_INFO;

// data logging
setup.logDisk =            0;          // Use the current drive
setup.BinLoggingEnabled =  TRUE;
setup.AsciiLogEnabled =  TRUE;
setup.AsciiLogInterval = 10;

// filenames
strcpy(setup.ShipName, "simple.SHP");
strcpy(setup.FishName, "Sidescan.SHP");
strcpy(setup.CoastName, "palosvrd.cst");
strcpy(setup.binFilename, "bin_nav.log");
strcpy(setup.ascFilename, "asci_nav.log");
strcpy(setup.logFilename, "logbook");

// line and site info
strcpy(setup.curSite.name, "Default Site");
strcpy(setup.curLine.name, "Default Line");

setup.curLine.start.lat    = 37.5 * DEG_TO_RAD;
setup.curLine.start.lon    = -122.2 * DEG_TO_RAD;
setup.curLine.end.lat      = 36.5 * DEG_TO_RAD;
setup.curLine.end.lon      = -122.0 * DEG_TO_RAD;

setup.curSite.pos.lat   = 36.5 * DEG_TO_RAD;
setup.curSite.pos.lon      = -122.0 * DEG_TO_RAD;

// filter constants
setup.filter.time =  30.0;            // go back no farther than 30 seconds
setup.filter.weight =  50;        // middle range 0-100

// printer constants
setup.printer.enabled  = 1;                 // enabled yes
setup.printer.interval = 300;               // 5 minutes
setup.printer.idx      = 1;
setup.printer.xScale   = 2.5;              
setup.printer.yScale   = 2.5;
setup.printer.columns  = 80;
setup.printer.lines    = 60;
strcpy(setup.printer.name, "HP PaintJet 16 Colors");


// plotter constants
setup.plot.interval = 10;
setup.plot.destination = PLOT_SCREEN;
setup.plot.sensor_idx = TRIMBLE_NAVTRAK;
setup.plot.start = 0.0;
setup.plot.stop  = 0.0;
setup.plot.scale  = 50000L;
setup.plot.center.lat  = 37.5 * DEG_TO_RAD;
setup.plot.center.lon  = -122.20 * DEG_TO_RAD;
setup.plot.center.lon  = -122.20 * DEG_TO_RAD;
setup.plot.HPcfg.p1.X = 250;
setup.plot.HPcfg.p1.Y = 596;
setup.plot.HPcfg.p2.X = 10250;
setup.plot.HPcfg.p2.Y = 7796;
setup.plot.port = (t_com *) NULL;
strcpy(setup.plot.filename, "C:\\USGS\\NAV\\BIN_NAV.LOG");

// Vrp constants
setup.Vrp.id      = VRP;          // The Stern location
setup.Vrp.x       = 0.0;          // The Stern location
setup.Vrp.y       = 0.0;          // The Stern location
setup.Vrp.z       = 0.0;          // The Stern location
setup.Vrp.dist    = 0.0;          // The Stern location
setup.Vrp.bearing = 0.0;          // The Stern location

// UTM
setup.utm.zoneNum = 11;           // UTM zone
setup.utm.zoneLtr = (char) 'R';   // UTM zone

// colors
setup.fg          = 0;
setup.bg          = 15;

// depth
setup.depth.depthCorrectionEnabled = TRUE;
setup.depth.soundVelocity = 1464.8;

// Only track the active sensor
for (i=0; i<MAX_COMPORTS; i++)
	{
	setup.displaySensors[i] = 0;
	}

// relays
setup.relay.enabled = 0;
setup.relay.address = 0x330;
setup.relay.status  = SUCCESS;
for (i=0; i < MAX_RELAYS; i++)
	{
	setup.relay.relaysEnabled[i]   = 0;
	setup.relay.relayStates[i]     = 0;
	setup.relay.relayIntervals[i]  = 0;
	setup.relay.inputsEnabled[i]   = 0;
	setup.relay.inputStates[i]     = 0;
	setup.relay.inputIntervals[i]  = 0;
	setup.relay.lastTime[i]        = 0L;
	}

// video
setup.videoEnabled = 0;

// miscellaneous
setup.misc.shipCentering  = TRUE;           // Keep the ship centered
setup.misc.shipScaling    = FALSE;          // Don't draw ship to scale
setup.misc.showCoastline  = FALSE;          // Do not draw Coasline
setup.misc.fixSymbol      = DOT;            // A simple dot

k = fwrite( (char *) &setup, sizeof(t_setup), 1, fp);
if (k == -1)
	{
	printf("Write failed %d\n", k);
	exit(1);
	}

fclose(fp);
return TRUE;
}



