/*----------------------------------------------------------------------------*/
// navdefs.h       Manifest constants for USGS nav software
// Blame: John Gann
// 29 April 1992
/*----------------------------------------------------------------------------*/
#define NAV_VERSION "YoNav V1.09"

// sensors
#define GPS  0x10
#define LORAN  0x20
#define RANGING  0x30

#define FIVE_MINUTES         300L

#define MAGNAVOX_1107          0
#define ASHTECH                1
#define DDMU540                2
#define NS7000                 3
#define TRIMBLE_4000           4
#define TRACKPOINT_II          5
#define BRIDGE_TERM            6
#define DATASONICS_CHIRP       7
#define KVH_GYRO               8
#define DDMU_TO_URI            9
#define TRACKPOINT_TO_URI     10
#define BUSTER_GYRO           11
#define TRIMBLE_NAVTRAK       12
#define TRIMBLE_10X           13
#define MUDSHARK              14
#define PDD_DEPTH_DIGITIZER   15
#define HP_PLOTTER            16
#define TRIMBLE_TANS          17

#define VRP                   -1           // Vehicle Reference Position
#define NO_GYRO               -1           // No Gyro Use CMG

// record sizes
#define NS7000_REC_SIZE        150
#define TRIMBLE_4000_REC_SIZE  120
#define MAGNAVOX_1107_REC_SIZE  35
#define ABC_GYRO_REC_SIZE       36
#define KVH_GYRO_REC_SIZE       19
#define BUSTER_GYRO_REC_SIZE     7
#define DDMU540_REC_SIZE        65
#define TRACKPOINT_REC_SIZE     66
#define NAVTRAK_REC_SIZE        52
#define VAX_REC_LENGTH         132
#define PDD_DEPTH_REC_SIZE       5
#define DEFAULT_REC_SIZE        80

// main display types
#define PLAN_DISPLAY                       0
#define LINE_DISPLAY                       1
#define POLAR_DISPLAY                      2
#define RANGE_REDUCTION_DISPLAY            3

// side panel display types
#define LINE_INFO                          0
#define SITE_INFO                          1

// directions for line generator
#define GENERATE_LEFT  0
#define GENERATE_RIGHT 1
#define GENERATE_BOTH  2

// Bridge terminal message packets
#define BRIDGE_SETUP           0x60
#define BRIDGE_ALERT           0x61
#define REQUIRE_ACKKNOWLEDGE   0
#define INFORMATION_ONLY       1

// PDISO 8 Optically Isolated Relay Board
#define MAX_RELAYS                 8

// PDD's internal sound velocity m/s
#define PDD_SOUND_VELOCITY    1500.0
#define PDD_ALARM_IO_LINE     1         // use digital IO line One from PDISO-8

// plotter definitions
#define PLOT_SCREEN           0
#define PLOT_PLOTTER          1

#define PLOT_BAUD             9600
#define PLOT_PARITY           'N'
#define PLOT_DATA_BITS         8
#define PLOT_STOP_BITS         1
#define PLOT_HANDSHAKE         0
#define MAX_PLOT_TICKS        20
#define PLOT_EOF              -3
#define PLOT_FILE_NULL        -4
#define PEN_DN                 0
#define PEN_UP                 1

// disk size limits
#define DISK_FREE_WARNING     20000L
#define DISK_WARNING_INTERVAL 60L

#define ARNET  1
#define BRIDGE_HISTORY_SIZE 100
#define BRIDGE_HISTORY_INTERVAL 10
#define BUTTONS_ONLY  0
#define BUTTONS_OR_MOTION 2
#define CLOCK 0x1A
#define CR  13  /* Carriage return */
#define DISABLE 0x0B
#define DRAND_MAX 32752.0
#define ESSENTIAL 2
#define FALSE 0
#define FLUSH 0x09
#define FUNC_19 0x13
#define GET_DATE 0x04
#define GET_TIME 0x02
#define HISTORY_SIZE 50
#define HISTORY_INTERVAL 5
#define INVALID 99
#define IQCHK 0x0A
#define FF  12  /* Form Feed */
#define LF  10  /* line feed */
#define MAX_COMPORTS 10
#define MAX_COASTLINE_SIZE   2500
#define MAX_COASTLINE_FILES  50
#define MAX_ENTITIES          30
#define MAX_GEN_LINES  100
#define MAX_PORT_BUF_SIZE    255
#define MAX_HYPFIX_STATIONS 7
#define MAX_VERTICES          30
#define	MAX_LAT	  1.570796326794897                // PI / 2
#define	MIN_LAT	 -1.570796326794897                // PI / 2
#define MAX_LINES                       100
#define MAX_SITES                       100
#define	MAX_LON	  3.141592653589793                // PI
#define	MIN_LON	 -3.141592653589793                // PI
#define	MAX_LSQ_ITERATIONS	 10
#define MAX_RANGES   6
#define MAX_TRISPO_RANGES   4
#define MAX_SENSORS 18
#define MAX_NAV_STATIONS 10
#define MAX_TDS      5
#define MPS_TO_KTS 1.9438445
#define NUM_RANGE_RINGS            5
#define KTS_TO_MPS .51444444
#define MOTION_ONLY  1
#define NO_WAIT 1
#define nyb2byt(a,b) ((((a)&0x000f)<<4)|((b)&0x000f))
#define OQCHK 0x16
#define RD2DG 57.29577951471995 /* radians to degrees multiplier */
#define ROBERTSON  11
#define RSINT 0x14
#define SET_HANDSHAKE 5
#define SYLEDIS 0
#define SYLEDIS_SR3  12
#define SUCCESS 0
#define TCURSOROFF 0x2020
#define TIME_OUT 0xFFFF
#define TRUE (!FALSE)
#define UNASSIGNED  99
#define WAIT 0

#define	RD2MTR	1768.5297		/* radians to meters approx !! */
#define PROP_VEL	    299.792458	/* loran propagation velocity */

#define LEFTB  1
#define RIGHTB 2

#define CANT_INIT_ESSENTIAL 10
#define UNKNOWN_SERIAL_MODE 11

#define	GRI_5930	0
#define	GRI_7980	1
#define	GRI_8970	2
#define	GRI_9960	3
#define	GRI_9990	4
#define	GRI_9940	5
#define	GRI_4990	6

#define XMIT_BYTE   0x01
#define READ_BYTE   0x02
#define COMM_STATUS 0x03
#define CONFIGURE   0x04
#define PROTOCOL    0x05
#define ZERO        0x00
#define SEND_BUFFER 0x14
#define READ_STRING 0x15
#define	T_GPS_0 315964800L

#define	PI	 3.141592653589793
#define	TWOPI	 6.283185307179586
#define	HALFPI	 1.570796326794897
#define	T_GPS_0 315964800L

#define ELLIPSOID 0
#define GEOID     1

#define LATITUDE_LONGITUDE            0
#define UNIVERSAL_TRANSVERSE_MERCATOR 1
#define STATE_PLANE                   2

#define DEC_DEG     0
#define DEG_MIN     1
#define DEG_MIN_SEC 2
#define RADIANS     3

#define METERS      0
#define FEET        1
#define SURVEY_FEET 2
#define YARDS        2

#define METERS_TO_YARDS       1.0936133
#define METERS_TO_FEET        3.28084
#define METERS_TO_SURVEY_FEET ( 3937.0/1200.0 )
#define FEET_TO_METERS        0.3048
#define YARDS_TO_METERS       (1.0 / METERS_TO_YARDS)
#define SURVEY_FEET_TO_METERS ( 1200.0/3937.0 )

#define DEG_TO_RAD ( PI/180.0 )
#define RAD_TO_DEG ( 1.0/DEG_TO_RAD ) 

#define	AE_WGS84	6378137.0
#define	E2_WGS84	0.006694380022903416

#define MIN_RECORD_SIZE 100
#define LAT 1
#define LON 2

#define OLD_NAV_AGE        30.0

#define OKAY  0
#define CALIBRATE  0
#define DEFAULT 1
#define DONE 1
#define CANCEL 2
#define HELP 3

#define BAD_FILE_OPEN                 -16
#define BAD_DATA_READ                 -17
#define BAD_FILE_POINTER              -19
#define BAD_FILE_SEEK                 -20
#define BAD_PATH_NAME                 -21
#define NOT_ENOUGH_DISK_SPACE         -22
#define BAD_MALLOC                    -23
#define FILE_AT_EOF                   -24
#define NULL_LINE                     -25
#define BAD_TAIL_POINTER              -26
#define DATUM_NOT_FOUND               -27
#define LINES_ALREADY_LOADED          -28
#define BAD_DAY_OF_YEAR               -29
#define BAD_POSITION                  -30
#define BAD_EOL_CONVERSION            -31
#define BAD_UTM_CONVERSION            -32
#define BAD_SOL_CONVERSION            -33
#define BAD_SETUP                     -34
#define NOT_ENOUGH_RANGES             -35
#define BAD_FORM_LOAD                 -36
#define NULL_OBSERVATIONS             -37
#define BAD_FORM_OPEN                 -38
#define NULL_NAV                      -39
#define BAD_FILE_WRITE                -40
#define FILE_NOT_OPEN                 -41
#define FILE_ALREADY_OPEN             -42
#define NULL_HISTORY                  -43
#define NOT_ENOUGH_HISTORY            -44
#define NO_NEW_HISTORY                -45
#define BAD_LATITUDE                  -46
#define BAD_LONGITUDE                 -47
#define NULL_TRACK                    -48
#define UNABLE_TO_CREATE_TMPFILE      -49
#define NOT_TIME_YET                  -50
#define LOGGING_NOT_ENABLED           -51
#define BAD_OFFSET                    -52
#define OLD_NAV_DATA                  -53
#define BAD_TRACKPOINT                -54
#define BAD_DDMU540                   -55
#define NO_LIVE_WINDOWS_BOARD         -56
#define RUBBER_SHAPE_LIMIT_ERR        -57
#define NO_MATCHING_FILES             -58
#define NO_MEMORY                     -59
#define NO_FISH_NAV                   -60
#define SITES_ALREADY_LOADED          -61
#define NULL_SITE                     -62
#define BAD_PLOTTER_READ              -63
#define NULL_COM                      -64
#define SERIAL_IO_TIMEOUT             -65
#define SHAPE_IS_FULL                 -66

// Ellipsoid and datum shift defs
#define NO_CONVERSION             0
#define MRE                       2
#define MOLODENSKY                3

#define COM_FILE_NAME            "nav.cmm"
#define GRAF_FILE_NAME           "nav.grf"
#define NAV_FILE_NAME            "nav.nav"
#define SHIP_FILE_NAME           "farnella.shp"
#define DATUM_FILE_NAME          "datums.dat"
#define CFG_FILE_NAME            "nav.cfg"
#define STATION_FILE_NAME        "stations.dat"
#define OFFSET_FILE_NAME         "nav.off"
#define VIDEO_FILE_NAME          "nav.vid"
#define LOG_BOOK_FILE_NAME       "nav.lgb"
#define SITE_FILE_NAME           "sites.dat"

// control lines constants
#define XON_XOFF 1
#define XON     17
#define XOFF    18
#define ETX                      0x03
#define DLE                      0x10

#define MENU_PICKS               5
#define TMP_TEMPLATE             "nvXXXXXX"

#define MAX_PACKET_SIZE 1024
#define COORDS_NOT_ON_SCREEN -1
#define SHIFT_UP              1
#define SHIFT_DOWN           -1

#define COMPUTE_NEW_ZONE 1
#define USE_PASSED_ZONE 0

#define	F1_KEY         	0x0001
#define	F2_KEY         	0x0002
#define	F3_KEY         	0x0003
#define	F4_KEY         	0x0004
#define	F5_KEY         	0x0005
#define	F6_KEY         	0x0006
#define	F7_KEY         	0x0007

// mouse definitions
#define UNKNOWN_MOUSE    0
#define BUS_MOUSE        1
#define SERIAL_MOUSE     2
#define INPORT_MOUSE     3
#define PS2_MOUSE        4
#define HP_MOUSE         5

// Mouse interrupts
#define PS2_IRQ          0
#define UNKNOWN_IRQ      1
#define COM2_IRQ         3
#define COM1_IRQ         4

// Graphic definitions
#define DOT                                 0
#define CROSS                               1
#define THE_X                               2
#define SQUARE_DOT                          3
#define CIRCLE_DOT                          4
#define TRIANGLE_DOT                        5
#define LIGHT_POLE                          6
#define FIRE_HYDRANT                        7
#define CROSSED_SQUARE                      8
#define CROSSED_CIRCLE                      9
#define THING_A_MA_BOB                     10
#define HORIZ_BRIDGE                       11
#define VERT_BRIDGE                        12
#define THE_Y                              13
#define FARNELLA                           14

#define G_POINT          0
#define G_LINE           1
#define G_OVAL           2

#define SOLID_LINE         0

#define BLACK            0
#define BLUE             1
#define GREEN            2
#define CYAN             3
#define RED              4
#define MAGENTA          5
#define BROWN            6
#define WHITE            7
#define GREY             8
#define BRIGHT_BLUE      9
#define BRIGHT_GREEN    10
#define BRIGHT_CYAN     11
#define BRIGHT_RED      12
#define BRIGHT_MAGENTA  13
#define BRIGHT_YELLOW   14
#define BRIGHT_WHITE    15



// mk_shape definitions
#define NO_POINTS      -1
#define ONLY_ONE_ADJACENT_POINT -2
#define INDEX_OUT_OF_RANGE  -100
#define ENTITY_NOT_FOUND    -101
#define NO_ENTITIES         -102
#define NULL_ENTITY         -103
#define G_SLOP 6

#define DRAW_POINT          0
#define DRAW_LINE           1
#define DRAW_OVAL           2

#define MOVE_POINT         10
#define MOVE_LINE          11
#define MOVE_OVAL          12

#define MODIFY_POINT       20
#define MODIFY_LINE        21
#define MODIFY_OVAL        22

#define DELETE_POINT       30
#define DELETE_LINE        31
#define DELETE_OVAL        32

#define EDIT_POINT_ATTRIBUTE       40
#define EDIT_LINE_ATTRIBUTE        41
#define EDIT_OVAL_ATTRIBUTE      42

#define OBJECTS_PER_FORM  10

