#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <dos.h>
#include <fcntl.h>
#include <process.h>
#include <memory.h>
#include <io.h>
#include <conio.h>
#include <malloc.h>
#include <float.h>
#include <time.h>
#include <sys\timeb.h>
#include <sys\types.h>
#include <sys\stat.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "offsets.h"
#include "nav_util.h"            // prototypes


// Private Data
static char * month_names[] = 
	{
	"JAN",
	"FEB",
	"MAR",
	"APR",
	"MAY",
	"JUN",
	"JUL",
	"AUG",
	"SEP",
	"OCT",
	"NOV",
	"DEC"
	};

static char *days[] = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};

static t_setup Setup;
static t_datum theDatum;
static char DatumLoaded = FALSE;
static char grid_ltr;                            // used for UTM
static char logBookFileOK = TRUE;                // used for debug logging
static FILE * theLogBookFile = (FILE *) NULL;    // used for debug logging
static int grid_num;                             // used for UTM
static t_track trackInfo;                        // orthogonal line distances
static t_rngBrg rngBrg;                          // range and bearing to curSite

/*----------------------------------------------------------------------------*/
/* Replace theChar with a space.  If theChar was found return TRUE else       */
/* return FALSE.                                                              */
/*----------------------------------------------------------------------------*/
int WhiteOutChar(char * str, char theChar)
{
int retv = 0;

while (*str)
	{
	if (*str == theChar)
		{
		*str = ' ';
		retv++;
		}
	str++;
	}

return retv;
}
	


/*----------------------------------------------------------------------------*/
// Replace all chars in source that also occur in target with theChar
/*----------------------------------------------------------------------------*/
int replaceMultiChar(char * source, char * target, char newChar)
{
int retv = FALSE;
char * pTarget = target;
char * pSource = source;
char targetChar;
char sourceChar;

while (targetChar = *pTarget)
	{
	pSource = source;
	while (sourceChar = *pSource)
		{
		if (sourceChar == targetChar)
			{
			*pSource = newChar;
			retv++;
			}
		pSource++;
		}
	pTarget++;
	}
return retv;
}
	



/*----------------------------------------------------------------------------*/
// Replace theChar with a passed newChar.
/*----------------------------------------------------------------------------*/
int replaceChar(char * str, char theChar, char newChar)
{
int retv = FALSE;

while (*str)
	{
	if (*str == theChar)
		{
		*str = newChar;
		retv++;
		}
	str++;
	}

return retv;
}
	

/*----------------------------------------------------------------------------*/
/* Parse out a string buffer into a radian value                              */
/*
Modified 9 April 92 to be nondestructive to input buffer
Method:

      1. These characters will produce negative radians: -, S, and W.
      2. First the hemisphere is determined.
      3. Next all non-numerical chars are replaced with spaces.
      4. Then the string is tokenized into an array of pointers.
			5. The number of discreet words determines how input will be processed.
      6. 1 word: dec. deg, 2: deg min, 3: deg min sec,  default: Error
*/
/*----------------------------------------------------------------------------*/
int PositionStrToRads(char inbuf[], double * rads, int mode)
{
char seps[] = " ,\t\n\"\x27\xF8NSEW+-:";   /* Field Separators */
char * token;
int token_count = 0;
int i=0, j=0;
char * words [10];
int hem = 1;
int retv = TRUE;
char tempBuf[255];

if (inbuf[0] == '\0')
	return  FALSE;

// copy the passed buffer to the temp buffer
strcpy(tempBuf, inbuf);

/* Convert input to upper case */
strupr(tempBuf);

switch (mode)
	{
	case LAT:
		if (WhiteOutChar(tempBuf, 'S') || WhiteOutChar(tempBuf, '-'))
			{
			hem = -1;
			}
		break;

	case LON:
		if (WhiteOutChar(tempBuf, 'W') || WhiteOutChar(tempBuf, '-'))
			{
			hem = -1;
			}
		break;
	}

/* Strip out chars that imply positive */
WhiteOutChar(tempBuf, '+');
WhiteOutChar(tempBuf, 'N');
WhiteOutChar(tempBuf, 'E');
	
token = strtok(tempBuf, seps);

while (token != (char *) NULL)
	{
	words[token_count] = (char *) malloc (strlen(token) + 1);
	strcpy(words[token_count], token);
	token_count++;
	token = strtok(NULL, seps);
	}

switch (token_count)
	{
	case 1:                /* Decimal degrees */
		*rads  = atof(words[0]) * DEG_TO_RAD;
		*rads *= (double) hem;
		break;

	case 2:                /* Degrees and minutes */
		*rads  =  atof(words[0]) * DEG_TO_RAD;
		*rads += (atof(words[1]) / 60.0) * DEG_TO_RAD;
		*rads *= (double) hem;
		break;

	case 3:                /* Degrees minutes and seconds */
		*rads  =  atof(words[0]) * DEG_TO_RAD;
		*rads += (atof(words[1]) / 60.0) * DEG_TO_RAD;
		*rads += (atof(words[2]) / 3600.0) * DEG_TO_RAD;
		*rads *= (double) hem;
		break;

	default:
		retv = FALSE;
		break;
	}

/* Check the Values before exiting */
if (mode == LAT)
	{
	if (fabs(*rads) > HALFPI)
		retv = FALSE;
	}
else
	{
	if (mode == LON)
		{
		if (fabs(*rads) > TWOPI)
			{
			*rads = fmod(*rads, PI);
			}
		}
	}

/* Free up the words */
while (token_count--)
	{
	free(words[token_count]);
	}

return retv;
}


/*----------------------------------------------------------------------------*/
/* Display the radians of lat and long as the proper string based on the      */
/* system and units.                                                          */
/* Input coordinates are assumed to be in WGS84 and are converted to local    */
/* datum if necessary.                                                        */
/* Only do Datum transformation if Xform is TRUE                              */
/*----------------------------------------------------------------------------*/
int PositionString(char LonBuf[], char LatBuf[], char AltBuf[], 
                   double Lon, double Lat, double Alt, char Xform)
{
char LatFmt[40];
char LonFmt[40];
int LatDeg;
int LatMin;
int LonDeg;
int LonMin;
long l;
char LatHem;
char LonHem;
double LatSec;
double LonSec;
double x, y, scale_factor;
int grid_num;
char grid_ltr;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 

// Write the Altitude Buffer 
AltitudeString(AltBuf, Alt);

LatHem = (char) ((rads_to_dms(Lat, &LatDeg, &LatMin, &LatSec) > 0) ? 'N' : 'S');
LonHem = (char) ((rads_to_dms(Lon, &LonDeg, &LonMin, &LonSec) > 0) ? 'E' : 'W');
	
switch (Setup.CoordSys)
	{
	case LATITUDE_LONGITUDE:
		switch (Setup.AngleType)
			{
			case DEC_DEG:
				sprintf(LatFmt, "%%15.%dlf", Setup.DecimalAccuracy);
				sprintf(LonFmt, "%%15.%dlf", Setup.DecimalAccuracy);
				sprintf(LonBuf, LonFmt, Lon * RAD_TO_DEG);
				sprintf(LatBuf, LatFmt, Lat * RAD_TO_DEG);
				break;

			case DEG_MIN:
				sprintf(LatFmt, "%%02d\xF8 %%02d.%%0%dld\x27 %%c", Setup.DecimalAccuracy);
				sprintf(LonFmt, "%%03d\xF8 %%02d.%%0%dld\x27 %%c", Setup.DecimalAccuracy);
				l = (long) ((LatSec / 60.0) * pow(10.0, (double) Setup.DecimalAccuracy));
				sprintf(LatBuf, LatFmt, LatDeg, LatMin, l, LatHem);
				l = (long) ((LonSec / 60.0) * pow(10.0, (double) Setup.DecimalAccuracy));
				sprintf(LonBuf, LonFmt, LonDeg, LonMin, l, LonHem);
				break;

			case DEG_MIN_SEC:
				sprintf(LonFmt, "%%03d\xF8 %%02d\x27 %%02d.%%0%dld\x22 %%c", 
                Setup.DecimalAccuracy);
				sprintf(LatFmt, "%%02d\xF8 %%02d\x27 %%02d.%%0%dld\x22 %%c", 
                Setup.DecimalAccuracy);
				l = (long) ((LatSec - (int) (LatSec)) * 
                     pow(10.0, (double) Setup.DecimalAccuracy));
				sprintf(LatBuf, LatFmt, LatDeg, LatMin, (int) LatSec, l, LatHem);
				l = (long) ((LonSec - (int) (LonSec)) * 
                     pow(10.0, (double) Setup.DecimalAccuracy));
				sprintf(LonBuf, LonFmt, LonDeg, LonMin, (int) LonSec, l, LonHem);
				break;

			case RADIANS:
				sprintf(LatFmt, "%%.%dlf", Setup.DecimalAccuracy);
				sprintf(LonFmt, "%%.%dlf", Setup.DecimalAccuracy);
				sprintf(LonBuf, LonFmt, Lon);
				sprintf(LatBuf, LatFmt, Lat);
				break;

			default:
				break;
			}
		break;

	case STATE_PLANE:
		break;

	case UNIVERSAL_TRANSVERSE_MERCATOR:
		ll_to_utm(Lat, Lon, &y, &x, &scale_factor, 
				&grid_num, &grid_ltr, 1, ellipse->a, ellipse->e2);
		AltitudeString(LonBuf, x);
		AltitudeString(LatBuf, y);
		break;
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
/* Given the latitude in radians, convert it to a string in the proper units  */
/*----------------------------------------------------------------------------*/
int LatString(char ret_buf[], double rads)
{
char fmt[40];
int retv;
int int_deg;
int int_min;
long l;
char hem;
double dec_sec;

retv =  rads_to_dms(rads, &int_deg, &int_min, &dec_sec);
hem = (char) ((retv > 0) ? 'N' : 'S');
	
switch (Setup.AngleType)
	{
	case DEC_DEG:
		sprintf(fmt, "%%15.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads * RAD_TO_DEG);
		break;

	case DEG_MIN:
		sprintf(fmt, "%%02d\xF8 %%02d.%%0%dld\x27 %%c", Setup.DecimalAccuracy);
		l = (long) ((dec_sec / 60.0) * pow(10.0, (double) Setup.DecimalAccuracy));
		sprintf(ret_buf, fmt, int_deg, int_min, l, hem);
		break;

	case DEG_MIN_SEC:
		sprintf(fmt, "%%02d\xF8 %%02d\x27 %%02d.%%0%dld\x22 %%c", Setup.DecimalAccuracy);
		l = (long) ((dec_sec - (int) (dec_sec)) * pow(10.0, (double) Setup.DecimalAccuracy));
		sprintf(ret_buf, fmt, int_deg, int_min, (int) dec_sec, l, hem);
		break;

	case RADIANS:
		sprintf(fmt, "%%.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads);
		break;

	default:
		sprintf(fmt, "%%.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads);
		break;

	}

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
/* Given the longitude in radians, convert it to a string in the proper units */
/*----------------------------------------------------------------------------*/
int LonString(char ret_buf[], double rads)
{
char fmt[40];
int retv;
int int_deg;
int int_min;
long l;
char hem;
double dec_sec;

retv =  rads_to_dms(rads, &int_deg, &int_min, &dec_sec);
hem = (char) ((retv > 0) ? 'E' : 'W');
	
switch (Setup.AngleType)
	{
	case DEC_DEG:
		sprintf(fmt, "%%15.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads * RAD_TO_DEG);
		break;

	case DEG_MIN:
		sprintf(fmt, "%%03d\xF8 %%02d.%%0%dld\x27 %%c", Setup.DecimalAccuracy);
		l = (long) ((dec_sec / 60.0) * pow(10.0, (double) Setup.DecimalAccuracy));
		sprintf(ret_buf, fmt, int_deg, int_min, l, hem);
		break;

	case DEG_MIN_SEC:
		sprintf(fmt, "%%03d\xF8 %%02d\x27 %%02d.%%0%dld\x22 %%c", Setup.DecimalAccuracy);
		l = (long) ((dec_sec - (int) (dec_sec)) * pow(10.0, (double) Setup.DecimalAccuracy));
		sprintf(ret_buf, fmt, int_deg, int_min, (int) dec_sec, l, hem);
		break;

	case RADIANS:
		sprintf(fmt, "%%.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads);
		break;

	default:
		sprintf(fmt, "%%.%dlf", Setup.DecimalAccuracy);
		sprintf(ret_buf, fmt, rads);
		break;

	}

return SUCCESS;
}





/*----------------------------------------------------------------------------*/
/* Convert the number to a formatted distance string in the proper units      */
/* with the proper accuracy                                                   */
/* returns TRUE if a known conversion was applied, else returns FALSE if no   */
/* conversion was applied                                                     */
/* No datum conversion is done                                                */
/*----------------------------------------------------------------------------*/
int AltitudeString(char ret_buf[], double number)
{
char fmt[40];
int retv = TRUE;

sprintf(fmt, "%%.%dlf", Setup.DecimalAccuracy);
switch(Setup.HorizUnits)
	{
	case METERS:
		sprintf(ret_buf, fmt, number);
		break;

	case FEET:
		sprintf(ret_buf, fmt, number * METERS_TO_FEET);
		break;

	case SURVEY_FEET:
		sprintf(ret_buf, fmt, number * METERS_TO_SURVEY_FEET);
		break;

	default:
		sprintf(ret_buf, fmt, number);
		retv = FALSE;
		break;
	}

return retv;
}




/*----------------------------------------------------------------------------*/
/* Convert radians to DMS and return sign of radians                          */
/*----------------------------------------------------------------------------*/
int rads_to_dms(double rads, int * int_deg, int * int_min, double * dec_sec)
{
double dec_deg, dec_min;

dec_deg = (fabs(rads * RAD_TO_DEG));
*int_deg = (int) floor(dec_deg);

dec_min = (dec_deg - (float) *int_deg) * 60.0;
*int_min = (int) floor(dec_min);

*dec_sec = (dec_min - (float) *int_min) * 60.0;
return (rads < 0.0) ? -1 : 1;
}



/*----------------------------------------------------------------------------*/
// return the DOS time equivalent of the passed GPS time of week
/*----------------------------------------------------------------------------*/
time_t GPStoDOS(double gps_time, int week)
{
time_t now = time(NULL);
long time_of_day = ((long) gps_time) % 86400L;
now -= now % 86400;         // remove the fraction of a day

return now + time_of_day;
}

/*----------------------------------------------------------------------------*/
// return the GPS time equivalent of the passed DOS time 
/*----------------------------------------------------------------------------*/
double DOStoGPS(time_t dos_time)
{
return (double) (dos_time + T_GPS_0);
}


/*----------------------------------------------------------------------------*/
// return pointer to a string of time	
/*----------------------------------------------------------------------------*/
char *doy_time(time_t tim)		
{
static char *tim_string = "      999 99:99:99 ";
struct tm *thisTime;
time_t now ;

if (tim == 0L)
	{
	now = time(NULL);
	thisTime = gmtime( &now );
	}
else
	{
	thisTime = gmtime( &tim);
	}

sprintf(tim_string,"%03d %02d:%02d:%02d",
      thisTime->tm_yday+1,
      thisTime->tm_hour, thisTime->tm_min, 
      thisTime->tm_sec);

return(tim_string);
}




/*----------------------------------------------------------------------------*/
// return the integer month given a string value  (range 0 - 11)
/*----------------------------------------------------------------------------*/
int month_str_to_int(char * month_str)
{
int i;

for (i=0; i<12; i++)
	{
	if (!strnicmp(month_names[i], month_str, 3))
		{
		return i;
		}
	}
return 0;
}



/*----------------------------------------------------------------------------*/
/* Return the length of a radian of latitude at the passed latitude           */
/*----------------------------------------------------------------------------*/
double length_of_radian_of_lat(double a, double e2, double lat_rad)
{
double sin2lat = sin(lat_rad) * sin(lat_rad);

return a * (1.0 - e2) / pow( (1.0 - e2 * sin2lat), 1.5);
}

/*----------------------------------------------------------------------------*/
/* Return the length of a radian of longitude at the passed latitude          */
/*----------------------------------------------------------------------------*/
double length_of_radian_of_lon(double a, double e2, double lat_rad)
{
double sin2lat = sin(lat_rad) * sin(lat_rad);

return a * cos(lat_rad) / sqrt(1.0 - e2 * sin2lat);
}



/*----------------------------------------------------------------------------*/
// Return a formatted time pointer
/*----------------------------------------------------------------------------*/
char *trimble_time(time_t tim)	
{
static char *tim_string = "DAY DOY DD-MON-YY HH:MM:SS";
struct tm *thisTime;
struct timeb timebuffer;

ftime( &timebuffer );
if (tim == 0L)
	{
	thisTime = gmtime( &(timebuffer.time) );
	}
else
	{
	thisTime = gmtime(&tim);
	}

sprintf(tim_string,"%s %03d %02d-%s-%02d %02d:%02d:%02d", 
        days[thisTime->tm_wday],
        thisTime->tm_yday+1,
        thisTime->tm_mday,
        month_names[thisTime->tm_mon],
        thisTime->tm_year,
        thisTime->tm_hour, 
        thisTime->tm_min, 
        thisTime->tm_sec);

return(tim_string);

}



/*----------------------------------------------------------------------------*/
// Return a formatted latitude string pointer
/*----------------------------------------------------------------------------*/
char *trimble_lat(double lat)		
{
double d, m;
double dec_minutes;
double minutes;
char *dec_str = "mmmm   ";
static char *string = "DD:MM.mmmmH ";
minutes = modf(fabs(lat),&d) * 60.0;
dec_minutes = modf(minutes,&m);

sprintf(dec_str, "%04d%c",  (int) (dec_minutes * 10000.0),(lat>0.0) ? 'N' : 'S');
sprintf(string,"%02d:%02d.%s", (int) d, (int) m, dec_str);

return string;
}


/*----------------------------------------------------------------------------*/
// Return a formatted longitude string pointer
/*----------------------------------------------------------------------------*/
char *trimble_lon(double lon)	
{
double d, m;
double dec_minutes;
double minutes;
char *dec_str = "mmmm   ";
static char *string = "DDD:MM.mmmmH ";
minutes = modf(fabs(lon),&d) * 60.0;
dec_minutes = modf(minutes,&m);

sprintf(dec_str, "%04d%c",  (int) (dec_minutes * 10000.0),(lon>0.0) ? 'E' : 'W');
sprintf(string,"%02d:%02d.%s", (int) d, (int) m, dec_str);

return string;
}



/*----------------------------------------------------------------------------*/
// return pointer to a string of time	that fits Clint's format
/*----------------------------------------------------------------------------*/
char *record_time(time_t tim)
{
static char *tim_string = "YYDDDHHMMSS.s  ";
struct tm *thisTime;
time_t now;

if (tim == 0L)
	{
	now = time(NULL);
	thisTime = gmtime( &now );
	}
else
	{
	thisTime = gmtime( &tim);
	}

sprintf(tim_string,"%03d%02d%02d%02d0",
      thisTime->tm_yday+1,
      thisTime->tm_hour, 
      thisTime->tm_min, 
      thisTime->tm_sec);

return(tim_string);
}




/*----------------------------------------------------------------------------*/
// Return a formatted time pointer for the display
/*----------------------------------------------------------------------------*/
char *display_time(time_t tim)	
{
static char *tim_string = "DAY DOY DD-MON-YY HH:MM:SS";
struct tm *thisTime;
struct timeb timebuffer;

ftime( &timebuffer );
if (tim == 0L)
	{
	thisTime = gmtime( &(timebuffer.time) );
	}
else
	{
	thisTime = gmtime(&tim);
	}

sprintf(tim_string,"%s %02d-%s-%02d %02d:%02d:%02d", 
        days[thisTime->tm_wday],
        thisTime->tm_mday,
        month_names[thisTime->tm_mon],
        thisTime->tm_year,
        thisTime->tm_hour, 
        thisTime->tm_min, 
        thisTime->tm_sec);

return(tim_string);

}






/*----------------------------------------------------------------------------*/
// Return a pointer to the Setup structure
/*----------------------------------------------------------------------------*/
t_setup * QuerySetup(void)
{
if (!Setup.Initialized)
	{
	if (!read_setup())
		{
		printf("Failed to load configuration File %s\n", CFG_FILE_NAME);
		exit(10);
		}
	}
return (t_setup *) &Setup;
}



/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int write_setup(void)
{
FILE *fp;
int  k;

if ((fp = fopen(CFG_FILE_NAME, "wb")) == NULL)
  {
  printf("\nCan't open file %s", CFG_FILE_NAME);
  return FALSE;
  }

// Set the Initialized parameter to False
Setup.Initialized = FALSE;

k = fwrite( (char *) &Setup, sizeof(t_setup), 1, fp);
if (k == -1)
	{
	printf("Write failed %d\n", k);
	exit(1);
	}

fclose(fp);
return TRUE;
}



/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int read_setup(void)
{
FILE *fp;
int  k;

if ((fp = fopen(CFG_FILE_NAME, "rb")) == NULL)
  {
  printf("\nCan't open file %s", CFG_FILE_NAME);
  return FALSE;
  }

k = fread( (char *) &Setup, sizeof(t_setup), 1, fp);
if (k == -1)
	{
	printf("Read failed %d\n", k);
	exit(1);
	}

Setup.Initialized = TRUE;

fclose(fp);
return TRUE;
}





/*----------------------------------------------------------------------------*/
// Read the next line out of the any binary file 
/*----------------------------------------------------------------------------*/
int getNextLine(FILE * fp, void * buf, int size)
{
int k;

if (fp == (FILE *) NULL)
	{
	return BAD_FILE_POINTER;
	}

if (feof(fp))
	{
	return FILE_AT_EOF;
	}

k = fread( (void *) buf, size, 1, fp);

if (k != 1)
	{
	return BAD_DATA_READ;
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
/* Return a pointer to the ellipse defined by the current datum               */
/*----------------------------------------------------------------------------*/
t_ellips * query_ellipsoid(void)
{
int retv; 

// Has the datum been loaded ??
if (!DatumLoaded)
	{
	if ((retv = loadDatum(QuerySetup()->Datum)) != SUCCESS)
		{
		printf("Failed to load datum file %s\n", DATUM_FILE_NAME);
		return (t_ellips *) NULL;
		}
	}

return (t_ellips *) &theDatum.ellipsoid;
}

/*----------------------------------------------------------------------------*/
/* Return a pointer to the current datum                                      */
/*----------------------------------------------------------------------------*/
t_datum * query_datum(void)
{
int retv; 

// Has the datum been loaded ??
if (!DatumLoaded)
	{
	if ((retv = loadDatum(QuerySetup()->Datum)) != SUCCESS)
		{
		printf("Failed to load datum file %s\n", DATUM_FILE_NAME);
		return (t_datum *) NULL;
		}
	}

return (t_datum *) &theDatum;
}



/*----------------------------------------------------------------------------*/
// Load the datum and ellipsoid information from the disk
/*----------------------------------------------------------------------------*/
int loadDatum( int this_datum )
{
FILE *fp;
int  retv;
t_datum datum;

if ((fp = fopen(DATUM_FILE_NAME, "rb")) == NULL)
  {
  printf("\nCan't open file %s", DATUM_FILE_NAME);
  return BAD_FILE_OPEN;
  }

while((retv = getNextLine(fp, &datum, sizeof(t_datum))) == SUCCESS)
	{
	if (datum.id == (char) this_datum)
		{
		memcpy((void *) &theDatum, (void *) &datum, sizeof(t_datum));
		DatumLoaded = TRUE;
		return SUCCESS;
		}
	}

return DATUM_NOT_FOUND;
}


 

/*----------------------------------------------------------------------------*/
// Set the PC Clock using passed Time                                         
/*----------------------------------------------------------------------------*/
int SyncTime ( time_t t)
{
// char buf[128];
struct dostime_t dos_time;
struct dosdate_t dos_date;
struct tm *now;

now = gmtime(&t);

dos_time.hsecond = (unsigned char) 0;
dos_time.second = (unsigned char) now->tm_sec;
dos_time.minute = (unsigned char) now->tm_min;
dos_time.hour = (unsigned char) now->tm_hour;
dos_date.day = (unsigned char) now->tm_mday;
dos_date.month = (unsigned char) (now->tm_mon + 1);
dos_date.year = (unsigned int) (now->tm_year + 1900);

if (_dos_settime ( &dos_time ))
	{
	// sprintf(buf, "Bad Time: %02d:%02d:%02d:%03d\n", dos_time.hour, dos_time.minute, dos_time.second, dos_time.hsecond);
	// printf("%s", buf);
	return ( FALSE );
	}

if (_dos_setdate ( &dos_date ))
	{
	// 	sprintf(buf, "Bad Date: %04d %02d %02d \n", dos_date.year, dos_date.month, dos_date.day);
	// printf("%s", buf);
	return ( FALSE );
	}

return 1;
}




/*----------------------------------------------------------------------------*/
// return an integer value from the passed char *
/*----------------------------------------------------------------------------*/
int getInt(char * string, int start, int length)
{
char work[256];
char * p = string + start;
register i = 0;

if (length > 256)
	return -1;

strset(work, '\0');
while ((work[i++] = *p++) && (i < length));

return atoi(work);
}

/*----------------------------------------------------------------------------*/
// return an integer value from the passed char *
/*----------------------------------------------------------------------------*/
double getDbl(char * string, int start, int length)
{
char work[256];
char * p = string + start;
register i = 0;

if (length > 256)
	return -1;

strset(work, '\0');
while ((work[i++] = *p++) && (i < length));

return atof(work);
}



/*----------------------------------------------------------------------------*/
// Convert the passed degrees minutes and seconds to radians
/*----------------------------------------------------------------------------*/
double dms_to_rads(int deg, double minutes, double sec, char hem)
{
double rads = 0.0;

rads  =  (double) abs(deg) * DEG_TO_RAD;
rads += (fabs(minutes) / 60.0) * DEG_TO_RAD;
rads += (fabs(sec) / 3600.0) * DEG_TO_RAD;
rads *= (hem == 'S' || hem == 'W') ? -1.0 : 1.0;
return rads;
}


/*----------------------------------------------------------------------------*/
// Convert the passed year and day of year to month and day of month
// day and month values are returned in struct tm style
//   i.e. days since Jan 1 (0-365) and months (0,11)
/*----------------------------------------------------------------------------*/
int year_doy_to_month_day(int year, int doy, int * month, int * day)
{
int cum_days = 0;
int i = 0;
int isLeapYear = FALSE;
static days_per_month[2][13] = {
	{0,31,28,31,30,31,30,31,31,30,31,30,31}, 	/* Normal year */
	{0,31,29,31,30,31,30,31,31,30,31,30,31}		/* Leap year */
				}; 

// if year is less than 100 add 1900 to it
if (year < 100)
	year += 1900;

// check passed values
if (doy < 0 || doy > 366)
	return BAD_DAY_OF_YEAR;

// see if its a leap year
isLeapYear = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0);

// compute the days of the year
while (cum_days < doy - days_per_month[isLeapYear][i])
	{
	cum_days += days_per_month[isLeapYear][i++];
	}
*month = i;
*day  = doy - cum_days;
return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// compute the intersection of the survey line
// and a line through the current position with the
// negative inverse slope of the survey line
// solve simultaneous equations by elimination method
/*----------------------------------------------------------------------------*/
int computeLineOffsets(double curLat, double curLon, t_line * curLine)
{
int i;
int retv = SUCCESS;                       // return value
double eqn[2][3];		                      // simultaneous equation matrix
double eqn00_hold;                        // temps
double eqn11_hold;                        // temps
double curX, curY;                        // current coordinates in X and Y
double intersectX, intersectY;            // current coordinates in X and Y
double intersectLat, intersectLon;        // current coordinates in X and Y
double fazm, bazm;                        // forward & back azimuth
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 
double temp;
double rotated_cross_brg;
double rotated_along_brg;
double rotated_toend_brg;
double lineBrgDeg = curLine->bearing * RAD_TO_DEG;

if (curLat == 0.0 || curLon == 0.0)
	return BAD_POSITION;

// get the grid coordinates of the current position
geo2merc(curLat,
         curLon,
         &curX,
         &curY);

/********
if (trackInfo.slope == DBL_MAX)         // No slope, i.e. North/South
	{
	intersectX = curX - trackInfo.eolX;
	intersectY = trackInfo.solY - trackInfo.solY;
	return SUCCESS;
	}
**********/

// set up coefficient matrix for simultaneous equations	
eqn[0][0] = trackInfo.slope;
eqn[0][1] = -1.0;	/* survey line equation  */
eqn[0][2] = trackInfo.slope * trackInfo.eolX - trackInfo.eolY;

eqn[1][0] = 1.0;
eqn[1][1] = trackInfo.slope;	/* normal line equation  */
eqn[1][2] = trackInfo.slope * curY + curX;

// solve simultaneous equations by Elimination  
for (i=0; i<3; i++)
	{
	eqn[1][i] = -1.0 * eqn[0][0] * eqn[1][i];
	eqn[1][i] = eqn[0][i] + eqn[1][i];
	}

eqn11_hold = eqn[1][1];

for (i=0; i<3; i++)  
	{
	eqn[1][i] = (1.0 / eqn11_hold) * eqn[1][i];
	eqn[0][i] = eqn[0][i] + eqn[1][i];
	}
	
eqn00_hold = eqn[0][0];

for (i=0; i<3; i++)  
	{
	eqn[0][i] = eqn[0][i] * (1.0 / eqn00_hold);
	}

intersectY = eqn[1][2];
intersectX = eqn[0][2];

// convert the XY to lat lon
merc2geo(intersectX, intersectY, 
          &intersectLat, &intersectLon);

// compute cross track distance and orientation  
arclen(curLat, curLon,
       intersectLat, intersectLon,
       &trackInfo.cross,
       &fazm,
       &bazm,
       ellipse->a,
       ellipse->b);

/*	rotate the survey clockwise to north 
/*	and determine left or right of track	*/
rotated_cross_brg = fazm * RAD_TO_DEG - lineBrgDeg;

if (rotated_cross_brg < 0.0)
	rotated_cross_brg += 360.0;

if ((rotated_cross_brg > 0.0) && (rotated_cross_brg < 180.0))
	trackInfo.cross *= -1.0;

/* compute along track distance and orientation  */
arclen(intersectLat, intersectLon,
       curLine->start.lat, curLine->start.lon,
       &trackInfo.along,
       &fazm,
       &bazm,
       ellipse->a,
       ellipse->b);

/*	rotate the survey clockwise to north 
/*	and determine left or right of track	*/
temp = fazm * RAD_TO_DEG;
if (temp < 0.0)
	{
	temp = temp + 360.0;
	}

rotated_along_brg = temp - lineBrgDeg;
if (rotated_along_brg < 0.0)
	{
	rotated_along_brg += 360.0;
	}

if ((rotated_along_brg > 270.0) && (rotated_along_brg <= 360.0))
	{
	trackInfo.along *= -1.0;
	}

if ((rotated_along_brg >= 0.0) && (rotated_along_brg < 90.0))
	{
	trackInfo.along *= -1.0;
	}

/* compute toend track distance and orientation  */
arclen(intersectLat, intersectLon,
       curLine->end.lat, curLine->end.lon,
       &trackInfo.toend,
       &fazm,
       &bazm,
       ellipse->a,
       ellipse->b);

/*	rotate the survey clockwise to north 
and determine left or right of track	*/

rotated_toend_brg = fazm * RAD_TO_DEG - lineBrgDeg;

if (rotated_toend_brg < 0.0)
	rotated_toend_brg += 360.0;

if ((rotated_toend_brg > 90.0) && (rotated_toend_brg < 270.0))
	trackInfo.toend *= -1.0;

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Reset the range and bearing structure for the given site
/*----------------------------------------------------------------------------*/
int resetRngBrg(t_site * theSite)
{
int retv = SUCCESS;            // return value

memcpy((void *) &rngBrg.site,
       (void *) theSite,
       sizeof(t_site));
rngBrg.initialized = TRUE;

return SUCCESS;
}

/*----------------------------------------------------------------------------*/
// Reset the track info structure for the given line
/*----------------------------------------------------------------------------*/
int resetTrack(t_line * theLine)
{
int retv = SUCCESS;            // return value
double numerator;              // used in slope calcs
double denominator;            // used in slope calcs

// sanity checks
if (theLine == (t_line *) NULL)
	return  NULL_LINE;

// get the grid coordinates of the line
geo2merc(theLine->start.lat, 
         theLine->start.lon, 
         &trackInfo.solX, 
         &trackInfo.solY);

geo2merc(theLine->end.lat, 
         theLine->end.lon, 
         &trackInfo.eolX, 
         &trackInfo.eolY);

// compute the slope of the line
numerator   = trackInfo.eolY - trackInfo.solY;
denominator = trackInfo.eolX - trackInfo.solX;

if (numerator == 0.0)         // No slope, East West line
	{
	trackInfo.eolY += 1.0;
	numerator = trackInfo.eolY - trackInfo.solY;
	}

if (denominator == 0.0)         // No slope, i.e. North/South
	{
	trackInfo.eolX += 1.0;
	denominator = trackInfo.eolX - trackInfo.solX;
	}

trackInfo.slope = numerator / denominator;

trackInfo.initialized = TRUE;
trackInfo.bearing = theLine->bearing;
trackInfo.length =  theLine->range;
strcpy(trackInfo.name, theLine->name);
}

/*----------------------------------------------------------------------------*/
// Compute Nav Comps
/*----------------------------------------------------------------------------*/
int doNavComps(t_nav * p, t_history * history)
{
t_setup * setup = QuerySetup();
t_line * theLine;
t_site * theSite;
int retv;

// check the history
if (history == (t_history *) NULL)
	return NULL_HISTORY;

// Was the setup returned OK?
if (setup == (t_setup *) NULL)
	{
	return BAD_SETUP;
	}
else
	{
	theLine = (t_line *) &setup->curLine;
	theSite = (t_site *) &setup->curSite;
	}

// check nav
if ((retv = validNav(p)) != SUCCESS)
	{
	return retv;
	}

// make sure that the track structure is current
if (!trackInfo.initialized)
	{
	resetTrack(theLine);
	}

// compute the line offsets
computeLineOffsets(p->lat, p->lon, theLine);

// compute the range and bearing to site
// make sure that the site structure is current
if (!rngBrg.initialized)
	{
	resetRngBrg(theSite);
	}

computeSiteRangeBearing(p->lat, p->lon, theSite);

// compute the speed and course over the ground
return computeSpeedCourse(history, &p->spd, &p->crs);
}


/*----------------------------------------------------------------------------*/
// return a pointer to the static trackInfo structure
/*----------------------------------------------------------------------------*/
t_track * queryTrackInfo(void)
{
return (t_track *) &trackInfo;
}


/*----------------------------------------------------------------------------*/
// return a pointer to the static rngBrg structure
/*----------------------------------------------------------------------------*/
t_rngBrg * querySiteInfo(void)
{
return (t_rngBrg *) &rngBrg;
}



/*----------------------------------------------------------------------------*/
// See if the Arnet Smartport is installed
/*----------------------------------------------------------------------------*/
int SmartPortInstalled(int verbose, unsigned int * IO_port)
{
register i;
unsigned char OKtoLook = TRUE;
unsigned char done = FALSE;
unsigned int port = 0x0102;
unsigned int thisWord;
unsigned char thisByte;
unsigned int avoid_these[] = {
                                 0x01F2,  // Fixed disk
                                 0x0202,  // Game I/O
                                 0x0362,  // Reserved
                                 0x0382,  // SDLC, bysynchronous 2
                                 0x03A2,  // Bisynchronous 1
                                 0x03B2,  // Monochrome Display and Printer
                                 0x03C2,  // Reserved
                                 0x03D2   // Color/Graphics Monitor Adapter
                                };

// Search ports 0102 hex thru 03F2 hex 
do
  {
  OKtoLook = TRUE;
  for (i=0; i<sizeof(avoid_these) / sizeof(avoid_these[0]); i++)
    {
    if (port == avoid_these[i])
      {
      OKtoLook = FALSE;
      break;
      }
    }

  if (OKtoLook)
    {
    // compare the contents of memory here to the smart port ID chars
    thisWord = inpw(port);
		if (verbose)
    	printf("Searched port %04X found %04X\n", port, thisWord);
    if (thisWord == 0x5053)          // looking for "SP"
      {
      done = TRUE;
			*IO_port = port - 2;
			if (verbose)
      	printf("Smartport found at port %04X (%04X)\n", *IO_port, thisWord);
	    thisByte = (unsigned char) inp(*IO_port);
			if (verbose)
				{
				printf("Ports: [");
				for (i=0; i<8; i++)
					{
					printf("%d ", (int) ((thisByte & (1 << i)) ? i+1 : 0) );
					}
				printf("] are active\n");
				}
      }
    }

  port++;
  
  } while ((!done) && (port < 0x03F2));
return done;
}




/*----------------------------------------------------------------------------*/
/* Parse out two string buffers into two radian values                        */
/* The strings may contain UTM/State Plane X and Y                           */
/* The returned values will always be radians                            
/*----------------------------------------------------------------------------*/
int PositionXYToRads(char y_buf[], char x_buf[], 
                     double * y_rads, double * x_rads,
                     int zone_num, char zone_ltr)
{
char buf[80];
int retv = SUCCESS;
double x=0.0, y=0.0;
t_ellips * elps;

switch (Setup.CoordSys)
	{
	case LATITUDE_LONGITUDE:
		if (!PositionStrToRads(y_buf, y_rads, LAT))
			retv = -1;
		if (!PositionStrToRads(x_buf, x_rads, LON))
			retv = -1;
		break;

	case UNIVERSAL_TRANSVERSE_MERCATOR:
		x = atof(x_buf);
		y = atof(y_buf);
		if (x <= 0.0 || y <= 0.0)
			{
			retv = -5;
			}
		else
			{
			elps = query_ellipsoid();
			utm_to_ll(y, x, y_rads, x_rads, zone_num, zone_ltr, elps->a, elps->e2);
			}
		break;

	case STATE_PLANE:
		x = atof(x_buf);
		y = atof(y_buf);
		if (x <= 0.0 || y <= 0.0)
			{
			retv = -5;
			}
		else
			{
			elps = query_ellipsoid();
			//sp_to_ll(y, x, y_rads, x_rads);
			}
		break;
	}

if (retv == SUCCESS)
	{
	if (fabs(*x_rads) > PI)
		retv = -3;

	if (fabs(*y_rads) > HALFPI)
		retv = -4;
	}
else
	{
	sprintf(buf, "Badly Formed Position String at Line %d\n>>%s<<\n>>%s<<\nretv = %d\n\n", 
            __LINE__, y_buf, x_buf, retv);
//	NotifyUser(-1, -1, "WARNING", buf);
	}

return retv;
}




/*----------------------------------------------------------------------------*/
// Compute speed over the ground using the passed history array
/*----------------------------------------------------------------------------*/
int computeSpeedCourse(t_history * p, double * speed, double * course)
{
double distance;
double bazm;
double elapsedTime;
static double lastTime = 0.0;
static double prevSpeed  = 0.0;
static double deltaSpeed = 0.0;
double newCourse = 0.0;
double newSpeed  = 0.0;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps
t_setup * setup = QuerySetup();
t_fix   * lastFix;
char haveTime = FALSE;
int idx;
int j;
char buf[256];

if (p == (t_history *) NULL)
	return NULL_HISTORY;

// Was the setup returned OK?
if (setup == (t_setup *) NULL)
	{
	return BAD_SETUP;
	}

// if there are fewer than two points in the history return
if (p->numPts < 3)
	return NOT_ENOUGH_HISTORY;

// search backwards in track history for the elapsed filter time
lastFix = (t_fix *) &p->fix[p->numPts - 1];

if (lastFix->pos.lat == 0.0 || lastFix->pos.lon == 0.0)
	{
	return BAD_POSITION;
	}

if (lastTime == lastFix->tim)
	{
	if (prevSpeed != 0.0 && deltaSpeed != 0.0)
		{
		*speed = prevSpeed + deltaSpeed;
		}
	return NO_NEW_HISTORY;
	}

lastTime = lastFix->tim;
idx = p->numPts - 2;

do
	{
	elapsedTime = lastFix->tim - p->fix[idx].tim;

	if (elapsedTime >= setup->filter.time)
		haveTime = TRUE;

	j = 0;
	j += sprintf(buf+j , "%s:%d while loop\n", __FILE__, __LINE__);
	j += sprintf(buf+j , "                elapsedTime: %.1lf haveTime: %d\n", 
             elapsedTime, haveTime);
	j += sprintf(buf+j , "                p->fix[%d].tim: %.1lf\n", 
             idx, p->fix[idx].tim);
	j += sprintf(buf+j , "                lastFix->fix.tim: %.1lf\n", 
             lastFix->tim);

	} while ((--idx > 0) && (!haveTime));

idx++;
if (p->fix[idx].pos.lat == 0.0 || p->fix[idx].pos.lon == 0.0)
	{
	return BAD_POSITION;
	}

arclen(p->fix[idx].pos.lat,
       p->fix[idx].pos.lon,
       lastFix->pos.lat,
       lastFix->pos.lon,
       &distance,
       course,
       &bazm,
       ellipse->a,
       ellipse->b);

if (*course < 0.0)
	*course += TWOPI;

newSpeed    = (distance / elapsedTime) * MPS_TO_KTS;

j = 0;
j += sprintf(buf+j , "%s:%d\n", __FILE__, __LINE__);
j += sprintf(buf+j , "                dist: %.1lf time %.1lf spd: %.1lf", 
             distance, elapsedTime, newSpeed);


if (prevSpeed == 0.0)
	{
	*speed  = newSpeed;
	}
else
	{
	*speed = ((prevSpeed * (100.0 - setup->filter.weight)) + newSpeed * setup->filter.weight) / 100.0;
	deltaSpeed = *speed - newSpeed;
	}

prevSpeed = *speed;

return p->numPts;
}




/*----------------------------------------------------------------------------*/
// Return a double precision system time
/*----------------------------------------------------------------------------*/
double systemTime(void)	
{
struct timeb timebuffer;

ftime( &timebuffer );
return (double) timebuffer.time + (double) timebuffer.millitm / 1000.0;
}


/*----------------------------------------------------------------------------*/
// Return a pointer to a dump of a fix structure
/*----------------------------------------------------------------------------*/
char * dumpFix(t_fix * fix, char * buf)
{
int j = 0;
long ltime;
char latBuf[40];
char lonBuf[40];
char altBuf[40];

if (buf == (char *) NULL)
	{
	return (char *) NULL;
	}

if (fix == (t_fix *) NULL)
	{
	j += sprintf(buf+j, "NULL fix structure");
	}

PositionString(lonBuf, latBuf, altBuf, fix->pos.lon, fix->pos.lat, 0.0, 0);
j += sprintf(buf+j, "%s", asctime(gmtime(&ltime)));
j += sprintf(buf+j, "%s\n%s\n", latBuf, lonBuf);

return (char *) buf;
}

/*----------------------------------------------------------------------------*/
// Close the logbook file
/*----------------------------------------------------------------------------*/
void closeLogbook(void)
{
if (theLogBookFile == (FILE *) NULL)
	return;

fclose(theLogBookFile);
theLogBookFile = (FILE *) NULL;
}

/*----------------------------------------------------------------------------*/
// Log something to the logbook
/*----------------------------------------------------------------------------*/
void logMsg(char * msg)
{
t_setup * config = QuerySetup();

if (msg == (char *) NULL)
	return;

if ((theLogBookFile == (FILE *) NULL) && (logBookFileOK) )
	{
	if ((theLogBookFile = fopen(config->logFilename, "a+")) == NULL)
  	{
  	printf("\nCan't open file %s", config->logFilename);
  	return ;
  	}
	}

if (logBookFileOK)
	{
	fprintf(theLogBookFile, "%s %s\n", getTimeStr(0.0), msg);
	fflush(theLogBookFile);
	}

}
	



/*----------------------------------------------------------------------------*/
// return pointer to a string of time	
/*----------------------------------------------------------------------------*/
char *getTimeStr(double tim)		
{
static char *tim_string = "      999 99:99:99 ";
struct tm *thisTime;
time_t now ;
time_t ltim = (long) tim;
int millisec = (tim == 0.0) ? 0 : (int) (1000.0 * (tim - (double) ltim));

if (ltim == 0L)
	{
	now = time(NULL);
	thisTime = gmtime( &now );
	}
else
	{
	thisTime = gmtime( &ltim);
	}


sprintf(tim_string,"%03d %02d:%02d:%02d.%03d ",
      thisTime->tm_yday+1,
      thisTime->tm_hour, thisTime->tm_min, 
      thisTime->tm_sec,
      millisec);

return(tim_string);
}



/*----------------------------------------------------------------------------*/
// return SUCCESS if Nav structure is OK else an error Code
/*----------------------------------------------------------------------------*/
int validNav(t_nav * p)
{
double age;
time_t now = time(NULL);
double dNow = (double) now;

// See if it's NULL
if (p == (t_nav *) NULL)
	{
	return NULL_NAV;
	}

// see if its been updated recently
if ((age = dNow - p->tim) > OLD_NAV_AGE)
	{
	return OLD_NAV_DATA;
	}

if (p->lat > MAX_LAT || p->lat < MIN_LAT || p->lat == 0.0)
	{
	return BAD_LATITUDE;
	}

if (p->lon > MAX_LON || p->lon < MIN_LON || p->lon == 0.0)
	{
	return BAD_LONGITUDE;
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// compare two positions
/*----------------------------------------------------------------------------*/
int isSamePos(t_pos * pos1, t_pos * pos2)
{
if (validPos(pos1) != SUCCESS)
	return FALSE;

if (validPos(pos2) != SUCCESS)
	return FALSE;

if ((pos1->lat == pos2->lat) &&
    (pos1->lon == pos2->lon))
	return TRUE;

return FALSE;
}




/*----------------------------------------------------------------------------*/
// return SUCCESS if pos structure is OK else an error Code
/*----------------------------------------------------------------------------*/
int validPos(t_pos * p)
{
if (p->lat > MAX_LAT || p->lat < MIN_LAT || p->lat == 0.0)
	{
	return BAD_LATITUDE;
	}

if (p->lon > MAX_LON || p->lon < MIN_LON || p->lon == 0.0)
	{
	return BAD_LONGITUDE;
	}

return SUCCESS;
}

/*----------------------------------------------------------------------------*/
// return SUCCESS if fix structure is OK else an error Code
/*----------------------------------------------------------------------------*/
int validFix(t_fix * p)
{
return validPos(&p->pos);
}



/*----------------------------------------------------------------------------*/
// The old non universal grid to geo function from Snyder
/*----------------------------------------------------------------------------*/
void merc2geo(double x, double y, double * lat, double * lon)
{
int i;
double T;
double temp;
double sinlat;
t_ellips * ellipse = query_ellipsoid();

T = pow(2.718281828459045,(-y / ellipse->a));

/* initial approximation of lat  */
*lat = PI / 2.0 - 2.0 * atan(T);

/* iterate  */
for (i=0; i<5; i++)  
	{
	sinlat = sin(*lat);
	temp = pow((1.0 - ellipse->e2 * sinlat) / 
             (1.0 + ellipse->e2 * sinlat),
             ellipse->e2 / 2.0);
	*lat = PI / 2.0 - 2 * atan(T * temp);
	}

/* solve for lon  */
*lon = x / ellipse->a - PI;

return;

}


/*----------------------------------------------------------------------------*/
// taken from Map Projections used by the USGS, John Snyder
// Bulletin # 1532, pages 63-70
/*----------------------------------------------------------------------------*/
void geo2merc(double lat, double lon, double * x, double * y)
{
double temp1;		/* double work variables */
double sinlat = sin(lat);
double tanlat = tan(PI / 4.0 + lat / 2.0);
t_ellips * ellipse = query_ellipsoid();

temp1 = pow((1.0 - ellipse->e2 * sinlat) / 
            (1.0 + ellipse->e2 * sinlat), 
            ellipse->e2 / 2.0);

*y = ellipse->a * log(tanlat * temp1);

if (lon < 0.0)
	{
	*x = ellipse->a * (lon + PI);
	}
else
	{
	*x = ellipse->a * (lon - PI);
	}

return;
}





/*----------------------------------------------------------------------------*/
// function computes projected latitude and longitude
// given an initial lat, lon and a range and bearing
// to new endpoint
// returns INVALID if invalid else returns 0
/*----------------------------------------------------------------------------*/
int	proj(double lat, double lon, double range, double bearing, 
         double * newlat, double * newlon, double a, double b)
{
double eccen;			/* sqrt(1.0 - smb2 / sma2) */
double eccen2;			/* eccen * eccen 	*/
double epsi;			/* eccen2 / (1.0 - eccen2)	*/

double gcoslat, gcoslat2;
double gsinlat, gsinlat2;
double r2, r3, rad_curv, temp;
double theta, theta2, theta3;
double ratio;
double sinpsi, cospsi, sindl, tantu;
double dlbdr, tandl;
double sma2 = a * a;
double smb2 = b * b;

	
if (lat > MAX_LAT || lat < MIN_LAT || lat == 0.0)
	{
	return BAD_LATITUDE;
	}

if (lon > MAX_LON || lon < MIN_LON || lon == 0.0)
	{
	return BAD_LONGITUDE;
	}

eccen = sqrt(1.0 - smb2 / sma2);
eccen2 = eccen * eccen;
epsi = eccen2 / (1.0 - eccen2);

gcoslat = cos(lat);
gcoslat2 = gcoslat * gcoslat;
gsinlat = sin(lat);
gsinlat2 = gsinlat * gsinlat;
 
            /*	Formula 1 */
 
r2 = -1.0 * epsi * gcoslat2 * cos(bearing) * cos(bearing);
 
            /*	Formula 2 */
 
r3 = 3.0 * epsi * (1.0 - r2) * gcoslat * gsinlat * cos(bearing);
rad_curv = a / sqrt(1.0 - (eccen2 * gsinlat2));      /* radius curv */
temp = range / rad_curv;
 
            /*	Formula 3 */
 
theta = temp - ((r2 * (1.0 + r2) * (pow(temp,3))) / 6.0) - r3 * (1.0 + 3.0 * r2) * (pow(temp,4)) / 24.0;
theta2 = theta * theta;
theta3 = theta * theta * theta;
 
            /*	Formula 4 */
 
ratio = 1.0 - ((r2 * theta2) / 2.0) - r3 * theta3 / 6.0;
 
            /*	Formula 5 */
 
sinpsi = gsinlat * cos(theta) + gcoslat * cos(bearing) * sin(theta);
cospsi = sqrt(1.0 - sinpsi * sinpsi);
 
            /*	Formula 6 */
 
sindl = sin(bearing) * sin(theta) / cospsi;
 
            /*	Formula 7 */
 
tantu = (1.0 + epsi) * sinpsi / cospsi * (1.0 - (eccen2 * ratio * gsinlat / sinpsi));
 
*newlat = atan(tantu);
tandl = sindl / sqrt(1.0 - sindl * sindl);
dlbdr = atan(tandl);
*newlon = lon + dlbdr;
 
return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Return a formatted time pointer
/*----------------------------------------------------------------------------*/
char *vaxTime(double tim)	
{
static char *tim_string = "1991/157 14:03:54.14 ";
struct tm *thisTime;
struct timeb timebuffer;
time_t ltim = (long) tim;
int millisec = (tim == 0.0) ? 0 : (int) (100.0 * (tim - (double) ltim));

ftime( &timebuffer );
if (tim == 0L)
	{
	thisTime = gmtime( &(timebuffer.time) );
	}
else
	{
	thisTime = gmtime(&ltim);
	}

sprintf(tim_string,"%04d/%03d %02d:%02d:%02d.%02d",
        thisTime->tm_year+1900,
        thisTime->tm_yday+1,
        thisTime->tm_hour, 
        thisTime->tm_min, 
        thisTime->tm_sec,
        millisec);

return(tim_string);

}



/*----------------------------------------------------------------------------*/
// Return a formatted latitude string pointer
/*----------------------------------------------------------------------------*/
char *vaxLatStr(double lat)		
{
static char *string = "DDD MM SS.sssH ";
int LatDeg;
int LatMin;
char LatHem;
double LatSec;
int LatFrac;

LatHem = (char) ((rads_to_dms(lat, &LatDeg, &LatMin, &LatSec) > 0) ? 'N' : 'S');
LatFrac = (int) (1000.0 * (LatSec - floor(LatSec)));

sprintf(string,"%03d %02d %02d.%03d%c", LatDeg, LatMin, (int) LatSec, LatFrac, LatHem);

return string;
}

/*----------------------------------------------------------------------------*/
// Return a formatted longitude string pointer
/*----------------------------------------------------------------------------*/
char *vaxLonStr(double lon)		
{
static char *string = "DDD MM SS.sssH ";
int LonDeg;
int LonMin;
char LonHem;
double LonSec;
int LonFrac;

LonHem = (char) ((rads_to_dms(lon, &LonDeg, &LonMin, &LonSec) > 0) ? 'E' : 'W');
LonFrac = (int) (1000.0 * (LonSec - floor(LonSec)));

sprintf(string,"%03d %02d %02d.%03d%c", LonDeg, LonMin, (int) LonSec, LonFrac, LonHem);

return string;
}



/*----------------------------------------------------------------------------*/
// pad a buffer with spaces to specified length
/*----------------------------------------------------------------------------*/
int padBuffer(char buf[], int length)
{
int curLen = strlen(buf);
int charsAdded = 0;

while (curLen < length)
	{
	buf[curLen++] = ' ';
	charsAdded++;
	}

buf[length] = '\0';

return charsAdded;
}



/*---------------------------------------------------------------------------*/
// return pointer to NMEA 0183 lat string
/*---------------------------------------------------------------------------*/
char * nmeaLat(double lat)
{
static char *string = "DDMM.mm,H ";
int LatDeg;
int LatMin;
char LatHem;
double LatSec;
int LatFrac;

LatHem = (char) ((rads_to_dms(lat, &LatDeg, &LatMin, &LatSec) > 0) ? 'N' : 'S');
LatFrac = (int) (100.0 * (LatSec / 60.0));

sprintf(string,"%02d%02d.%02d,%c", LatDeg, LatMin, LatFrac, LatHem);
return string;
}


/*---------------------------------------------------------------------------*/
// return pointer to NMEA 0183 lon string
/*---------------------------------------------------------------------------*/
char * nmeaLon(double lon)
{
static char *string = "DDMM.mm,H ";
int LonDeg;
int LonMin;
char LonHem;
double LonSec;
int LonFrac;

LonHem = (char) ((rads_to_dms(lon, &LonDeg, &LonMin, &LonSec) > 0) ? 'E' : 'W');
LonFrac = (int) (100.0 * (LonSec / 60.0));

sprintf(string,"%03d%02d.%02d,%c", LonDeg, LonMin, LonFrac, LonHem);
return string;
}

/*----------------------------------------------------------------------------*/
// return the grid number and letter from the config file
/*----------------------------------------------------------------------------*/
int getUTMParam( int * grid_num, char * grid_ltr)
{
*grid_num = Setup.utm.zoneNum;
*grid_ltr = Setup.utm.zoneLtr;
return *grid_num;
}

/*----------------------------------------------------------------------------*/
// Set the grid number and letter in the config file
/*----------------------------------------------------------------------------*/
int setUTMParam( int grid_num, char grid_ltr)
{
Setup.utm.zoneNum = grid_num;
Setup.utm.zoneLtr = grid_ltr;
return grid_num;
}


/*----------------------------------------------------------------------------*/
// Compute the range and bearing to the passed site
/*----------------------------------------------------------------------------*/
int computeSiteRangeBearing(double lat, double lon, t_site * theSite)
{
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 
double bazm;

return arclen(lat, lon,
       theSite->pos.lat, theSite->pos.lon,
       &rngBrg.range,
       &rngBrg.bearing,
       &bazm,
       ellipse->a,
       ellipse->b);
}



/*----------------------------------------------------------------------------*/
// Insert theChar at the specified location in src and copy results into dest
// Return a pointer to dest
/*----------------------------------------------------------------------------*/
char * insertCharAtPos(char * src, int location, int theChar, char * dest)
{
int len = strlen(src);
int ch;
int i=0;

char *s = src;
char *d = dest;

// check for NULL source
if (src == (char *) NULL)
	return (char *) NULL;

// if the location is < 0 make it 0
if (location < 0)
	location = 0;

// if location specified is greater than string length, append the char to the 
// end of src
if (len < location)
	{
	strcpy(dest, src);
	*(dest+len) = (char) theChar;
	*(dest+len+1) = (char) '\0';
	return dest;
	}

// must be a normal copy
while (ch = *s++)
	{
	if (i++ == location)
		*d++ = (char) theChar;

	*d++ = (char) ch;
	}

// don't forget to terminate it
*d = '\0';

return dest;
}

/*----------------------------------------------------------------------------*/
// wait for for n ticks and then return;
/*----------------------------------------------------------------------------*/
void sleep(int ticks)
{
clock_t initTick = clock();
while (ticks + initTick > clock());
}



/*----------------------------------------------------------------------------*/
// take a char pointer to time and return the floating point equivalent
/*----------------------------------------------------------------------------*/
double getTimeDbl(char * timString)		
{
double d = 0.0;
int retv = 0;
struct tm * thisTime;
int mil = 0;
time_t now;

// Get rid of the common delimeters
WhiteOutChar(timString, ':');
WhiteOutChar(timString, '#');
WhiteOutChar(timString, '/');
WhiteOutChar(timString, '&');
WhiteOutChar(timString, '.');

// Get the current time so that we have a year value
now = time(NULL);
thisTime = gmtime( &now );

// parse out the time
retv = sscanf(timString, "%3d %2d %2d %2d %3d", 
                          &thisTime->tm_yday,
                          &thisTime->tm_hour, 
                          &thisTime->tm_min, 
                          &thisTime->tm_sec,
			                    &mil);

// get the month and day from the day of year and year
getMonthDay(thisTime->tm_year, thisTime->tm_yday, 
           &thisTime->tm_mon, &thisTime->tm_mday);

if (retv != 5)
	{
	return -1.0;
	}

now = mktime(thisTime);
return ((double) now + ((double) mil / 1000.0));
}




/*----------------------------------------------------------------------------*/
// take a char pointer to time	and return the flating point equivalent
/*----------------------------------------------------------------------------*/
int	getMonthDay(int year, int day_of_year, int * month, int * day_of_month)
{
int isLeapYear = FALSE;
static days_per_month[2][12] = {
	{31,28,31,30,31,30,31,31,30,31,30,31}, 	/* Normal year */
	{31,29,31,30,31,30,31,31,30,31,30,31}		/* Leap year */
				}; 
int dayTot = 0;        // Dos counts days since January 1 (0-365) range
int i      = 0;

// check for leap year
isLeapYear =  (year % 4 == 0 && year % 100 != 0 || year % 400 == 0);

while ((dayTot + days_per_month[isLeapYear][i]) < day_of_year)
	{
	dayTot += days_per_month[isLeapYear][i++];
	}

*month = i;
*day_of_month = day_of_year - dayTot;

return *day_of_month;
}


/*----------------------------------------------------------------------------*/
// rotate a bearing x degrees 
// Positive = Clockwise
// Negative = antiClockwise
/*----------------------------------------------------------------------------*/
double rotateBrg(double brgRad, double rotation)
{
return  fmod(brgRad + rotation, TWOPI);
}

/*----------------------------------------------------------------------------*/
// return Mouse Type and port
/*----------------------------------------------------------------------------*/
int qryMouse(int * type, int * irq)
{
union REGS inregs, outregs;

inregs.x.ax = 36;
int86(0x33, &inregs, &outregs);

*type = (int) outregs.h.ch;
*irq  = (int) outregs.h.cl;

return *type;
}


/*----------------------------------------------------------------------------*/
// replace the newline char at the end of a string with \0  
/*----------------------------------------------------------------------------*/
char * replaceNewline(char * s)
{
char *c;

c = strchr(s, '\n');
if(c != NULL)
	*c = '\0';
      
return(s);
}



/*----------------------------------------------------------------------------*/
// Report the amount of dik space available on the passed drive
/*----------------------------------------------------------------------------*/
unsigned long diskFree(unsigned int drive_nbr)
{
unsigned long bytes_cluster;
struct   diskfree_t dfinfo;
   
if ((_dos_getdiskfree( drive_nbr, &dfinfo )) == 0) 
	{
	bytes_cluster =dfinfo.sectors_per_cluster * dfinfo.bytes_per_sector;
	return dfinfo.avail_clusters * bytes_cluster;
	}

return 0L;
}





/*----------------------------------------------------------------------------*/
// return the number of the drive in the filename
// i.e. A: = 1 B: = 2 etc
/*----------------------------------------------------------------------------*/
unsigned int getDriveNumber(char * filename)
{
char drive[_MAX_DRIVE];
char dir[_MAX_PATH];
char fname[15];
char ext[15];

_splitpath(filename, drive, dir, fname, ext);
return toupper(drive[0]) - 'A'+ 1;
}

