/*----------------------------------------------------------------------------*/
// File: plot.c   USGS Navigation, plot historical ship track
// Blame: John Gann
// Start Date: 5 June 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <io.h>
#include <share.h>
#include <limits.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"
#include "utmlib.h"
#include "mnutils.h"
#include "lines.h"
#include "plot.h"
#include "filehndl.h"
#include "graph.h"
#include "fileio.h"
#include "serial.h"
#include "sensor.h"

// Private data
static FILE * plotFp = (FILE *) NULL;
static int plotHandle = -1;
static t_graf savePlotGraf;
static t_plot plot;
static double lastPlotTime = 0.0;
static long plotRecCnt = 0L;
static long plotEOF;
static plotterWasOpen = FALSE;

// Global data
int postPlotActive = FALSE;

// External data
extern t_graf graf;
extern char DisplayingSomething;
extern char RefreshRequired;
extern int graphWn;


/*----------------------------------------------------------------------------*/
// Call the appropriate plotter function
/*----------------------------------------------------------------------------*/
int doPlotFm(ob_desc * ob)
{
if (postPlotActive)
	{
	return doTermPlotFm(ob);
	}
else
	{
	return doPlotCfgFm(ob);
	}
}

/*----------------------------------------------------------------------------*/
// do the Plot setup form
/*----------------------------------------------------------------------------*/
int doPlotCfgFm(ob_desc * ob)
{
int retv;
fm_desc * fm;
char * frf = "plot_cfg.frf";
ftag * intervalTag;
ftag * startTag;
ftag * stopTag;
ftag * scaleTag;
ftag * latTag;
ftag * lonTag;
ftag * fileTag;
t_setup * config = QuerySetup();
ltag * l_tag;             // field tag pointer
char x_buf[40];
char y_buf[40];
int zone_num;
char zone_ltr;

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// copy the current plot config to a local copy
memcpy((void *) &plot, (void *) &config->plot, sizeof(t_plot));

// Initialize UTM stuff
zone_num = grid_zone_EW(plot.center.lon);
zone_ltr = grid_zone_NS(plot.center.lat);

if (plot.destination == PLOT_SCREEN)
	{
	fm->ob_lst[2].flags |=  IS_CHECKED;
	fm->ob_lst[3].flags &= ~IS_CHECKED;
	}
else
	{
	fm->ob_lst[3].flags |=  IS_CHECKED;
	fm->ob_lst[2].flags &= ~IS_CHECKED;
	}

// Initialize the ltag to the correct sensor 
l_tag = (ltag *) fm->ob_lst[5].tag;
l_tag->curitem = plot.sensor_idx;

// update the fix interval
intervalTag = (ftag *) fm->ob_lst[4].tag;
sprintf(intervalTag->fld, "%d", plot.interval);

// update the start time
startTag = (ftag *) fm->ob_lst[6].tag;
strcpy(startTag->fld, getTimeStr(plot.start));

// update the stop time
stopTag = (ftag *) fm->ob_lst[7].tag;
strcpy(stopTag->fld, getTimeStr(plot.stop));

// update the scale
scaleTag = (ftag *) fm->ob_lst[8].tag;
sprintf(scaleTag->fld, "%ld", plot.scale);

// update the filename
fileTag = (ftag *) fm->ob_lst[11].tag;
fm->ob_lst[11].flags |= IS_EXECUTABLE;
fm->ob_lst[11].flags &= ~IS_EDITABLE;
fm->ob_lst[11].task = getPlotFileName;
sprintf(fileTag->fld, "%s", plot.filename);

OpenForm ( fm, 10, 40); 

// Initialize the form with the default plot coordinates
UpdatePositionEntryFields(fm,                          // the form
                           4,                          // pos offset on form
                           plot.center.lon,    // lat radian value
                           plot.center.lat);   // lon radian value

// Set up task to handle position entry fields
SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			/* See if all of the fields are valid */
			latTag = (ftag *) fm->ob_lst[9].tag;
			strcpy(y_buf, latTag->fld);
			lonTag = (ftag *) fm->ob_lst[10].tag;
			strcpy(x_buf, lonTag->fld);
			if (PositionXYToRads(y_buf, x_buf, 
                           &plot.center.lat, &plot.center.lon, 
                           zone_num, zone_ltr))
				{
				retv = 99;        /* Stay in LOOP */
				break;
				}

			// Get the site name
			plot.interval = atoi(intervalTag->fld);
			plot.destination = (fm->ob_lst[2].flags & IS_CHECKED) ?
                                 PLOT_SCREEN : PLOT_PLOTTER;
			plot.sensor_idx = l_tag->curitem;
			plot.start = getTimeDbl(startTag->fld);
			plot.stop  = getTimeDbl(stopTag->fld);
			plot.scale  = atol(scaleTag->fld);
			strcpy(plot.filename, fileTag->fld);

			// if everything is OK copy it to the config file
			memcpy((void *) &config->plot, (void *) &plot, sizeof(t_plot));

			// start the plotting procedure
			initPlot();
			break;

		case 1:                                  // CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL && retv != 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

// Queue up an Escape message
QueueMsg(CreateMsg(ESC_MSG));

return retv;
}



/*----------------------------------------------------------------------------*/
// dump the plot struct
/*----------------------------------------------------------------------------*/
void dumpScale(t_plot *plot)
{
char buf[1024];
int j = 0;

j += sprintf(buf+j, "Plotter Interval: %d\n", plot->interval);
j += sprintf(buf+j, "Plotter destination: %d\n", plot->destination);
j += sprintf(buf+j, "Plotter sensor: %d\n", plot->sensor_idx);
j += sprintf(buf+j, "Plotter start: %.1lf\n", plot->start);
j += sprintf(buf+j, "Plotter stop: %.1lf\n", plot->stop);
j += sprintf(buf+j, "Plotter scale: %ld\n", plot->scale);
j += sprintf(buf+j, "Plotter center lat: %.8lf\n", plot->center.lat);
j += sprintf(buf+j, "Plotter center lon: %.8lf\n", plot->center.lon);
j += sprintf(buf+j, "Plotter file: %s\n", plot->filename);
j += sprintf(buf+j, "Plotter center_UTM X: %.2lf Y: %.2lf\n", 
                          plot->center_utm.x, plot->center_utm.y);
j += sprintf(buf+j, "Plotter center X: %5d Y: %5d\n", 
                          plot->HPcfg.center.X, plot->HPcfg.center.Y);
j += sprintf(buf+j, "Plotter p1 X: %5d Y: %5d\n", 
                          plot->HPcfg.p1.X, plot->HPcfg.p1.Y);
j += sprintf(buf+j, "Plotter p2 X: %5d Y: %5d\n", 
                          plot->HPcfg.p2.X, plot->HPcfg.p2.Y);
j += sprintf(buf+j, "Plotter xres: %.8lf\n", plot->xres);
j += sprintf(buf+j, "Plotter yres: %.8lf\n", plot->yres);

// logMsg(buf);
NotifyUser(-1, -1, "Yo", buf);
}



/*----------------------------------------------------------------------------*/
// Initialize system for plotting history to screen or hardcopy
/*---------------------------------------------------------------------------*/
int initPlot(void)
{
int retv;

// open the plotfile
if ((retv = openPlotFile()) != SUCCESS)
	{
 	NotifyUser(-1,-1,plot.filename, "Can't open plot file" );
	}

// Initialize the static variable for interval checking
lastPlotTime = 0.0;

// Initialize the static plotRecCnt
plotRecCnt = 0L;

// Save the end of file offset
fseek(plotFp, 0L, SEEK_END);
plotEOF = ftell(plotFp);
rewind(plotFp);

if (plot.destination == PLOT_SCREEN)
	{
	retv = initScreenPlot();
	}
else
	{
	retv = initHPplot();
	}

// Don't let the system draw for a while
if (retv == SUCCESS)
	postPlotActive = TRUE;

return retv;
}


/*----------------------------------------------------------------------------*/
// Get File logging name object task
/*----------------------------------------------------------------------------*/
int getPlotFileName(ob_desc * ob)
{
char filename[_MAX_PATH];
ftag * tag;

// copy the current field contents to filename
tag = (ftag *) ob->tag;
strcpy(filename, tag->fld);

if ( get_input_file(filename, "Enter Plot Filename", filename) )
	{
	strcpy(tag->fld, filename);
	DrawTaggedOb(ob);
	}

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// Open the plotfile
// returns SUCCESS if OK else a negativ error code
/*----------------------------------------------------------------------------*/
int openPlotFile(void)
{
// Open the file in share mode because it may be the one we are writing to
plotHandle = sopen( plot.filename, 
                O_BINARY | O_RDONLY,
                SH_DENYNO, S_IREAD);

if (plotHandle == -1)
	{
 	return BAD_FILE_OPEN;
	}

if ((plotFp = fdopen(plotHandle, "rb")) == (FILE *) NULL)
	{
 	return BAD_FILE_OPEN;
 	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Close the plotfile
/*----------------------------------------------------------------------------*/
void closePlotFile(void)
{
if (plotFp != (FILE *) NULL)
	{
	fclose(plotFp);
	plotHandle = -1;
	plotFp = (FILE *) NULL;
	}
}

/*----------------------------------------------------------------------------*/
// Terminate the post plotting function
/*---------------------------------------------------------------------------*/
void termPlot(void)
{
char buf[80];

closePlotFile();
postPlotActive = FALSE;
DisplayingSomething = FALSE;

sprintf(buf, "%ld Recs Plotted\n", plotRecCnt);
scrollMsg(buf);

// restore original grafics mode
if (plot.destination == PLOT_SCREEN)
	{
	memcpy((void *) &graf, (void *) &savePlotGraf, sizeof(t_graf));

	// force the screen to refresh
	RefreshRequired = TRUE;
	}
else
	{
	sendPlotter("SP0;");               // put the pen away
	closePlotterPort();
	}
}


/*----------------------------------------------------------------------------*/
// Read from the plotfile and update the screen
/*----------------------------------------------------------------------------*/
int doPostPlotting(void)
{
t_nav record;
clock_t start = clock();
clock_t now;
double elapsedTime;
int retv;

// keep reading for MAX_CLOCK_TICKS clock ticks
do 
	{
	// get the next record in th file
	if ((retv = readNavRec(plotFp, &record)) != SUCCESS)
		{
		termPlot();
		return SUCCESS;
		}

	// Is it time to stop ??
	if (record.tim > (double) plot.stop)
		{
		// printf("record Time %s\n", getTimeStr(record.tim));
		// printf("stop time %s\n", getTimeStr(plot.stop));
		termPlot();
		return SUCCESS;
		}

	// does this record qualify??
	// check the record type
	if (record.id == (unsigned char) plot.sensor_idx)
		{
		// check the time interval
		elapsedTime = record.tim - lastPlotTime;
		if (elapsedTime >= (double) plot.interval)
			{
			postPlotNav(&record);
			plotRecCnt++;
			lastPlotTime = record.tim;
			}
		}

	now = clock();
	} while (clock() - start < MAX_PLOT_TICKS);

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Post plot the nav record
/*----------------------------------------------------------------------------*/
int postPlotNav(t_nav * rec)
{
if (plot.destination == PLOT_SCREEN)
	{
	drawPostTrack(rec->lat, rec->lon, BLACK);
	}
else
	{
	if (plotRecCnt == 0L)
		{
		plotMoveLatLon(rec->lat, rec->lon, TRUE);
		}
	else
		{
		plotMoveLatLon(rec->lat, rec->lon, FALSE);
		}
	}
return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// Fetch the Last nav Record
/*----------------------------------------------------------------------------*/
int getLastNavRec(FILE * fp, t_nav * rec)
{
long origPos;
int retv = SUCCESS;

// see if we are near the end of file
if (fp == (FILE *) NULL)
	{
	scrollMsg("NULL Nav File\n");
	return PLOT_FILE_NULL;
	}

// save the current file position
origPos = ftell(fp);
printf("plotEOF = %ld\n", plotEOF);
printf("origPos = %ld\n", origPos);

// go to the end of file
fseek(fp, - (long) sizeof(t_nav), SEEK_END);
printf("lastRecPos = %ld\n", ftell(fp));

// read the last record in the file
if ((retv = readNavRec(fp, rec)) != SUCCESS)
	{
	termPlot();
	}

// reset the file pointer to its original position
fseek(fp, origPos, SEEK_SET);

return retv;
}




/*----------------------------------------------------------------------------*/
// read the current record in the Nav file with Erro checking
/*----------------------------------------------------------------------------*/
int readNavRec(FILE * fp, t_nav * navRec)
{
#define END_OF_FILE -100
int k;

// see if we are near the end of file
if (fp == (FILE *) NULL)
	{
	scrollMsg("NULL Nav File\n");
	return PLOT_FILE_NULL;
	}

k = fread( (void *) navRec, sizeof(t_nav), 1, fp);
if (k != 1)
	{
	if (feof(fp))
		{
		return END_OF_FILE;
		}
	else
		{
		printf("Read failed %d in %s:%d\n", k, __FILE__, __LINE__);
		return BAD_DATA_READ;
		}
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Draw the post plot trackline
/*----------------------------------------------------------------------------*/
void drawPostTrack(double lat, double lon, int color)
{
int x, y;
static int lastX, lastY;
int lastWn = QryActiveWn();

// Set up graphics
SetActiveWn(graphWn, 0);
SetClipArea();

pushGrEnv();

PenSize( 1, 1 );
PenMode(zREPz);
PenPattern( 1 );
PenDash( 0 );
HideCursor();
PenColor(color);

// get screen coords
latLonToScreen(lat, lon, &x, &y);
if (plotRecCnt == 0L)
	{
	MoveTo(x, y);
	}
else
	{
	MoveTo(lastX, lastY);
	}

LineTo(lastX = x, lastY = y);
	
popGrEnv();

SetActiveWn(lastWn, 0);
ShowCursor();
}



/*----------------------------------------------------------------------------*/
// Give User opportunity to stop the plot
/*----------------------------------------------------------------------------*/
int doTermPlotFm(ob_desc * ob)
{
if (!QueryUser( -1, -1, "Yo",     
          "Do You want to stop Plotting?" ))
	{
	termPlot();
	}

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// Set up a screen plot
/*----------------------------------------------------------------------------*/
int initScreenPlot(void)
{
int lastWn = QryActiveWn();

// Save the current graphics setup
memcpy((void *) &savePlotGraf, (void *) &graf, sizeof(t_graf));

// Set the new scale
graf.scale = plot.scale;
updateScale();

// Set the new screen center
ClearGraphicsArea();

// set the active window to 0
SetActiveWn(0, 0);
setCenter(plot.center.lat, plot.center.lon);
SetActiveWn(lastWn, 0);

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Set up a HPGL Plot 
/*----------------------------------------------------------------------------*/
int initHPplot(void)
{
int result;
char buf[256];
double scale_factor;
static int computeNewZone = TRUE;
int grid_num;
char grid_ltr;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps

getUTMParam(&grid_num, &grid_ltr);

ll_to_utm(plot.center.lat, 
          plot.center.lon,                             // center pos
          &plot.center_utm.y, &plot.center_utm.x,      // returns UTM xy 
          &scale_factor,                               // returns scale factor
          &grid_num,                                   // returns zone number
          &grid_ltr,                                   // returns zone letter
          USE_PASSED_ZONE,                             // compute new zone
          ellipse->a,                                  // semi major axis
          ellipse->e2);                                // semi minor of WGS84

// Open the plotter port if not already open
plot.port = queryComByNav(HP_PLOTTER);
if (plot.port == (t_com *) NULL)
	{
	plotterWasOpen = FALSE;
	// Let the User assign the plotter port using the sensor index
	if ((result = doSensorForm((ob_desc *) NULL)))
		{
		termPlot();
		return result;
		}

	// check it again
	plot.port = queryComByNav(HP_PLOTTER);

	// See if a port is open and enabled for the plotter
	if (plot.port == (t_com *) NULL)
		{
		NotifyUser(-1, -1, "Yo", "Plotter port Not Enabled");
		return NULL_COM;
		}
	}
else          // plotter port is already open
	{
	plotterWasOpen = TRUE;
	}


// get P1 P2 from the plotter
if ((result = getP1P2()) != SUCCESS)
	{
	sprintf(buf,"initHPplot(): %d Can't Get P1 P2 at %s\n", 
               	result, plot.port->portname);
	NotifyUser(-1, -1, "Yo", buf);
	return BAD_PLOTTER_READ;
	}

// compute the resolution
plot.xres = (double) (plot.HPcfg.p2.X - plot.HPcfg.p1.X) / (double) plot.scale;
plot.yres = (double) (plot.HPcfg.p2.Y - plot.HPcfg.p1.Y) / (double) plot.scale;

// find center plotter 
plot.HPcfg.center.Y = plot.HPcfg.p1.Y + (plot.HPcfg.p2.Y - plot.HPcfg.p1.Y) / 2;
plot.HPcfg.center.X = plot.HPcfg.p1.X + (plot.HPcfg.p2.X - plot.HPcfg.p1.X) / 2;

// dumpScale(&plot);

// set the character size
sendPlotter("SI.2,.3;");

// draw the border
plotMoveXY(plot.HPcfg.p1.X, plot.HPcfg.p1.Y, PEN_UP);
plotMoveXY(plot.HPcfg.p1.X, plot.HPcfg.p2.Y, PEN_DN);
plotMoveXY(plot.HPcfg.p2.X, plot.HPcfg.p2.Y, PEN_DN);
plotMoveXY(plot.HPcfg.p2.X, plot.HPcfg.p1.Y, PEN_DN);
plotMoveXY(plot.HPcfg.p1.X, plot.HPcfg.p1.Y, PEN_DN);

// draw the title
plotLabelXY(plot.filename, 
            plot.HPcfg.p1.X,
            plot.HPcfg.p2.Y);
sendPlotter("CP;");

// draw the start time
plotLabel("Start: ");
plotLabel(getTimeStr(plot.start));
sendPlotter("CP;");

// draw the stop time
plotLabel(" Stop: ");
plotLabel(getTimeStr(plot.stop));
sendPlotter("CP;");

// draw the scale
plotLabel("Scale: ");
sprintf(buf, "%ld", plot.scale);
plotLabel(buf);
sendPlotter("CP;");

return SUCCESS;
}





/*----------------------------------------------------------------------------*/
// Send the string to the plotter with limited error checking
/*----------------------------------------------------------------------------*/
int  sendPlotter(unsigned char * buf)
{
unsigned char  ch;
unsigned char * p = buf;

// logMsg(buf);

while (ch = *p++)
	put_char(plot.port->portnum, ch);

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Query the plotter for P1 P2 locations for setting the resolution
/*----------------------------------------------------------------------------*/
int  getP1P2(void)
{
int result;
char buf[128];

sendPlotter("IN;OP;");

if ((result = readPlotter()) <= SUCCESS)
	return result;

scrollMsg(plot.port->buf);
WhiteOutChar(buf, ',');
result = sscanf(plot.port->buf, "%d, %d, %d, %d", &plot.HPcfg.p1.X, 
                             &plot.HPcfg.p1.Y, 
                             &plot.HPcfg.p2.X, 
                             &plot.HPcfg.p2.Y);

sprintf(buf, "result = %d\np1: %d %d\np2: %d %d", result,
                             plot.HPcfg.p1.X, 
                             plot.HPcfg.p1.Y, 
                             plot.HPcfg.p2.X, 
                             plot.HPcfg.p2.Y);
// NotifyUser(-1, -1, "Yo", buf);
sendPlotter("SP1;");

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Query the plotter for P1 P2 plotter resolution
/*----------------------------------------------------------------------------*/
int  getPlotterRes(void)
{
int result;
char buf[128];

sendPlotter("OF;");

if ((result = readPlotter()) <= SUCCESS)
	return result;

scrollMsg(plot.port->buf);
WhiteOutChar(buf, ',');
result = sscanf(plot.port->buf, "%d %d", &plot.HPcfg.xRes, 
                             &plot.HPcfg.yRes);

NotifyUser(-1, -1, "Yo", buf);

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// Disable the plotter if it was not previously enabled
/*----------------------------------------------------------------------------*/
int closePlotterPort(void)
{
if (!plotterWasOpen)
	{
	if (plot.port != (t_com *) NULL)
		{
		disable_one_port(plot.port->portnum);
		plot.port->enabled = FALSE;
		plot.port = (t_com *) NULL;
		}
	}
return SUCCESS;
}





/*----------------------------------------------------------------------------*/
// Read the plotter port
/*----------------------------------------------------------------------------*/
int readPlotter(void)
{
int bytesWaiting;
char ch = 1;
int i = 0;

if (plot.port == (t_com *) NULL)
	{
	return NULL_COM;
	}

bytesWaiting = iq_check(plot.port->portnum);
if (bytesWaiting <= 0)
	{
	return BAD_PLOTTER_READ;
	}

while(ch != CR && ch > 0)
	{
	ch = (char) get_char(plot.port->portnum);
	// printf("%d ", ch);
	if (ch < 0 || ch == CR)
		{
		break;
		}
	else
		{
		plot.port->buf[i++] = ch;
		}
	}

// Null terminate it
plot.port->buf[i] = '\0';

return i;
}

/*----------------------------------------------------------------------------*/
// Convert latitude longitude to plotter coordinates
/*----------------------------------------------------------------------------*/
int lat_lon_to_plotter(double lat, double lon, int * x, int * y)
{
double scale_factor;
double utmX, utmY;
int retval = TRUE;     // Assume that the coords are within the window
double dx, dy;
double half_scale =  (double) plot.scale / 2.0;
long lx, ly;
int grid_num;
char grid_ltr;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps

getUTMParam(&grid_num, &grid_ltr);

// get the UTM coordinate of the passed lat and lon
ll_to_utm(lat, 
          lon,
          &utmY, &utmX,                                // returns UTM xy 
          &scale_factor,                               // returns scale factor
          &grid_num,                                   // returns zone number
          &grid_ltr,                                   // returns zone letter
          USE_PASSED_ZONE,                             // compute new zone
          ellipse->a,                                  // semi major axis
          ellipse->e2);                                // semi minor of WGS84

// compute the offset from the center of the screen
dx = plot.center_utm.x - utmX;
dy = plot.center_utm.y - utmY;

// check the deltas to see if we are within the window
if ((fabs(dx) > half_scale) || (fabs(dy) > half_scale))
	retval = FALSE;                                     // set up the return val

// convert the offset to screen coordinates in pixels
lx = (long) (dx * -plot.xres) + (long) plot.HPcfg.center.X;
ly = (long) (dy * -plot.yres) + (long) plot.HPcfg.center.Y;

// make sure they are still integers before assigning them
if (lx < (long) INT_MIN)
	lx = (long) INT_MIN;

if (lx > (long) INT_MAX)
	lx = (long) INT_MAX;

if (ly < (long) INT_MIN)
	ly = (long) INT_MIN;

if (ly > (long) INT_MAX)
	ly = (long) INT_MAX;

*x = (int) lx;
*y = (int) ly;

return retval;
}




/*----------------------------------------------------------------------------*/
// draw a label centered at the passed location
/*----------------------------------------------------------------------------*/
int plotLabelXY(char * label, int x, int y)
{
int len = strlen(label);
int charWidth = 5;

plotMoveXY(x - len * charWidth, y, PEN_UP);
return plotLabel(label);
}





/*----------------------------------------------------------------------------*/
// draw a label at the current pen position
/*----------------------------------------------------------------------------*/
int plotLabel(char * label)
{
char buf[256];

if (label == (char *) NULL)
	return -1;

sprintf(buf, "LB%s%c", label, ETX);
return sendPlotter(buf);
}



/*----------------------------------------------------------------------------*/
// move the plotter pen to a specified lat lon
/*----------------------------------------------------------------------------*/
int plotMoveLatLon(double lat, double lon, int penUp)
{
int x, y;

// get the plotter coords
lat_lon_to_plotter(lat, lon, &x, &y);

return plotMoveXY(x, y, penUp);
}


/*----------------------------------------------------------------------------*/
// return the destination of the active plot
/*----------------------------------------------------------------------------*/
int isScreenPlot(void)
{
return !(plot.destination);
}


/*----------------------------------------------------------------------------*/
// move the plotter pen to a specified x and y
/*----------------------------------------------------------------------------*/
int plotMoveXY(int x, int y, int penUp)
{
char buf[80];

if (penUp)
	{
	sprintf(buf, "PU;PA%0d,%0d;PD;PU;", x, y);
	}
else
	{
	sprintf(buf, "PD;PA%0d,%0d;PU;", x, y);
	}

return sendPlotter(buf);
}

