#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <time.h>
#include <conio.h>

#include "navdefs.h"
#include "navtypes.h"

static t_setup setup;
static char * fixSymbolNames[] = 
							{
							"Dot",
							"Cross",
							"The X"
							};

void main(void);
int read_setup(void);

void main(void)
{
read_setup();
}




/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int read_setup(void)
{
FILE *fp;
int  i, k;

if ((fp = fopen(CFG_FILE_NAME, "rb")) == NULL)
  {
  printf("\nCan't open file %s", CFG_FILE_NAME);
  fclose(fp);
  return FALSE;
  }


k = fread( (char *) &setup, sizeof(t_setup), 1, fp);
if (k == -1)
	{
	printf("Read failed %d\n", k);
	exit(1);
	}

printf("Initialized       %d\n", setup.Initialized);
printf("Coordinate System %d\n", setup.CoordSys);
printf("Angle Type %d\n", setup.AngleType);
printf("Horizontal Units %d\n", setup.HorizUnits);
printf("Decimal Accuracy %d\n", setup.DecimalAccuracy);
printf("Datum Index  %d\n", setup.Datum);
printf("Gyro Index  %d\n", setup.Gyro);
printf("Display Index  %d\n", setup.DisplayType);
printf("Side Panel %d\n", setup.SidePanelType);

// logging
printf("\nLog Disk Drive         %d\n", setup.logDisk);
printf("Binary LoggingEnabled  %d\n", setup.BinLoggingEnabled);
printf("Ascii Logging Enabled  %d\n", setup.AsciiLogEnabled);
printf("Ascii Logging Interval  %d\n", setup.AsciiLogInterval);

printf("Current Line  %s\n", setup.curLine.name);
printf("   %.6lf", setup.curLine.start.lat * RAD_TO_DEG);
printf("   %.6lf\n", setup.curLine.start.lon * RAD_TO_DEG);
printf("   %.6lf", setup.curLine.end.lat * RAD_TO_DEG);
printf("   %.6lf\n", setup.curLine.end.lon * RAD_TO_DEG);
printf("Current Site  %s\n", setup.curSite.name);
printf("   %.6lf", setup.curSite.pos.lat * RAD_TO_DEG);
printf("   %.6lf\n", setup.curSite.pos.lon * RAD_TO_DEG);
printf("Ship File  %s\n", setup.ShipName);
printf("Fish File  %s\n", setup.FishName);
printf("Coastline File  %s\n", setup.CoastName);
printf("Binary Nav %s\n", setup.binFilename);
printf(" Ascii Nav %s\n", setup.ascFilename);
printf("   Logbook %s\n", setup.logFilename);
printf("Filter:   Time:%.0lf    Weight:%d\n", 
               setup.filter.time, 
               setup.filter.weight);
printf("Printer:   %s enabled to write every %d seconds\n", 
               (setup.printer.enabled) ? "is" : "is Not", 
               setup.printer.interval);
printf("     idx         = %d\n", setup.printer.idx);
printf("     name        = %s\n", setup.printer.name);
printf("     xScale      = %.2lf\n", setup.printer.xScale);
printf("     yScale      = %.2lf\n", setup.printer.yScale);

// plotter constants
printf("Plotter Interval: %d\n", setup.plot.interval);
printf("Plotter destination: %d\n", setup.plot.destination);
printf("Plotter sensor: %d\n", setup.plot.sensor_idx);
printf("Plotter start: %.1lf\n", setup.plot.start);
printf("Plotter stop: %.1lf\n", setup.plot.stop);
printf("Plotter scale: %ld\n", setup.plot.scale);
printf("Plotter center lat: %.8lf\n", setup.plot.center.lat);
printf("Plotter center lon: %.8lf\n", setup.plot.center.lon);
printf("Plotter file: %s\n", setup.plot.filename);
printf("Plotter center_UTM X: %.2lf Y: %.2lf\n", 
                          setup.plot.center_utm.x, setup.plot.center_utm.y);
printf("Plotter center X: %5d Y: %5d\n", 
                          setup.plot.HPcfg.center.X, setup.plot.HPcfg.center.Y);
printf("Plotter p1 X: %5d Y: %5d\n", 
                          setup.plot.HPcfg.p1.X, setup.plot.HPcfg.p1.Y);
printf("Plotter p2 X: %5d Y: %5d\n", 
                          setup.plot.HPcfg.p2.X, setup.plot.HPcfg.p2.Y);
printf("Plotter xres: %.8lf\n", setup.plot.xres);
printf("Plotter yres: %.8lf\n", setup.plot.yres);

printf("Vrp:   dist = %.1lf   bearing = %.1lf\n", setup.Vrp.dist, 
           RAD_TO_DEG * setup.Vrp.bearing);

printf("utm:   %02d%c\n", setup.utm.zoneNum, setup.utm.zoneLtr);
printf("foreground:  %02d\n", setup.fg);
printf("background:  %02d\n", setup.bg);
printf("depth Correction:  %02d\n", setup.depth.depthCorrectionEnabled);
printf("soundVelocity:  %.2lf\n", setup.depth.soundVelocity);
printf("Displayed Sensors:\n");
for (i=0; i<MAX_COMPORTS; i++)
	{
	printf("%2d ", setup.displaySensors[i]);
	}
printf("\n");

printf("\nRelays:\n");
printf("Relay Board is %s\n", (setup.relay.enabled) ? "Enabled" : "Disabled");
printf("Base Address %X\n", setup.relay.address);
printf("Board status %X\n", setup.relay.status);
for (i=0; i<MAX_RELAYS; i++)
	{
	printf("Relay %d is %s  Interval %d lastClosure %ld\n", 
                     i, 
                     (setup.relay.relaysEnabled[i]) ? "on " : "off",
                     setup.relay.relayIntervals[i],
                     setup.relay.lastTime[i]);
	}
printf("\n");

// video
printf("Video %s enabled\n", (setup.videoEnabled) ? "is" : "is not");
printf("\n");

// miscellaneous
printf("shipCentering is %s\n", (setup.misc.shipCentering) ? "On" : "Off");
printf("shipScaling is %s\n",   (setup.misc.shipScaling) ? "On" : "Off");
printf("drawCoastline is %s\n", (setup.misc.showCoastline) ? "On" : "Off");
printf("fixSymbol is %s\n", fixSymbolNames[setup.misc.fixSymbol]);

fclose(fp);
return TRUE;
}




