/*----------------------------------------------------------------------------*/
/* File: sensor.c  A Menuet version of an RS232 Parameter entry screen        */
/* Author: John Gann                                                          */
/* Start Date: Feb 11 1992                                                    */
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"      // constants
#include "navtypes.h"

#include "nav.h"            // prototypes
#include "sensor.h"         // prototypes
#include "serial.h"         // prototypes
#include "graph.h"          // prototypes
#include "mnutils.h"       // prototypes
#include "nav_util.h"       // prototypes

#define SENSOR_LENGTH 30
#define RS232_LENGTH 16
#define OKAY  0
#define DEFAULT 1
#define CANCEL 2
#define HELP 3

#define SENSOR_LENGTH 30
#define RS232_LENGTH 16
#define NEXT 1
#define PREVIOUS 14
#define SAVE 2
#define BYEBYE 3
#define MAX_INTERVAL  300
#define MIN_INTERVAL  0

// Global data 
t_com com_struct[MAX_COMPORTS];
t_nav navSensors[MAX_COMPORTS];

// private data
static int	ActiveComPorts=-1;
static fm_desc * rs232_fm;       // form pointer
static char * UNABLE_TO_LOAD_MSG = "Unable to load File";
static char * DYNAMIC_MSG = " Dynamic ";
static char * STATIC_MSG =  " Static  ";

/* external data */
extern char _Debug;
extern char _LoggingActive;
extern FILE * LOG_fp;
extern fm_desc * mainMenu;
extern char d_chars[];
extern char s_chars[];
extern char p_chars[];
extern t_nav * curNav;
extern t_com * curCom;

/*----------------------------------------------------------------------------*/
/* Read the com file into a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int read_com_file(char * filename)
{
FILE *fp;
int i, k;

if ((fp = fopen(filename, "rb")) == NULL)
	{
	printf("\nCan't open config file %s", filename);
	fclose(fp);
	return FALSE;
	}

for (i= 0; i < MAX_COMPORTS; i++)
	{
	k = fread( (char *) &com_struct[i], sizeof(t_com), 1, fp);
	if (k == -1)
		{
		fclose(fp);
		printf("\nfread failed");
		return FALSE;
		}
	}

fclose(fp);
return TRUE;
}



/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int write_com_file(char * filename)
{
FILE *fp;
int i, k;

if ((fp = fopen(filename, "wb")) == NULL)
	{
	printf("\nCan't open file %s", filename);
	fclose(fp);
	return FALSE;
	}

for (i= 0; i < MAX_COMPORTS; i++)
	{
	k = fwrite( (char *) &com_struct[i], sizeof(t_com), 1, fp);
	if (k == -1)
		{
		fclose(fp);
		printf("\nfwrite failed");
		return FALSE;
		}
	}

fclose(fp);
return TRUE;
}




/*----------------------------------------------------------------------------*/
// Attempt to open all the file pointers and return the number actually opened
/*----------------------------------------------------------------------------*/
int how_many_com_ports(void)   /* find number of com ports */
{
register int i, result;
register int item = 0;
int num = 0;

if (ActiveComPorts != -1)
	{
	return ActiveComPorts;
	}

for (i=0; i<MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled)
		{
		result = set_baud(i, (int) 0x2580, (unsigned char) 'N', (unsigned char) 8, (unsigned char) 1, 0);
		if (!result)
			{
			num++;
			}
		}
	}
	
return num;
}
	
	

/*----------------------------------------------------------------------------*/
// Initialize serial IO
/*----------------------------------------------------------------------------*/
void init_serial_io(void)
{
char buf[80];
int i = 0;
int result = 0;

// Open all the file pointers
if (ActiveComPorts == -1)
	{
	close_com_ports();
	ActiveComPorts = how_many_com_ports();
	}

/* initialize indices */
for (i=0; i < MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled) 
		{
		result = set_baud((unsigned int) i, 
                      com_struct[i].baud, 
                      com_struct[i].parity,
                      com_struct[i].data,
                      com_struct[i].stop,
                      com_struct[i].handshake);
		if (result)	
			{
			com_struct[i].enabled = 0;
			com_struct[i].sensor_index = 99;
			sprintf(buf,"set_baud(): %d Can't Open %s", 
                  result, com_struct[i].portname);
			NotifyUser(-1, -1, __FILE__, buf);
			}
		}
	}
return;
}



/*----------------------------------------------------------------------------*/
// Close any open com ports
/*----------------------------------------------------------------------------*/
void close_com_ports(void)
{
ActiveComPorts = -1;
}




/*---------------------------------------------------------------------------*/
// Set up the form with the indexed structure
/*---------------------------------------------------------------------------*/
static void update_setup_form(int idx)
{
ftag * f_tag;             // field tag pointer
ltag * l_tag;             // field tag pointer
char buf[80];             // scratch area

// Point the ltag to the baud list in the form
l_tag = (ltag *) rs232_fm->ob_lst[12].tag;
sprintf(buf, "%d", com_struct[idx].baud);
l_tag->curitem = get_ltag_match((char *) buf, l_tag);
DrawTaggedOb(&rs232_fm->ob_lst[12]);

// Update the current sensor
l_tag = (ltag *) rs232_fm->ob_lst[13].tag;
l_tag->curitem = get_ltag_match(com_struct[idx].sensor_name, l_tag);
DrawTaggedOb(&rs232_fm->ob_lst[13]);

// update the interval field
f_tag = (ftag *) rs232_fm->ob_lst[17].tag;
sprintf(f_tag->fld, "%3d", com_struct[idx].interval);
DrawTaggedOb(&rs232_fm->ob_lst[17]);
}


/*---------------------------------------------------------------------------*/
// Set the parity baud and stop bit flags
/*---------------------------------------------------------------------------*/
static void set_all_flags(int idx)
{
// set the enabled flag
if (com_struct[idx].enabled)
	{
	set_flag_fm_group (rs232_fm, 9, 1 );
	}
else
	{
	set_flag_fm_group (rs232_fm, 9, 0 );
	}

switch (toupper(com_struct[idx].parity))
	{
	case 'N':
		set_flag_fm_group (rs232_fm, 1, 0 );
		break;

	case 'O':
		set_flag_fm_group (rs232_fm, 1, 1 );
		break;

	case 'E':
		set_flag_fm_group (rs232_fm, 1, 2 );
		break;
	}

switch (com_struct[idx].data)
	{
	case 7:
		set_flag_fm_group (rs232_fm, 2, 0 );
		break;

	case 8:
		set_flag_fm_group (rs232_fm, 2, 1 );
		break;
	}

switch (com_struct[idx].stop)
	{
	case 1:
		set_flag_fm_group (rs232_fm, 3, 0 );
		break;

	case 2:
		set_flag_fm_group (rs232_fm, 3, 1 );
		break;
	}

// Set handshake flags
set_flag_fm_group (rs232_fm, 4, com_struct[idx].handshake );
}


/*---------------------------------------------------------------------------*/
// Update the edit field
/*---------------------------------------------------------------------------*/
static void update_edit_field(int idx)
{
char * p_baud;
char * p_com;
ftag * f_tag;             // ftag pointer
ltag * l_tag;             // list tag pointer
unsigned char parity_flag;
unsigned char data_flag;
unsigned char stop_flag;

// get baud string
l_tag = (ltag *) rs232_fm->ob_lst[12].tag;
p_baud = l_tag->list[l_tag->curitem];

// get port string
p_com = com_struct[idx].portname;

parity_flag = (unsigned char) get_flag_fm_group(rs232_fm, 1);
data_flag = (unsigned char) get_flag_fm_group(rs232_fm, 2);
stop_flag = (unsigned char) get_flag_fm_group(rs232_fm, 3);

f_tag = (ftag *) rs232_fm->ob_lst[11].tag;
sprintf(f_tag->fld, "%s: %s, %c, %c, %c" ,p_com, p_baud, p_chars[parity_flag], d_chars[data_flag], s_chars[stop_flag]);
DrawTaggedOb(&rs232_fm->ob_lst[11]);
}



/*---------------------------------------------------------------------------*/
// Update the Com record 
/*---------------------------------------------------------------------------*/
static void update_com_record(int idx)
{
char * p_baud;
char * p_sensor;
ltag * l_tag;             // list tag pointer
ftag * f_tag;             // field tag pointer
unsigned char parity_flag;
unsigned char data_flag;
unsigned char stop_flag;
int result=0;
char buf[80];
unsigned char enabled;
int interval;
int handshake;

// get baud string
f_tag = (ftag *) rs232_fm->ob_lst[17].tag;
interval =  atoi(f_tag->fld);

// get baud string
l_tag = (ltag *) rs232_fm->ob_lst[12].tag;
p_baud = l_tag->list[l_tag->curitem];

// get sensor string
l_tag = (ltag *) rs232_fm->ob_lst[13].tag;
p_sensor = l_tag->list[l_tag->curitem];
com_struct[idx].sensor_index = l_tag->curitem;
// navSensors[idx].id = com_struct[idx].sensor_index;

parity_flag = (unsigned char) get_flag_fm_group(rs232_fm, 1);
data_flag = (unsigned char) get_flag_fm_group(rs232_fm, 2);
stop_flag = (unsigned char) get_flag_fm_group(rs232_fm, 3);
handshake = get_flag_fm_group(rs232_fm, 4);

com_struct[idx].baud = atoi(p_baud);
strcpy(com_struct[idx].sensor_name, p_sensor);

com_struct[idx].interval = min(interval, MAX_INTERVAL);
com_struct[idx].interval = max(interval, MIN_INTERVAL);
com_struct[idx].parity = (unsigned char) p_chars[parity_flag];
com_struct[idx].data = (unsigned char) (d_chars[data_flag] - 0x30);
com_struct[idx].stop = (unsigned char) (s_chars[stop_flag] - 0x30);
com_struct[idx].handshake = handshake;
com_struct[idx].psd_word = build_parity_byte( com_struct[idx].parity,
                                             com_struct[idx].data,
                                             com_struct[idx].stop);

// if the port has been enabled and was not already open, open it
enabled = (unsigned char) get_flag_fm_group(rs232_fm, 9);
if (enabled && (!com_struct[idx].enabled))
		{
		com_struct[idx].enabled = enabled;
		result = set_baud(com_struct[idx].portnum, 
             com_struct[idx].baud, 
             com_struct[idx].parity,
             com_struct[idx].data,
             com_struct[idx].stop,
             com_struct[idx].handshake);
		if (result)
			{
			sprintf(buf, "Failed to enable port %s (%d)\n", 
                    com_struct[idx].portname, 
                    result);
			NotifyUser(-1, -1, __FILE__, buf);
			com_struct[idx].enabled = FALSE;
			}
		}
	else
		{
		com_struct[idx].enabled = enabled;
		}

// Now reset the port
if (com_struct[idx].enabled)
	{
	result = set_baud( com_struct[idx].portnum, 
                   com_struct[idx].baud, 
                   com_struct[idx].parity,
                   com_struct[idx].data,
                   com_struct[idx].stop,
                   com_struct[idx].handshake);
	if (result)	
		{
		com_struct[idx].enabled = 0;
		sprintf(buf,"%s: (%d) Can't Set Baud, Parity etc on %s", 
               		"do_setup()", result, com_struct[idx].portname);
		NotifyUser(-1, -1, __FILE__, buf);
		}
	}

}



/*---------------------------------------------------------------------------*/
// Select a device and its RS232 Parameters 
/*---------------------------------------------------------------------------*/
int doSensorForm(ob_desc * ob)
{
static int idx=0;
int retv=0;
int rs232_retv=0;
char title[80];
char * frfFile = "rs232_x.frf";

if ( ( rs232_fm = loadFRF ( frfFile ) ) == (fm_desc *) NULL ) 
	{
	return BAD_FORM_OPEN;
	}

sprintf(title, "Select Parameters for %s", com_struct[idx].portname);
rs232_fm->title = (char *) title;

set_all_flags(idx);
OpenForm( rs232_fm, 10, 40);
update_setup_form(idx);
update_edit_field(idx);

do 	
	{
	rs232_retv = ProcessObList ( rs232_fm->ob_lst, rs232_fm->ob_cnt, 0 );
	switch ( rs232_retv )
		{
		case OKAY    : 
			update_com_record(idx);
			update_setup_form(idx);
			break;
				
		case PREVIOUS : 
			if (--idx < 0)
				idx = MAX_COMPORTS - 1;
			sprintf(title, "Select Parameters for %s", com_struct[idx].portname);
			rs232_fm->title = (char *) title;
			set_all_flags(idx);
			update_setup_form(idx);
			DrawForm ( rs232_fm );
			break;
		
		case NEXT : 
			if (++idx >= MAX_COMPORTS)
				idx = 0;
			sprintf(title, "Select Parameters for %s", com_struct[idx].portname);
			rs232_fm->title = (char *) title;
			set_all_flags(idx);
			update_setup_form(idx);
			DrawForm ( rs232_fm );
			break;
		
		case BYEBYE    :
			break;
			
		case SAVE  :
			// update the config file with new settings
			update_com_record(idx);
			break;
		}         /* end of switch */

	update_edit_field(idx);
	} while ( (rs232_retv != OKAY)  && (rs232_retv != BYEBYE)) ;

CloseForm ( rs232_fm );
DestroyFRFForm(rs232_fm);

return SUCCESS;
}

/*------------------------------------------------------------------*/
// Initialize navSensor Ids
/*------------------------------------------------------------------*/
void initNavSensors(void)
{
int i;

memset((void *) navSensors, 0x00, sizeof(navSensors));

for (i=0; i< MAX_COMPORTS; i++)
	{
	navSensors[i].id = com_struct[i].portnum;
	}
}




/*------------------------------------------------------------------*/
// Pick the active nav sensor from a list of active sensors
/*------------------------------------------------------------------*/
int pickActiveSensor(ob_desc * ob, char * title)
{
fm_desc *lf;
int max_length = 0, i;
char *names [ MAX_COMPORTS ];
int cur_slot, top_slot, retv, count = 0;
static int last_slot = 0;                   // last recently used slot
char * err_msg =   "Unable to match nav sensor\n";

/* load the sensor names */
if ((count = loadSensorNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, "Yo", "No Sensors are Enabled." );
   return -1;
   }

/* create a list form */
lf = CreateListForm ( title, 7, 4, max_length+2, 8, 0 );

lf->ob_lst [  9 ].label  = "Okay";
lf->ob_lst [  9 ].flags |= IS_EXITABLE;
lf->ob_lst [  9 ].key    = 'O';
lf->ob_lst [  9 ].state  = 8;
   
lf->ob_lst [ 10 ].label  = "Default";
lf->ob_lst [ 10 ].flags |= IS_EXITABLE;
lf->ob_lst [ 10 ].key    = 'D';
lf->ob_lst [ 10 ].state  = 8;
   
lf->ob_lst [ 11 ].label  = "Cancel";
lf->ob_lst [ 11 ].flags |= IS_EXITABLE;
lf->ob_lst [ 11 ].key    = 'C';
lf->ob_lst [ 11 ].state  = 8;
   
lf->ob_lst [ 12 ].label  = "Help";
lf->ob_lst [ 12 ].flags |= IS_EXITABLE;
lf->ob_lst [ 12 ].key    = 'H';
lf->ob_lst [ 12 ].state  = 8;

cur_slot = top_slot = (last_slot < count && last_slot >= 0) ? last_slot : 0 ;

/* open a form to put the list on */
OpenForm( lf, 10 , ob->area.Ymax);

do {
retv = ProcessListForm(lf, names, count, &top_slot, &cur_slot );

switch ( retv ) 
	{
	case 9 :        // Okay
		last_slot = cur_slot;
		// read the comport
		i = atoi(&names[cur_slot][3]);

		curNav = queryNavPtr(i - 1);
		if (curNav == (t_nav *) NULL)
			{
			NotifyUser(-1, -1, "Error", err_msg);
			Puke(err_msg);
			}
		curCom = &com_struct[i-1];              // global ptr to current nav port
		break;
   
	case 10:        // Default
		break;

	case 11 :       // Cancel
		break;
   
	case 12 :       // Help
		break; 
	}
} while ( (retv != NOT_SET) && ( retv != 11 ) && (retv != 9) );

// clean up alloced memory
i = count;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
CloseForm   ( lf );
DestroyForm ( lf );

if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}



/*----------------------------------------------------------------------------*/
// Load the sensor names
/*----------------------------------------------------------------------------*/
int loadSensorNames(char * names[], int * max_length)
{
int i = 0;
int count = 0;
char buf[80];

for (i=0; i<MAX_COMPORTS; i++)
	{
	if ( (com_struct[i].enabled) &&
      ((com_struct[i].sensor_index == ASHTECH) ||
       (com_struct[i].sensor_index == DDMU540) ||
       (com_struct[i].sensor_index == NS7000)  ||
       (com_struct[i].sensor_index == TRIMBLE_NAVTRAK)  ||
       (com_struct[i].sensor_index == MAGNAVOX_1107)  ||
       (com_struct[i].sensor_index == TRIMBLE_TANS)  ||
       (com_struct[i].sensor_index == TRIMBLE_4000)) )
		{
		sprintf(buf, "%-5s %s", com_struct[i].portname, com_struct[i].sensor_name);
		names [ count ] = ( char * ) malloc ( strlen ( buf ) + 1 );
		if ( names [ count ] == (char *) NULL ) 
			{
			i = count;
			while ( i ) 
				free ( names [ --i ] );
			NotifyUser(-1, -1, "ERROR", "Not Enough Memory." );
			return -1;
			}
		else 
			{
			strcpy ( names [ count++ ], buf );
			*max_length = max ( *max_length, (int) strlen ( buf ) );
			}
		}  
	}
return count;
}


/*---------------------------------------------------------------------------*/
// Select the Active Sensor
/*---------------------------------------------------------------------------*/
int doActiveSensor(ob_desc * ob)
{
return pickActiveSensor(ob, "Select Active Sensor");
}



/*----------------------------------------------------------------------------*/
// Get Gyro Type
/*----------------------------------------------------------------------------*/
int doGyroSelection ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "gyropick.frf";

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// set up Type of Gyro
fm->ob_lst [ 2 ].flags &= ~IS_CHECKED;
fm->ob_lst [ 3 ].flags &= ~IS_CHECKED;
fm->ob_lst [ 4 ].flags &= ~IS_CHECKED;
fm->ob_lst [ 5 ].flags &= ~IS_CHECKED;

switch (QuerySetup()->Gyro)
	{
	case MAGNAVOX_1107:
		fm->ob_lst [ 2 ].flags |=  IS_CHECKED;
		break;

	case KVH_GYRO:
		fm->ob_lst [ 3 ].flags |=  IS_CHECKED;
		break;

	case BUSTER_GYRO:
		fm->ob_lst [ 4 ].flags |=  IS_CHECKED;
		break;

	case NO_GYRO:
		fm->ob_lst [ 5 ].flags |=  IS_CHECKED;
		break;
	}

OpenForm ( fm, ob->area.Xmin, ob->area.Ymax ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the flags
			switch (get_flag_fm_group(fm, 1))
				{
				case 0:     // MAGNAVOX_1107
					QuerySetup()->Gyro = MAGNAVOX_1107;
					break;
			
				case 1:    // KVH
					QuerySetup()->Gyro = KVH_GYRO;
					break;
			
				case 2:
					QuerySetup()->Gyro = BUSTER_GYRO;
					break;

				case 3:
					QuerySetup()->Gyro = NO_GYRO;
					break;
				}

			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


