/*----------------------------------------------------------------------------*/
// File: video.c   USGS Navigation, allow user to view, capture real-time video
// Blame: John Gann
// Start Date: 28 April 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>
#include <lw.h>                           /* Header file for LiveWindows. */

#include "navdefs.h"
#include "navtypes.h"
#include "video.h"
#include "nav_util.h"
#include "mnutils.h"
#include "sites.h"
#include "nav.h"
#include "filehndl.h"

static int videoID   = 0;   
int videoWn   = NOT_SET;   
static int videoX   = 10;   
static int videoY   = 32;   
static int videoWnWidth   = 100;   
static int videoWnHeight   = 100;   
static rect videoRect;
static rect videoHotSpot;
static char videoInitialized = FALSE;
static char * WARNING_MSG   = "Yo";

// externals
extern int lwtuner(int channel, int finetune, int opts);
extern char RefreshRequired;
extern int graphWnWidth;
extern int graphWnHeight;
extern rect graphWnRect;
extern int graphWn;

static int videoActive = FALSE;
static t_video Video;

/*----------------------------------------------------------------------------*/
// Open the ideo board into a menuet window
/*----------------------------------------------------------------------------*/
int openLiveWindows(int x1, int y1, int x2, int y2)
{
int ercode;
int skew, xalign, yalign;
int scale = SCALE_TO_FIT;
int keyMode = KEY_XYCOLOR;
int lastWn = QryActiveWn();
int slot = -1;
char buf[128];

// Turn off the live windows error trapping function
lwsystem(1, 0);

// open the live windows board
if (!videoInitialized)
	{
	ercode = lwinit(NEWMEDIA, NM_PORTADDR, NM_MEMADDR, ICON_NOSHOW);
	if (ercode<=0)
		{
		sprintf(buf, "LiveWindows init error %d\n", ercode);
		NotifyUser(-1, -1, WARNING_MSG, buf);
		return NO_LIVE_WINDOWS_BOARD;
		}
	else
		{
		videoInitialized = TRUE;
		//  Perform some type of alignment 
		skew = 2;
		lwskew(skew);               /* Default skew, adjust for your VGA card.*/
		xalign = 45;
		yalign = 32;                /* Default alignment values. */
		lwalign(xalign, yalign);    /* Adjust for your VGA card. */
		setVideoControls();
		}
	}

// Make sure we are working in absolute coordinates
SetActiveWn(0, 0);

//  Open a video window 
slot = lwopen(Video.source, x1, y1, x2, y2, scale, keyMode, EXPLODE, videoID);
if (slot < 0)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Failed to open Live Windows Board");
	SetActiveWn(lastWn, 0);
	return NO_LIVE_WINDOWS_BOARD;
	}
else
	{
	videoID = slot;
	videoActive = TRUE;
	}

SetActiveWn(lastWn, 0);
return SUCCESS;
}

/*----------------------------------------------------------------------------*/
// terminate the video board
/*----------------------------------------------------------------------------*/
void termVideo(void)
{
if (!videoActive)
	return;

if (videoInitialized)
	{
	lwsystem(1, 0);
	lwclose(1, HORIZ_CURTAIN);         /* Close it.      */
	lwexit(EXIT_VIDEO_OFF);	      /* Turn off Video */
	videoActive = FALSE;
	videoInitialized = FALSE;
	}
CloseWindow(videoWn, 0);
DestroyWindow(videoWn);
videoWn = NOT_SET;
}



/*----------------------------------------------------------------------------*/
// Create a Window for the video
/*----------------------------------------------------------------------------*/
int doVideo(ob_desc * ob)
{
int retv;
t_setup * config = QuerySetup();

if (config->misc.allowVideo)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Live Windows Not Enabled");
	return NOT_SET;
	}

retv = initLiveWindows();

if (retv == SUCCESS)
	doVideoControls(ob);

return retv;
}

/*----------------------------------------------------------------------------*/
// refresh the video window when it has been obliterated by a scale change
/*----------------------------------------------------------------------------*/
void refreshVideo(void)
{
int lastWn = QryActiveWn();
attr_blk * at;

if (!videoActive)
	return;

CopyAttrBlk(attrWnDef, attrUsr1);
at = GetAttrBlk(attrUsr1);
at->fg_clr = 15;
//	at->bg_clr = 0;
	at->bg_clr = 2;
SetWnRecAttr(videoWn, WORK_AREA, attrUsr1 );

CloseWindow(videoWn, 0);

if (!OpenWindow(videoWn, videoX,  videoY))
	{
	printf("Unable to Open the Video Window\n");
	getch();
	return;
	}
else
	{
	DupRect ( QryWnRect ( videoWn, WN_PHYS_RECT ), &videoRect );
	SetActiveWn(0, 0);
	}

SetActiveWn(lastWn,0);
}



/*----------------------------------------------------------------------------*/
// update the vide coordinates after a move or resize
/*----------------------------------------------------------------------------*/
void updateVideoCoords(void)
{
mn_win_rec * wnRec = QryWnRec(videoWn);
int retv = SUCCESS;

videoX = wnRec->wn_xorg;
videoY = wnRec->wn_yorg;
videoWnWidth = wnRec->wn_curw;
videoWnHeight = wnRec->wn_curh;
SetRect(&videoHotSpot, videoX + videoWnWidth-5, videoY + videoWnHeight-5, 
                       videoX + videoWnWidth,   videoY + videoWnHeight);
// reopen the video
if (videoInitialized)
	{
	lwsystem(1, 0);
	lwclose(1, HORIZ_CURTAIN);         /* Close it.      */
	lwexit(EXIT_VIDEO_OFF);	      /* Turn off Video */
	videoActive = FALSE;
	videoInitialized = FALSE;
	}
retv = openLiveWindows(videoX, 
                       videoY,
                       videoX+videoWnWidth,
                       videoY+videoWnHeight);

if (retv != SUCCESS)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Can't relocate live windows");
	termVideo();
	}
}



/*----------------------------------------------------------------------------*/
// See if passed point was inside the video hot spot
/*----------------------------------------------------------------------------*/
int ptInVideoHotSpot(point * pt)
{
return PtInRect(pt, &videoHotSpot);
}





/*----------------------------------------------------------------------------*/
// Adjust video controls
/*----------------------------------------------------------------------------*/
int doVideoControls( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "vid_ctrl.frf";
ktag * volumeTag;
ktag * brightTag;
ktag * contrastTag;
ktag * colorTag;
ktag * hueTag;
ltag * channelTag;

/* Load the FRF file */
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

fm->ob_lst[1].flags |= IS_EXECUTABLE;
fm->ob_lst[1].task   = knobTask;
fm->ob_lst[2].flags |= IS_EXECUTABLE;
fm->ob_lst[2].task   = knobTask;
fm->ob_lst[3].flags |= IS_EXECUTABLE;
fm->ob_lst[3].task   = knobTask;
fm->ob_lst[4].flags |= IS_EXECUTABLE;
fm->ob_lst[4].task   = knobTask;
fm->ob_lst[5].flags |= IS_EXECUTABLE;
fm->ob_lst[5].task   = knobTask;

volumeTag   = (ktag *) fm->ob_lst[1].tag;
brightTag   = (ktag *) fm->ob_lst[2].tag;
contrastTag = (ktag *) fm->ob_lst[3].tag;
colorTag    = (ktag *) fm->ob_lst[4].tag;
hueTag      = (ktag *) fm->ob_lst[5].tag;
channelTag  = (ltag *) fm->ob_lst[10].tag;

// set up the form
set_flag_fm_group(fm, 3, Video.source-1);
channelTag->curitem = Video.channel-2;

OpenForm ( fm, ob->area.Xmin-40, ob->area.Ymin ); 
PositionKnob(&fm->ob_lst[1], Video.volume);
PositionKnob(&fm->ob_lst[2], Video.brightness);
PositionKnob(&fm->ob_lst[3], Video.contrast);
PositionKnob(&fm->ob_lst[4], Video.color);
PositionKnob(&fm->ob_lst[5], Video.hue);

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the knob values between 0 and 100
			readKnob(volumeTag, &Video.volume);
			readKnob(brightTag, &Video.brightness);
			readKnob(contrastTag, &Video.contrast);
			readKnob(colorTag, &Video.color);
			readKnob(hueTag, &Video.hue);
			Video.source = 1 + get_flag_fm_group(fm, 3);
			setVideoControls();
			break;

		case STOP_VIDEO:
			termVideo();
			break;

		case SAVE_IMAGE:
			saveVideoImage();
			break;

		case 7:          // video source 1
		case 8:          // video source 2
		case 9:          // video source 3
			Video.source = retv - 6;
			lwsetvsource(Video.source);
			break;

		case 10:          // channel
			Video.channel = atoi(channelTag->list[channelTag->curitem]);
			lwtuner(Video.channel, 0, 0);
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL-1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}



/*----------------------------------------------------------------------------*/
// read the video file
/*----------------------------------------------------------------------------*/
int read_video(char * filename)
{
FILE *fp;
int k;

if ((fp = fopen(filename, "rb")) == NULL)
  {
  printf("\nCan't open file %s", filename);
  }

k = fread( (void *) &Video, sizeof(t_video), 1, fp);
if (k == -1)
	{
	fclose(fp);
	return FALSE;
	}

fclose(fp);
return TRUE;
}



/*----------------------------------------------------------------------------*/
// Save the video file 
/*----------------------------------------------------------------------------*/
int write_video(char * filename)
{
FILE *fp;
int k;

if ((fp = fopen(filename, "wb")) == NULL)
  {
  return FALSE;
  }

 k = fwrite( (char *) &Video, sizeof(t_video), 1, fp);
 if (k == -1)
   {
   fclose(fp);
   printf("\nfwrite failed");
   return FALSE;
   }

fclose(fp);
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Set the controls on the video board
/*----------------------------------------------------------------------------*/
void setVideoControls(void)
{
lwsetaudio(VOLUME_LEFT,  Video.volume);
lwsetaudio(VOLUME_RIGHT, Video.volume);
lwsetvideo(BRIGHTNESS, Video.brightness);
lwsetvideo(CONTRAST, Video.contrast);
lwsetvideo(REDGAIN,   Video.color);
lwsetvideo(GREENGAIN, Video.color);
lwsetvideo(BLUEGAIN,  Video.color);
lwsetvideo(HUE,       Video.hue);
lwsetvsource(Video.source);
lwtuner(Video.channel, 0, 0);
}


/*----------------------------------------------------------------------------*/
// read the knob and compute the curent percent
/*----------------------------------------------------------------------------*/
int readKnob(ktag * knob, int * val)
{
return * val = IScaled(knob->curR - knob->minR, 100, knob->maxR - knob->minR);
}


/*----------------------------------------------------------------------------*/
// Knob Task
// extract the hook value and update the video board and return
/*----------------------------------------------------------------------------*/
int knobTask(ob_desc * knob)
{
int control = (int) knob->hook;
int value;

// get the value from the form
value = AdjustKnob(knob);

// the live windows board interprets zero as the default value
value = (value == 0) ? 1: value;

switch(control)
	{
	case LW_VOLUME:
		lwsetaudio(VOLUME_LEFT,  value);
		lwsetaudio(VOLUME_RIGHT, value);
		Video.volume = value;
		break;

	case LW_BRIGHTNESS:
		lwsetvideo(BRIGHTNESS, value);
		Video.brightness = value;
		break;

	case LW_CONTRAST:
		lwsetvideo(CONTRAST, value);
		Video.contrast = value;
		break;

	case LW_COLOR:
		lwsetvideo(REDGAIN,   value);
		lwsetvideo(GREENGAIN, value);
		lwsetvideo(BLUEGAIN,  value);
		Video.color = value;
		break;

	case LW_HUE:
		lwsetvideo(HUE,       value);
		Video.hue = value;
		break;
	}
return value;
}

/*----------------------------------------------------------------------------*/
// return TRUE if the video window is currently open
/*----------------------------------------------------------------------------*/
int isVideoActive(void)
{
return videoActive;
}


/*----------------------------------------------------------------------------*/
// display the passed video filename if it exists
/*----------------------------------------------------------------------------*/
int displayVideoFile( t_videoHook * v)
{
int retv = SUCCESS;

if (v->fileName[0] == '\0')
	{
	return BAD_PATH_NAME;
	}

if (v->imageType != NATIVE)
	{
	return WRONG_IMAGE_TYPE;
	}

// compute the width & height of the window needed
videoX        = v->x1;
videoY        = v->y1;
videoWnWidth  = abs(v->x2 - v->x1);
videoWnHeight = abs(v->y2 - v->y1);
SetRect(&videoHotSpot, videoX + videoWnWidth-5, videoY + videoWnHeight-5, 
                       videoX + videoWnWidth,   videoY + videoWnHeight);
// reopen the video
if (videoInitialized)
	{
	lwsystem(1, 0);
	lwclose(1, HORIZ_CURTAIN);         /* Close it.      */
	lwexit(EXIT_VIDEO_OFF);	      /* Turn off Video */
	videoActive = FALSE;
	videoInitialized = FALSE;
	}
retv = openLiveWindows(videoX, 
                       videoY,
                       videoX+videoWnWidth,
                       videoY+videoWnHeight);

if (retv != SUCCESS)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Can't relocate live windows");
	termVideo();
	}

// set the key color for the video
lwsetcolor(QryBG(QryWnAttr(videoWn, WORK_AREA)));     // Set the key color
lwfreeze(FREEZE_ON);

retv = lwload(v->x1, v->y1, v->x2, v->y2, v->fileName, 0);

return retv;
}




/*----------------------------------------------------------------------------*/
// display the passed video filename Without knowledge of scale
/*----------------------------------------------------------------------------*/
int displayRawVideoFile( char * filename)
{
int retv = SUCCESS;

if (filename == (char *) NULL)
	{
	return BAD_PATH_NAME;
	}

if (videoInitialized)
	{
	lwsystem(1, 0);
	lwclose(1, HORIZ_CURTAIN);                // Close it. 
	lwexit(EXIT_VIDEO_OFF);	                  // Turn off Video
	videoActive = FALSE;
	videoInitialized = FALSE;
	}

// reopen the video
retv = openLiveWindows(videoX, 
                       videoY,
                       videoX+videoWnWidth,
                       videoY+videoWnHeight);

if (retv != SUCCESS)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Can't relocate live windows");
	termVideo();
	}

// set the key color for the video
lwsetcolor(QryBG(QryWnAttr(videoWn, WORK_AREA)));     // Set the key color
lwfreeze(FREEZE_ON);

retv = lwload(videoX, videoY, 0, 0, filename, 0);

return retv;
}








/*----------------------------------------------------------------------------*/
// Save the video image to the current site file hook
/*----------------------------------------------------------------------------*/
int saveVideoImage(void)
{
int retv;
char filename[_MAX_PATH];

// get the target filename
if ((getVideoFileName(filename)) == (char *) NULL)
	{
	NotifyUser(-1, -1, "yo", "Unable to get video filename");
	return  BAD_PATH_NAME;
	}

// freeze the display
lwfreeze(FREEZE_ON);

// Save the video image as a plain file
retv = lwsave(0, 
	         videoX, 
           videoY,
           videoX+videoWnWidth, 
           videoY+videoWnHeight,
           filename, 
           0);
/********
retv = lwsavetiff(0, 
	         videoX, 
           videoY,
           videoX+videoWnWidth, 
           videoY+videoWnHeight,
           filename, 
           0);
*********/

// store the filename in a hook in the current site
setVideoHook(filename);

// Unfreeze the display
lwfreeze(FREEZE_OFF);
}



/*----------------------------------------------------------------------------*/
// return a pointer to name of file where a video image should be stored
/*----------------------------------------------------------------------------*/
char * getVideoFileName(char * buf)
{
char tmpname[_MAX_PATH] = {"vdXXXXXX"};

// Create a temp file name to store the output to.
if (mktemp(tmpname) == NULL)
	{
	return (char *) NULL;
	}
else
	{
	strcpy(buf, tmpname);
	}
return buf;
}


/*----------------------------------------------------------------------------*/
// set the name of the video file in a site structure
/*----------------------------------------------------------------------------*/
int setVideoHook(char * buf)
{
t_rngBrg * rB = querySiteInfo();
t_site * site;
t_videoHook * v;

if (rB == (t_rngBrg *) NULL)
	{
	return NULL_SITE ;
	}

v = (t_videoHook *) &rB->site.vHook;

v->x1 = videoX;
v->y1 = videoY;
v->x2 = videoX+videoWnWidth;
v->y2 = videoY+videoWnHeight;
strcpy(v->fileName, FileNameP(buf));
v->imageType = NATIVE;

// Find the site that matches the current site
site = queryMatchingSite(&rB->site);
if (site == (t_site *) NULL)
	{
	NotifyUser (-1, -1, WARNING_MSG, "Unable to match current Site\n");
	return NO_SITE_MATCH;
	}
else
	{
	memcpy((void *) &site->vHook,
         (void *) v,
          sizeof(t_videoHook));
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// dump a video hook struct 
/*----------------------------------------------------------------------------*/
char * dumpVideoHook(t_videoHook * v, char * buf)
{
int j = 0;

if (v == (t_videoHook *) NULL)
	return (char *) NULL;

j += sprintf(buf+j, " fileName %s\n", v->fileName);
j += sprintf(buf+j, "       x1 %3d\n", v->x1);
j += sprintf(buf+j, "       y1 %3d\n", v->y1);
j += sprintf(buf+j, "       x2 %3d\n", v->x2);
j += sprintf(buf+j, "       y2 %3d\n", v->y2);
j += sprintf(buf+j, "imageType %3d\n", v->imageType);

return buf;
}

/*----------------------------------------------------------------------------*/
// thaw the video
/*----------------------------------------------------------------------------*/
void thawVideo(void)
{
lwfreeze(FREEZE_OFF);
}


/*----------------------------------------------------------------------------*/
// Create and open a video window if one has not been opned yet
/*----------------------------------------------------------------------------*/
int initLiveWindows(void)
{
int retv=SUCCESS;
attr_blk * at;
t_setup * config = QuerySetup();

// See if the video window has already been created
if (videoWn == NOT_SET)
	{
	// read in the video file 
	if (!read_video(VIDEO_FILE_NAME))
		{
		NotifyUser(-1, -1, VIDEO_FILE_NAME, "Error reading video file");
		}

	// Create Video area
	if ((videoWn = CreateWindow(
                  NULL, 
                  NULL, 
                  NULL, 
                  videoWnWidth, 
                  videoWnHeight
                  )) == -1)
		{
		printf("Unable to Create the Video Window\n");
		getch();
		return FALSE;
		}

	videoX = graphWnRect.Xmin;
	videoY = graphWnRect.Ymax - videoWnHeight;

	CopyAttrBlk(attrWnDef, attrUsr1);
	at = GetAttrBlk(attrUsr1);
	at->fg_clr = 15;
//	at->bg_clr = 0;
	at->bg_clr = 2;
	SetWnRecAttr(videoWn, WORK_AREA, attrUsr1 );

	if (!OpenWindow(videoWn, videoX,  videoY))
		{
		printf("Unable to Open the Video Window\n");
		getch();
		return FALSE;
		}
	else
		{
		SetRect(&videoHotSpot, videoX + videoWnWidth-8, videoY + videoWnHeight-8, 
                           videoX + videoWnWidth,   videoY + videoWnHeight);
		DupRect ( QryWnRect ( videoWn, WN_PHYS_RECT ), &videoRect );
		SetActiveWn(0, 0);
		}

	// set the key color for the video
	lwsetcolor(QryBG(QryWnAttr(videoWn, WORK_AREA)));     // Set the key color

	// load the video
	retv = openLiveWindows(videoX, 
                       videoY,
                       videoX+videoWnWidth, 
                       videoY+videoWnHeight);

	if (retv != SUCCESS)
		{
		NotifyUser(-1, -1, WARNING_MSG, "Can't Open Live Windows");
		return -1;
		}
	}
return retv;
}



/*----------------------------------------------------------------------------*/
// Menu pick for viewing a video file
/*----------------------------------------------------------------------------*/
int viewVideoFile(ob_desc *callingOb)
{
char filename[_MAX_PATH];
int retv;
t_setup * config = QuerySetup();

if (config->misc.allowVideo)
	{
	NotifyUser(-1, -1, WARNING_MSG, "Live Windows Not Enabled");
	return NOT_SET;
	}

strcpy(filename, CurrentDir());
strcat(filename, "\\vd*.*");

if ( get_input_file(filename, "Select Video Filename", filename) )
	{
	retv = displayRawVideoFile(filename);
	}
return retv; 
}


